// -*-c++-*-

/*!
  \file sample_communication.h
  \brief sample communication planner Header File
*/

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifndef SAMPLE_COMMUNICATION_H
#define SAMPLE_COMMUNICATION_H

#include "communication.h"

#include <rcsc/game_time.h>
#include <rcsc/types.h>
#include <rcsc/player/world_model.h>

// For set-play communication
#include <setplay/runtime/setplay_communication.h>

namespace rcsc {
    class WorldModel;
}

class FCPortugalPlayer;

class SampleCommunication
    : public Communication
        // ,public fcportugal::setplay::IMessageSender
{
private:

    FCPortugalPlayer* M_agent;
    int M_current_sender_unum;
    int M_next_sender_unum;
    rcsc::GameTime M_ball_send_time;
    rcsc::GameTime M_teammate_send_time[12];
    rcsc::GameTime M_opponent_send_time[12];

public:

    SampleCommunication(FCPortugalPlayer* agent);
    ~SampleCommunication();

    bool execute(  );

    int currentSenderUnum() const { return M_current_sender_unum; }

    int nextSenderUnum() const { return M_next_sender_unum; }

    void setSetplayCommunication( setplay::SetplayCommunication* setplay_communication) {
        M_setplay_communication = setplay_communication;
    }

private:
    int availableMessageLength() const;

    void updateCurrentSender( );

    void updatePlayerSendTime( const rcsc::WorldModel & wm,
                               const rcsc::SideID side,
                               const int unum );

    bool shouldSayBall(  );
    bool shouldSayOpponentGoalie();
    bool goalieSaySituation( );

    bool saySetplayCommunication ( );

    bool sayBallAndPlayers();
    bool sayBall();
    bool sayPlayers( );
    bool sayGoalie( );

    bool sayIntercept( );
    bool sayOffsideLine( );
    bool sayDefenseLine( );

    bool sayOpponents( );
    bool sayTwoOpponents( );
    bool sayThreeOpponents( );

    bool saySelf( );

    bool sayStamina( );
    bool sayRecovery( );

    void attentiontoSomeone( );
    bool decideSetplayAttentionTo();

    // Setplay Communication
    /*void send(const fcportugal::setplay::InitMessage& message) const;
    void send(const fcportugal::setplay::StepMessage& message) const;
    void send(const fcportugal::setplay::ActionDoneMessage& message) const;
    void send(const fcportugal::setplay::FinishMessage& message) const;
    void send(const fcportugal::setplay::AbortMessage& message) const;*/

    setplay::SetplayCommunication* M_setplay_communication;
};

#endif
