#include "flux_manager.h"

#include <limits>

#include <rcsc/common/server_param.h>
#include <rcsc/common/logger.h>

#include "strategy_2d.h"


#include <fcportugal_player.h>

#include <strategy/tactic/tactic.h>
#include <strategy/tactic_config.h>

using namespace std;
using namespace rcsc;

using namespace soccer;
using namespace soccer::flux;
using namespace soccer::strategy;
using namespace soccer::tactic;

namespace fcportugal
{

    FluxManager::FluxManager( )
    {
        M_previous_action_target_position = rcsc::Vector2D(2.0,0.0);
        M_previous_evaluation = -numeric_limits<double>::min();
        M_action_target_position = M_previous_action_target_position;
        M_action_evaluation = M_previous_evaluation;
        M_action_origin = 0.0;
        M_action_previous_origin=0.0;
    }

    FluxManager& FluxManager::instance()
    {
        static FluxManager instance;

        return instance;
    }

    void FluxManager::setWorld(const rcsc::WorldModel* world)
    {
        M_world = world;
    }

    double FluxManager::evaluate(const rcsc::Vector2D &start_position,
                                 const rcsc::Vector2D &end_position)
    {
        double flux_value = evaluateSituation(start_position, end_position);
        double safety_value = evaluateSafety(start_position, end_position);
        double easiness_value = evaluateEasy(start_position, end_position);

        if ( !TacticConfig::instance().useTactics() )
        {
            /* const Tactic& current_tactic = fcportugal::strategy::Strategy2D::i().currentTactic();
            return current_tactic.weightFlux() * flux_value
                 + current_tactic.weightSafety() * safety_value
                 + current_tactic.weightEasy() * easiness_value; */
        }

        return   0.6 * flux_value
               + 0.3 * safety_value
               + 0.2 * easiness_value;
    }

    //Safety of end_position compared with ball_position
    double FluxManager::evaluateSafety(const rcsc::Vector2D& start_position,
                                       const rcsc::Vector2D& end_position)
    {
        //logDef;
        // double start_safety_value = calculateSafety(start_position);
        // Should consider intermediate points between according to planning positions
        double end_safety_value = calculateSafety(start_position, end_position);
        //float SafetyValue = SafetyValueFinal - SafetyValueBall;
        // double safety_diff = end_safety_value - start_safety_value;

        return end_safety_value;
    }


    double FluxManager::calculateSafety(const Vector2D& start_position,
                                        const Vector2D& end_position)
    {
        Vector2D self_position = M_world->self().pos();
        double dist_to_end_pos = start_position.dist( end_position );
        int time_to_end_pos = M_world->self().playerType().cyclesToReachDistance( dist_to_end_pos );
        double y_offset = time_to_end_pos * ServerParam::i().defaultPlayerSpeedMax();
        rcsc::Ray2D safety_ray(start_position, end_position);
        rcsc::Triangle2D safety_triangle(start_position,
                                         Vector2D(end_position.x, end_position.y + y_offset),
                                         Vector2D(end_position.x, end_position.y - y_offset));

        // How many players are within the triangle? At which distance from the safety line
        Vector2D best_opp_interception_pos = Vector2D::INVALIDATED;
        int best_opp_interception_time = time_to_end_pos;

        // Opponents are sorted by distance from self
        for ( PlayerPtrCont::const_iterator it = M_world->opponentsFromSelf().begin();
              it != M_world->opponentsFromSelf().end();
              it++ )
        {
            const PlayerObject* opp = *it;
            if ( safety_triangle.contains( opp->pos() ) )
            {
                Vector2D opp_interception_pos = safety_ray.nearestPoint( opp->pos() );
                if ( opp_interception_pos == Vector2D::INVALIDATED )
                {
                    continue;
                }
                double opp_dist_to_intercept_pos = opp->pos().dist( opp_interception_pos );
                int opp_interception_time = opp->playerTypePtr()->cyclesToReachDistance( opp_dist_to_intercept_pos );
                double self_dist_to_intercept_pos = self_position.dist( opp_interception_pos );
                int time_to_intercept_pos = M_world->self().playerTypePtr()->cyclesToReachDistance( self_dist_to_intercept_pos );
                if ( time_to_intercept_pos >= opp_interception_time )
                {   // Opponent will get there faster than me
                    best_opp_interception_pos = opp_interception_pos;
                    best_opp_interception_time = opp_interception_time;
                    break;
                }
            }
        }

        if ( best_opp_interception_time < time_to_end_pos )
        {
            double interception_safety_value = (double) best_opp_interception_time / time_to_end_pos;
            return interception_safety_value;
        }

        return 1.0; // Totally safe
    }

    /*double FluxManager::calculateSafety(const Vector2D&,
                                        const Vector2D& end_position)
    {
        const ServerParam& SP = ServerParam::i();
        const Vector2D& ball_pos = M_world->ballPosition();

        if (fabs(end_position.x) > 0.98 * SP.pitchHalfLength() ||
            fabs(end_position.y) > 0.98 * SP.pitchHalfWidth())
        { //point outside field
            return 0.0; //speed up
        }

        double safety_value = 0.0, line_safety_value = 5.0, point_safety_value = 5.0;

        int closest_opp_unum = 0, closest_teammate_unum = 0;
        double opp_closest_time = 100.0, our_closest_time = 100.0;
        Ray2D shoot_ray(ball_pos, end_position - ball_pos);

        double ball_dist_to_end_pos = ball_pos.dist(end_position);
        double player_dist_to_end_pos, player_dist_to_interc_pos, ball_dist_to_interc_pos;

        double player_time_to_end_pos , player_time_to_intercept_pos;
        Vector2D interception_pos = Vector2D::INVALIDATED;

        stringstream debug_our;
        stringstream debug_opp;
        for ( unsigned int unum = 1; unum <= (unsigned int) SP.maxPlayer(); unum++)
        {
            if ( unum == 1 )
            {
                continue;
            }

            rcsc::Vector2D player_pos = M_world->playerPosition( OUR, unum );
            if ( player_pos == Vector2D::INVALIDATED )
            {
                continue;
            }

            // disconsider own goalie on 2D... TODO consider negative... (own passes)
            // TODO Should check if player pointer is not NULL
            interception_pos = closestPoint( shoot_ray, our->pos() );
            player_dist_to_end_pos = our->pos().dist( end_position );
            player_dist_to_interc_pos= our->pos().dist( interception_pos );

            // Set a drift factor according to confindence on object position
            // TODO Abstract this in the world model notion when calculating distances
            if ( our->seenPosCount() > 1 )
            {
                double player_drift_factor = SP.defaultPlayerSpeedMax() / 0.5
                                             * our->seenPosCount();

                // Not further apart than 5m from where I last saw him? Why?
                player_drift_factor = player_drift_factor < 5.0 ? player_drift_factor : 5.0;
                player_dist_to_end_pos += player_drift_factor;
                player_dist_to_interc_pos += player_drift_factor;
            }

            ball_dist_to_interc_pos = ball_pos.dist(interception_pos);
            if (ball_dist_to_interc_pos > ball_dist_to_end_pos)
            {
                player_dist_to_interc_pos = player_dist_to_end_pos;
                interception_pos = end_position;
            }

            player_time_to_end_pos = player_dist_to_end_pos / SP.defaultPlayerSpeedMax();
            player_time_to_intercept_pos = player_dist_to_interc_pos / SP.defaultPlayerSpeedMax();

            if (player_time_to_end_pos  < our_closest_time )
            {
                our_closest_time  = player_time_to_end_pos , closest_teammate_unum  = unum;
            }

            const AbstractPlayerObject* opp = WM.theirPlayer(unum);
            if ( opp == NULL )
            {
                debug_opp << unum << " ";
            }
            else
            {
                interception_pos = closestPoint( shoot_ray, opp->pos() );
                player_dist_to_end_pos = opp->pos().dist( end_position );
                player_dist_to_interc_pos = opp->pos().dist( interception_pos );

                // TODO Use seen pos count for confidence M_world.parentWorld().theirPlayer(i)->seenPosCount()
                if ( opp->seenPosCount() > 1)
                {  //If confidence is not good player may be Closer
                        double player_drift_factor = SP.defaultPlayerSpeedMax() / 0.5 * opp->seenPosCount();

                        // Opp max drift factor == 8.0
                        if (player_drift_factor>8.0) player_drift_factor = 8.0;

                        // Assume that player is closer than we think
                        player_dist_to_end_pos -= player_drift_factor;
                        player_dist_to_interc_pos -= player_drift_factor;

                        // Truncate distance to non-negative values
                        player_dist_to_end_pos = player_dist_to_end_pos > 0.0 ? player_dist_to_end_pos : 0.0;
                        player_dist_to_interc_pos = player_dist_to_interc_pos > 0.0 ? player_dist_to_end_pos : 0.0;
                }

                ball_dist_to_interc_pos = ball_pos.dist(interception_pos);
                if (ball_dist_to_interc_pos > ball_dist_to_end_pos)
                {
                    player_dist_to_interc_pos = player_dist_to_end_pos;
                    interception_pos = end_position;
                }

                player_time_to_end_pos = player_dist_to_end_pos / SP.defaultPlayerSpeedMax();
                player_time_to_intercept_pos = player_dist_to_interc_pos / SP.defaultPlayerSpeedMax();

                if ( player_time_to_end_pos < opp_closest_time )
                {                        // Update info about the opponent that is nearest to end pos
                    opp_closest_time  = player_time_to_end_pos, closest_opp_unum = unum;
                }

                if (player_time_to_intercept_pos < 1.0) line_safety_value  = 0;
                else if (player_time_to_intercept_pos < 2.0 && ball_dist_to_interc_pos  > 8.0)
                    line_safety_value  = 0;
                else if (player_time_to_intercept_pos < 3.0 && ball_dist_to_interc_pos  > 12.0)
                    line_safety_value  = 0;
                else if (player_time_to_intercept_pos < 5.0 && ball_dist_to_interc_pos  > 20.0)
                    line_safety_value  = 0;
                else if (player_time_to_intercept_pos < 2.0 && ball_dist_to_interc_pos  > 5.0)
                    line_safety_value  -= 3;
                else if (player_time_to_intercept_pos < 3.0 && ball_dist_to_interc_pos  > 8.0)
                    line_safety_value  -= 3;
                else if (player_time_to_intercept_pos < 5.0 && ball_dist_to_interc_pos  > 15.0)
                    line_safety_value  -= 3;
                else if (player_time_to_intercept_pos < 8.0 && ball_dist_to_interc_pos  > 20.0)
                    line_safety_value  -= 3;
                else if (player_time_to_intercept_pos < 3.0 && ball_dist_to_interc_pos  > 6.0)
                    line_safety_value -= 2;
                else if (player_time_to_intercept_pos < 5.0 && ball_dist_to_interc_pos  > 12.0)
                    line_safety_value -= 2;
                else if (player_time_to_intercept_pos < 8.0 && ball_dist_to_interc_pos  > 15.0)
                    line_safety_value -= 2;
            }
        }


        // Calculate the ball point safety value
        if (our_closest_time  <= opp_closest_time - 8.0 ) point_safety_value  = 5.0;  //LPR 2010 HACK
        else if (our_closest_time  <= opp_closest_time - 4.0) point_safety_value  = 4.0;  //LPR 2010 HACK
        else if (our_closest_time  <= opp_closest_time - 2.0) point_safety_value  = 3.0;  //LPR 2010 HACK
        else if (our_closest_time  <= opp_closest_time ) point_safety_value  = 2.0;  //LPR 2010 HACK
        else if (our_closest_time  <= opp_closest_time + 2.0) point_safety_value  = 1.0;  //LPR 2010 HACK
        else point_safety_value  = 0.0;  //LPR 2010 HACK

        // Truncate safety values
        if (line_safety_value  < 0.0) line_safety_value  = 0.0;
        if (point_safety_value  < 0.0) point_safety_value  = 0.0;

        safety_value  = ( line_safety_value + point_safety_value ) / 2.0;



        dlog.addText( Logger::POSITIONING,
                     __FILE__ " Could not find Our(%s) Opp(%s)",
                     debug_our.str().c_str(), debug_opp.str().c_str());

        return safety_value ;
        }*/

    // HACK for ray geometry operation
    rcsc::Vector2D FluxManager::closestPoint(const rcsc::Ray2D& ray, const rcsc::Vector2D& point)
    {
        rcsc::Line2D line(ray.origin(), ray.dir());
        Vector2D closest_point = line.projection(point);
        return ray.inRightDir(closest_point)
                ? closest_point
                : ray.origin();
    }


    double FluxManager::evaluateSituation(const rcsc::Vector2D& start_position,
                                          const rcsc::Vector2D& end_position)
    {
        Vector2D truncated_start_position = ServerParam::i().pitchRect().nearestPoint( start_position );
        Vector2D truncated_end_position = ServerParam::i().pitchRect().nearestPoint( end_position );

        double dist_to_end_pos = truncated_start_position.dist(truncated_end_position);

        int self_time_to_end_pos = M_world->self().playerType().cyclesToReachDistance( dist_to_end_pos );

        Vector2D pos( truncated_start_position.x, truncated_start_position.y );
        Vector2D motion_step = Vector2D( truncated_end_position.x - truncated_start_position.x,
                                         truncated_end_position.y - truncated_start_position.y ).normalize()
                               * M_world->self().playerType().realSpeedMax();

        double avg_flux = 0.0;
        double start_flux_value = flux( truncated_start_position );
        double end_flux_value = flux( truncated_end_position );
        for ( int t = 0; t < self_time_to_end_pos - 2 ; t++ )
        {
            avg_flux = avg_flux + flux(pos + motion_step);
        }
        avg_flux = start_flux_value + avg_flux + end_flux_value;
        avg_flux = avg_flux / self_time_to_end_pos;

        double flux_value = 0.0;

        Vector2D interception_point = Vector2D::INVALIDATED;
        rcsc::Line2D start_to_end_pos_line(start_position, end_position);

        int is_goal = 0;
        double field_half_length = ServerParam::i().pitchHalfLength();
        double field_half_width = ServerParam::i().pitchHalfWidth();
        double goal_half_width = ServerParam::i().goalHalfWidth();

        if (end_position.x > 0.98 * field_half_length)
        { // Possible goal
            rcsc::Line2D opp_goal_line(Vector2D(field_half_length, -goal_half_width),
                                       Vector2D(field_half_length, goal_half_width));
            interception_point = start_to_end_pos_line.intersection(opp_goal_line);
            bool shootTheirGoal = end_position.x > field_half_length && //crosses their goalline
                                  end_position.x > interception_point.x && // shoot from anywhere and really inside goal
                                  (fabs(interception_point.y) < 0.6 * goal_half_width || // shoot in front and inside goal
                                    (fabs(interception_point.y) < 0.95 * goal_half_width &&
                                     fabs(start_position.y)< goal_half_width) ||
                                     (fabs(interception_point.y) < 0.95 * goal_half_width &&
                                  fabs(start_position.y)< goal_half_width)); // shoot in front and inside goal
            is_goal =  shootTheirGoal ? 3 : -2; // 3 - Goal, 2 - out
        }
        else if (end_position.x < -field_half_length)
        { // Possible own goal or ball
           rcsc::Line2D our_goal_line(Vector2D(-field_half_length,-goal_half_width),
                                      Vector2D(-field_half_length,goal_half_width));
           interception_point = start_to_end_pos_line.intersection(our_goal_line);
           bool shootOurGoal = end_position.x < -0.98*field_half_length && end_position.x < interception_point.x &&
                               fabs(interception_point.y) < 1.5 * goal_half_width; // crosses our goalline
           //not even close of our goal Line2D - DANGER - improve this...
           is_goal =  shootOurGoal ? 3 : -2; // 3 - Goal, 2 - out
        }
        else if (end_position.y > 0.95* field_half_width)
        { // Posible ball out
           rcsc::Line2D side_line(Vector2D(-field_half_length, 0.99 * field_half_width),
                                  Vector2D(field_half_length, 0.99*field_half_width));
           interception_point = start_to_end_pos_line.intersection(side_line);
           is_goal = -2; //out side line
        }
        else if ( end_position.y < -0.95 * field_half_width )  { // Posible ball out
           rcsc::Line2D side_line(Vector2D(-field_half_length,-0.99*field_half_width),
                                  Vector2D(field_half_length,-0.99*field_half_width));
           interception_point = start_to_end_pos_line.intersection(side_line);
           is_goal = -2; //out side line
        }

        if ( interception_point == Vector2D::INVALIDATED )
        {
            interception_point = ServerParam::i().pitchRect().nearestPoint( end_position );
            dlog.addText( Logger::TEAM,
                         __FILE__ " No valid interception found for ball_pos (%.2f, %.2f) end_pos(%.2f,%.2f) ",
                         start_position.x, start_position.y,
                         end_position.x, end_position.y);
        }

        switch ( is_goal )
        {
            case 3:  // shoot! you will score!
                flux_value = 2.0 + flux(interception_point) / 5.0;
                break;
            case -3: // no own goals!
                flux_value = -5.0 + flux(interception_point) / 5.0;
                break;
            case -2: // ball out of bounds
                flux_value = -3.0 + flux(interception_point) / 5.0;
                break;
            default:
                // Other situations
                flux_value = avg_flux / 5.0; // Why divide by 5.0? If this is done for all cases, then => no impact
                break;
        }

        return flux_value - start_flux_value; // Previously 2.0 was added? why? No impact
    }

    double FluxManager::fluxDirection(const rcsc::Vector2D& position,
                                          double distance,
                                          double start_angle,
                                          double end_angle,
                                          double angle_step)
    {
        assert ( (start_angle <= end_angle && angle_step > 0) ||
                 (start_angle >= end_angle && angle_step < 0) );

        double flux_value = 0.0,
               flux_direction = 0.0,
               flux_max = numeric_limits<double>::min();
        for(double angle = start_angle ; angle < end_angle; angle += angle_step)
        {
            rcsc::Vector2D end_position = position +
                                          rcsc::Vector2D(distance, 0.0).rotate(AngleDeg::normalize_angle(angle));
            flux_value = flux(end_position);
            if ( flux_value > flux_max )
            {
                flux_max = flux_value;
                flux_direction = angle;
            }
        }
        return flux_direction;
    }


    double FluxManager::evaluateEasy(const Vector2D& start_position,
                                     const Vector2D& end_position)
    {
        Vector2D ball_vector_to_end_pos(end_position.x - M_world->ball().pos().x,
                                        end_position.y - M_world->ball().pos().y);
        double ball_dist_to_end_pos = ball_vector_to_end_pos.r();
        ball_vector_to_end_pos.normalize();

        Vector2D ball_vector_to_start_pos(M_world->ball().pos().x - start_position.x,
                                          M_world->ball().pos().y - start_position.y);
        ball_vector_to_start_pos.normalize();

        // AngleDeg ball_angle_diff = ball_vector_to_end_pos.dir() - ball_vector_to_start_pos.dir();

        //player should kick to his front
        AngleDeg angle_diff = ball_vector_to_end_pos.dir() - M_world->self().body();

        //HACK LPR2010Jun - correct it... TODO
        double vel_diff = M_world->ball().vel().r() - ( ball_dist_to_end_pos / 10.0);

        double easiness_value = 1.0;
        // Update easiness value based on angle_diff
        if (angle_diff.degree() < 45) { easiness_value  = 4.0; }
        else if (angle_diff.degree() < 90) { easiness_value  = 3.0; }
        else if (angle_diff.degree() < 145) { easiness_value  = 2.0; }

        // Update easiness value based on ball distance to end position
        if ( ball_dist_to_end_pos >= 8.0 && ball_dist_to_end_pos < 15.0 )
        { //difficult short and long kicks
            easiness_value ++;
        }
        else if ( ball_dist_to_end_pos < 3.0 || ball_dist_to_end_pos > 30.0 )
        { //difficult short and long kicks
            easiness_value -=2;
        }
        else
        { //difficult short and long kicks
            easiness_value--;
        }

        // Update easiness value based on velocity diff
        if (vel_diff < 1.0) { easiness_value  +=2; }
        else if (vel_diff < 1.5) { easiness_value  +=1; }
        else if (vel_diff < 3.0) { easiness_value --; }
        else if (vel_diff < 4.0) { easiness_value -=2; }
        else { easiness_value -=3; }

        return easiness_value;
    }


    void FluxManager::update( )
    {
        // Only update the flux if the ball is kickable by the player
        if ( !M_world->lastKickerSide() == M_world->ourSide() &&
             M_world->self().isKickable() )
        {
            return;
        }

        dlog.addText( Logger::POSITIONING, __FILE__ " FluxManager: Updating flux ");

        Vector2D start_position = M_world->ball().pos();
        AngleDeg self_body_angle = M_world->self().body();
        double max_action_evaluation = std::numeric_limits<double>::min();
        Vector2D selected_position = M_next_position;

        AngleDeg start_angle = self_body_angle;  //LPR2010May
        double dist_step =  50 / 10.0; // M_league_config.playerKickDistanceMax() / 10.0
        double angle_step = 10.0;

        // Iterate hypothesis for angles
        for ( AngleDeg angle = start_angle;
              angle.degree() < 360.0 + start_angle.degree();
              ( (angle - self_body_angle).degree() < 50.1) ? angle += AngleDeg(5.0) : angle_step += angle_step )
        {
            Vector2D orientation = Vector2D(1.0, 0.0).rotate(angle);
            for( double dist = dist_step;
                dist < 50; // M_league_config.playerKickDistanceMax();
                dist += dist_step )
            {
                Vector2D end_position = start_position + orientation * dist;
                double action_evaluation = evaluate(start_position, end_position);
                if ( action_evaluation > max_action_evaluation)
                {
                    max_action_evaluation = action_evaluation;
                    selected_position  = end_position;
                }
            }
        }

        //Vector2D intOur, intTheir, intOut;
        M_next_position = selected_position;
       // int out = updateOutcome( selected_position );

        M_action_evaluation  = max_action_evaluation; //LPR2010Jun
        M_flux_direction = fluxDirection(selected_position, 0, 0, 0, 0);// M_league_config.playerSpeedMax() / 20 );

        /*dlog.addText( Logger::POSITIONING,
                     __FILE__ " FluxManager: CurPos(%.2f,%.2f), FluxPos(%.2f,%.2f), Eval(%.2f)",
                     M_world.self().pos().x, M_world.self().pos().y,
                     M_next_position.x, M_next_position.y,
                     M_action_evaluation);*/
    }

    /*int FluxManager::updateOutcome(const rcsc::Vector2D& position)
    {
        // M_dmu_action_out = LineIsGoalOrOut(selected_position , intOur, intTheir, intOut);
    }*/

    void FluxManager::fluxChanged( )
    {
        dlog.addText( Logger::DRIBBLE, __FILE__ " Flux changed");
    }
}
