#ifndef FCP_FLUX_MANAGER_H
#define FCP_FLUX_MANAGER_H

#include <rcsc/geom/vector_2d.h>
#include <rcsc/geom/ray_2d.h>
#include <strategy/flux/flux_manager.h>
#include <strategy/singleton.h>
#include <rcsc/player/world_model.h>

class FCPortugalPlayer;

namespace fcportugal
{
    class FluxManager
            : public soccer::flux::FluxManager
    {
    public:
        FluxManager();

        static FluxManager& instance();

        void setWorld(const rcsc::WorldModel* world);
        void update( );
    protected:

        /*!
          \brief Evaluate the usefulness of going from start_position to an end position
          \returns Returns a double precision percentage directly proportional to the usefulness
          */
        double evaluate(const rcsc::Vector2D& start_position,
                        const rcsc::Vector2D& end_position);

        double evaluateSituation(const rcsc::Vector2D& start_position,
                                 const rcsc::Vector2D& end_position);

        /*!
          \brief Calculate the safety of going from a start position to an end position
          \returns Returns a double precision percentage that directly proportional to the safety
          */
        virtual double calculateSafety(const rcsc::Vector2D& start_position,
                                       const rcsc::Vector2D& end_position);

        /*!
          \brief Evaluate the easiness to make a motion from a start position to an end position
          \returns Returns a double precision percentage that is directly proportional to the easiness of the motion
          */
        virtual double evaluateEasy(const rcsc::Vector2D& start_position,
                                    const rcsc::Vector2D& end_position);

        /*!
          \brief Evaluate the safety of going from a start position to an end position
          \returns Returns a double precision percentage that directly proportional to the safety
          */
        virtual double evaluateSafety(const rcsc::Vector2D& start_position,
                                      const rcsc::Vector2D& end_position);

        double fluxDirection(const rcsc::Vector2D& position,
                             double distance,
                             double start_angle,
                             double end_angle,
                             double angle_step);


        void fluxChanged( );

        /*!
          \brief Calculate the outcome of moving to a given position
          */
        //int updateOutcome(const rcsc::Vector2D& position);
    private:
        rcsc::Vector2D closestPoint(const rcsc::Ray2D& ray, const rcsc::Vector2D& point);

        const rcsc::WorldModel* M_world;
    };

}

#endif // FLUX_MANAGER_H
