/* -*- Mode: C++ -*- */

/* MemPlanning.C
 * F.C.Portugal
 * Luis Paulo Reis - 2003
*/

/* Flexible Planning for FC Portugal! */

#include "planning.h"

#include <limits>

#include <fcportugal_player.h>

#include <sstream>
#include <stdlib.h>
#include <vector>

#include <rcsc/common/logger.h>

#include "chain_action/action_generator.h"

#include <rcsc/types.h>
#include <rcsc/player/intercept_table.h>
#include <rcsc/player/player_object.h>
#include <rcsc/common/server_param.h>

#include <rcsc/action/bhv_body_neck_to_point.h>
#include <rcsc/action/bhv_body_neck_to_ball.h>
#include <rcsc/action/bhv_scan_field.h>

#include <rcsc/action/neck_turn_to_player_or_scan.h>
#include <rcsc/action/neck_turn_to_ball_or_scan.h>

#include <fcportugal/strategy_2d.h>

#include <setplay/model/setplay.h>
#include <setplay/model/geom.h>
#include <setplay/model/conditions.h>
#include <setplay/model/actions.h>
#include <setplay/model/object.h>
#include <setplay/communication/messages.h>

#include <strategy/tactic/tactic.h>
#include <strategy/setplay_config.h>

#define SETPLAY_FILE "setplay.conf"

#define PLAYOFF_WAITTIME 100;

using namespace std;
using namespace rcsc;

using namespace fcportugal;
using namespace fcportugal::strategy;
using namespace setplay;
using namespace soccer;
using namespace soccer::strategy;

namespace fcportugal
{

    Planner::Planner(FCPortugalPlayer* agent)
        : M_agent(agent) // HACK For selecting setplays to be used
    {
        // Loading of setplays should be done externally deferred to strategy

    }

    Planner::~Planner()
    {
        // Reclaim memory
    }

    bool Planner::init( )
    {
      stringstream cbr_filename;
      cbr_filename << soccer::strategy::SetplayConfig::i().cbrFilePrefix()
                   << M_agent->world().self().unum() << ".conf";

      //Reading of setplays and setup of setplay_manager // Depends on usage of CBR...
      /*if ( agent->strategy().setplaysConfig().useCbr() )
      {
        M_setplay_manager = new SetplayExecutorWithCBR(string(SETPLAY_FILE),
                                         true,//useComm
                                         true,//limitedComm,
                                         new setplay::PlayerID(TA_OUR, agent->world().self().unum()),
                                         2,//repeat time
                                         agent->fcp_world_model(),
                                         cbr_filename.str(),
                                         agent->strategy().setplaysConfig().writeCbrToFile(),//Write to file
                                         //Write after execution
                                         agent->strategy().setplaysConfig().writeCbrToFileAfterExecution(),
                                         agent->strategy().setplaysConfig().weightThisGame(),
                                         agent->strategy().setplaysConfig().weightThisTeam(),
                                         agent->strategy().setplaysConfig().weightOtherTeams());
      }
      else
      {
        M_setplay_manager = new SetplayExecutor(string(SETPLAY_FILE),
                                          true,//useComm
                                          true,//limitedComm,
                                          new setplay::PlayerID(TA_OUR, agent->world().self().unum()),
                                          2,//repeat time
                                          agent->fcp_world_model());
      }
        //DEBUG
        std::cerr<<"Player loaded "<< numberOfSetplays() <<" setplays."<<endl;
        */
      /* if ( soccer::strategy::SetplayConfig::i().useCbr() )
      {

      }*/
        return true;
    }

    void Planner::initGlobals()
    {

    }

    //LMOTA Decision to start or end a setplay and update SBSP
    void Planner::execute( )
    {
        // bool was_plan_in_execution = PlanInExecution(); // To delay 1 cycle the start of my action

        /// No planning during penalty kicks
        if ( M_agent->world().gameMode().isPenaltyKickMode() )
        {
            return;
        }

        if ( !M_setplay_interpreter.hasActiveSetplay() ) // && playerParticipates(M_agent->world().self().unum()))
        {
            // Last condition should be dismissed ! All players should make a decision in regard to the current setplay

            // Execute the setplay
            // dlog.addText( Logger::PLAN, __FILE__" Going for Planning Decision");
            // PlanningDecision( );

            /// Done, always, by every player! => If leader performed the last action => send transition message!
            // update( agent );
            decisionStartPlan();
            return;
        }

        SetplayMediator::update();
    }

    /*int Planner::waitPlayOff(){
      int ScoreDif = agent->world().ourSide() == LEFT ? agent->world().gameMode().scoreLeft() : agent->world().gameMode().scoreRight();
      //bool attack = Mem->WS->PO_BallPos.x > 5.0;
      bool GameBegin = agent->world().time().cycle() < 100;
      bool GameSec = agent->world().time().cycle() > 2000;
      bool GameEnd = (agent->world().time().cycle() > 5500 && agent->world().time().cycle() < 6000)
                      ||(agent->world().time().cycle() > 7500 && agent->world().time().cycle() < 8000);
      bool PartEnd = GameEnd ||
        (agent->world().time().cycle() > 2500 && agent->world().time().cycle() < 3000)||
        (agent->world().time().cycle() > 6500 && agent->world().time().cycle() < 7000);

      int WaitTime = PLAYOFF_WAITTIME; //constant defined above this
        //if (attack) WaitTime = 100; else WaitTime = 100;

        if (ScoreDif<=-3) WaitTime = 130; //era 180  LPR 2011 //Losing Hard lets wait
        if (ScoreDif>3) WaitTime = 30; //LPR 2011 //winning Hard lets go
        //    else if (ScoreDif>=3 || ScoreDif == -1)
        //		{ if (attack) WaitTime = 30; else WaitTime = 50; } //Wining HARD Lets PLAY t!!! Loosing by one? Lets Go

        // LMOTA was:180, but was ruining setplay start...
         else if (ScoreDif == 1 && GameSec) WaitTime = 100;   //Wining Lets Wait!!!  LPR 2011 era 160
        //    else if (ScoreDif == 1) WaitTime = 180;   //Wining Lets Wait!!!

        if (GameBegin) WaitTime = 30;
        if (GameEnd && ScoreDif == 1) WaitTime = 150;  //LPR 2011  era 195
        if (GameEnd && ScoreDif == -1) WaitTime = 50;  //LPR 2011 era 30
        if (GameEnd && agent->world().ball().pos().x > 20.0) WaitTime = 0;

        // LMOTA for free kicks... If at part end, and ball is forward in the field,
        // Shoot as soon as possible...
        // Indirect free kicks are not presently being detected as play modes!
        if(PartEnd && agent->setplay_context().playMode() == rcss::clang::FreeKick_Our && agent->world().ball().pos().x > 30.0)
           WaitTime = 0;
        return WaitTime;
    } */

    void Planner::decisionStartPlan(  )
    {
       dlog.addText( Logger::PLAN, "Calling decide start plan!" );

       if ( M_setplay_interpreter.hasActiveSetplay() )
       {
           return;
       }

       dlog.addText( Logger::PLAN, "Setplay is inactive Kickable %d FastestTeammate %d",
                     M_agent->world().self().isKickable(),
                     M_agent->world().interceptTable()->fastestTeammate() != NULL ?
                        M_agent->world().interceptTable()->fastestTeammate()->unum() :
                        -1);

       if ( !M_agent->world().self().isKickable() )
       {
           // Hopefully, someone will dash to the ball location
           return;
       }

       // LMOTA: REMOVED
       // Check for delay when not in play-on
       // The same goes for the goalie catch, since the player can move the ball:
       // if Setplay is not immediatly started, standard moves are executed...
       //if(Mem->PlayMode== PM_Play_On || Mem->PlayMode==PM_Drop_Ball
          //|| Mem->PlayMode==PM_My_Goalie_Free_Kick
       // )
       //setplayTimer=0;
       //else
       //setplayTimer++;

      // Poor implementation => Will analyze setplays that are not even used by the current tactic
      // This should be a virtual method which should be overriden by this class
      // This should be delegated to the implementer which should return a list of the admissible setplays (in this case according to the tactic)

      // Whenever tactics change => Update the admissible set-plays to be used by set-play selector

      M_setplay_selector->selectSetplay( M_setplay_interpreter );

      if ( !M_setplay_interpreter.hasActiveSetplay() )
      {
          // No relevant set-play available
          return;
      }

      // TODO
      // When should we decide to start the set-play? as soon as the ball is kickable?
      // What if the game is stopped?

      // Choose one of the possible Setplays
      // As a simple initial option, the first available one will be chosen,
      // only if the player is the one closest to the ball

        if ( !M_setplay_interpreter.hasActiveSetplay() )
        {
              dlog.addText(Logger::PLAN, "No setplay could be chosen");
              return;
        }

        dlog.addText(Logger::PLAN, "Chose setplay id # %d", M_setplay_interpreter.currentSetplay()->id() );


        // Check if leadPlayer is a playerID. If so, and it isn't my number, will exit
        if ( M_setplay_interpreter.currentStep()->hasLeader() &&
             M_setplay_interpreter.currentStepLeader() == *M_self )
        {
            return;
        }

        // DEBUG Positions upon instantiation

        // Instantiate the chosen setplay
        SetplayStartMessage init_msg( *M_self,
                                      M_agent->world().time().cycle(),
                                      M_setplay_interpreter.currentSetplay()->id() );
        for ( Parameters::const_iterator it = M_setplay_interpreter.currentSetplay()->parameters()->begin();
              it != M_setplay_interpreter.currentSetplay()->parameters()->end();
              it++)
        {
            init_msg.push( it->first, M_setplay_interpreter.resolve( it->first)->clone() );
        }

        stringstream ss;
        for ( Players::const_iterator itp = M_setplay_interpreter.currentSetplay()->players()->begin();
              itp != M_setplay_interpreter.currentSetplay()->players()->end(); itp++)
        {
            PlayerAffiliation player = M_setplay_interpreter.interpret( **itp );
            ss << " " << player.number();
        }

        //DEBUG
        dlog.addText(Logger::PLAN,
                   "Player#%d InitMessage used to start Setplay #%d (%d players) in cycle %d: Partic: %d %s",
                   M_self->number(),
                   M_setplay_interpreter.currentSetplay()->id(),
                   M_setplay_interpreter.currentSetplay()->players()->size(),
                   M_world_model->currentTime(),
                   M_setplay_interpreter.currentSetplay()->playerVars().size(),
                   ss.str().c_str() );

        // Set a memo to self for the next iteration
        // receive ( init_msg );
      }


    /* void Planner::updateByHear( )
    {
        // Check if a setplay init message was received
        fcportugal::AudioMemory* audio_memory
                = const_cast<fcportugal::AudioMemory*>((const fcportugal::AudioMemory*) &M_agent->world().audioMemory());

        // Check for init messages
        if ( !audio_memory->setplayStartSetplayMessages().empty() )
        {
            const fcportugal::AudioMemory::InitMessageHeard* last_init_msg
                    = audio_memory->setplayInitMessages().back();

            // dlog.addText(Logger::PLAN, "Received an init message %s",
                         // last_init_msg->message().toString().c_str());

            // Is there an active setplay
            if ( M_active_setplay == NULL )
            {
                //dlog.addText( Logger::PLAN,
                  //           __FILE__ " Starting a new setplay %d",
                    //         last_init_msg->message().setplayNumber);
                // receive(last_init_msg->message());
            }
            else
            {
                dlog.addText( Logger::PLAN, __FILE__ " DANGER A setplay is already active");

                // If it is the same should it be restarted?

                // If I'm the lead player, only I can communicate
                // assert (am_current_leader() && last_init_msg->sender() == agent->world().self().unum());
                 if ( am_current_leader() && last_init_msg->sender() != M_agent->world().self().unum())
                 {
                     dlog.addText( Logger::PLAN,
                                  "BUG? Received a setplay message from a non-leader #%d",
                                  last_init_msg->sender());
                 }
                 else
                 {
                     receive(last_init_msg->message());
                 }
            }
            dlog.addText(Logger::PLAN, "Finished processing init message");
                         // , last_init_msg->message().toString().c_str());
            audio_memory->clearInitMessages();
        }

        // Check for step messages
        if ( !audio_memory->setplayStepMessages().empty() )
        {
            const fcportugal::AudioMemory::StepTransitionMessageHeard* last_step_msg
                    = audio_memory->setplayStepMessages().back();
            dlog.addText(Logger::PLAN, "Received a step message");
            // , last_step_msg->message().toString().c_str());
            if ( !setplay_active() )
            {
                dlog.addText( Logger::PLAN,
                             __FILE__ " PreAnalysis: Received a step message %s but no setplay is active");
                             // last_step_msg->message().toString().c_str());
            }
            else
            {
                receive(last_step_msg->message());
                dlog.addText(Logger::PLAN, "Finished processing step message %s");
                             // last_step_msg->message().toString().c_str());
            }
            audio_memory->clearStepMessages();
        }
    }*/


    //execution of plan: choose and execute one of the available actions (only executed by setplay participants)
/*    void Planner::decision(  )
    {
      dlog.addCircle(Logger::PLAN,
                     rcsc::Circle2D(M_agent->world().self().pos(), 1.0),
                     am_current_leader() ? "white" : "blue",
                     true);

      //Check if there are actions to execute
      // vector<vector<Action* > *> * sp_actions =   possibleActions(agent->fcp_world_model());
      ::setplay::IAction* chosen_action = nextAction();

      // DEBUG
      // dlog.addText(Logger::PLAN, "Next action %d available actions (%s) in cycle %d",
      //              agent->world().self().unum(),

      //              sp_actions ? sp_actions->size() : 0,
      //              ss.str().c_str(),
      //              agent->world().time().cycle());

        bool last_action = true;
        // Fall-back => If I'm not the leader of the step and I have no more actions to do, repeat the last
        // This will likely continue until a new transition is chosen
        if (chosen_action == NULL && !am_current_leader())
        {
            // If a next step was already chosen and I have a positional action at the start of the step
            if ( has_next_step() )
            {
                // Check if I have a position action in the next step to do => It might not be relevant to do so
                const IParticipation* next_step_participation = M_next_step->participation( M_context_provider->self() );
                if ( next_step_participation != NULL && next_step_participation->location() != NULL )
                {
                    chosen_action = new ActPos( next_step_participation->location()->clone() );
                    stringstream ss;
                    // chosen_action->print(ss);
                    dlog.addText( Logger::PLAN, __FILE__
                                     " All transition actions done, moving the anticipated position "
                                     " %s of the next step",
                                     ss.str().c_str());
                }
            }
        }


        if (chosen_action)
        {
            stringstream ss;
            // chosen_action->print(ss);
            dlog.addText(Logger::PLAN, "Chosen action %s", ss.str().c_str());
        }
        else
        {
            dlog.addText(Logger::PLAN, "No action chosen");
        }

        // It is probably useful to turn the neck in the direction of a relevant player
        // some actions are directed to a player. Also, when executing no action, the players should look at the lead player
        unsigned int playerToTurnTo=0;

        bool willGoToPosition=false;

        // setplay position in current step

        // This should be transformed into an action !!!
        Vector2D setplay_position =  myCurrentPositioning();

        dlog.addText(Logger::PLAN,
                     "chosenAction: %s, setplayPos: %s",
                     chosen_action ? "yes" : "no",
                     setplay_position != Vector2D::INVALIDATED ? "yes" : "no");

        // If there is no chosen action, player will move to it's position
        if ( chosen_action==NULL )
        {
          if ( !am_current_leader() )
          { // RL
            willGoToPosition = true; //go to position if not leadPlayer
          }
          else
          { //will dribble while deciding which action to take
              if( setplay_position != Vector2D::INVALIDATED )
              {
                  Point2D* setplay_position_simple = new Point2D(new Decimal(setplay_position.x),
                                                                    new Decimal(setplay_position.y));
                  // Dribble to setplay position
                  ActDribble dribble(setplay_position_simple);
                  dribble.update ( M_context_provider );
                  dribble.getExecuted( *M_agent );
              }
              willGoToPosition=false;
            }
        }
        else
        { // chosenAction != NULL
          stringstream out;
          // chosen_action->print(out);
          dlog.addText(Logger::PLAN, __FILE__ " %s Active: %s Done: %s PlayMode: %s",
                       out.str().c_str(),
                       chosen_action->active?"yes":"no",
                       chosen_action->done?"yes":"no" ,
                       fcportugal::WorldModel::S_play_mode_strings[M_agent->world().gameMode().getServerPlayMode()]);

          // Instantiate the action if it has not yet been instantiated
          if ( chosen_action->active && !chosen_action->instantiated() )
          {
              chosen_action->update ( M_context_provider );
          }

          // Relevant players based on the current player action who participates in the setplay
          // TODO What if he doesn't participate in the play or the step?
          // => Relevant player should be the leader but omit this logic from the agent

          vector<const IPlayerID*>* relevantPlayers = chosen_action->relevantPlayers();

          if (relevantPlayers && !relevantPlayers->empty() && relevantPlayers->at(0) != NULL )
          {
             playerToTurnTo = relevantPlayers->at(0)->number();
            //DEBUG
            stringstream ss;
            // ((setplay::PlayerID*)relevantPlayers->at(0))->print(ss);
            // ss<<(int)playerToTurnTo<<" ori: " <<relevantPlayers->at(0)->value()->number;
            //  const Point2D* relevantPlayerPos = agent->setplay_context().playerPos(*relevantPlayers->at(0)->value());
            // dlog.addLine(Logger::PLAN, agent->world().self().pos, Vector2D(relevantPlayerPos->getX()->value(),
            //                       relevantPlayerPos->getY()->value());
            // dlog.addText(Logger::PLAN, "Jogador relevante e %s", ss.str().c_str());
          }

          // Execute only when action is active, ie, waitTime as elapsed
          if ( !chosen_action->active )
          {
            if( am_current_leader() && setplay_position != Vector2D::INVALIDATED )
            {
                if ( M_agent->fcp_world_model().playMode() == PlayOn )
                { // Will dribble
                    Point2D* setplay_position_simple = new Point2D(new Decimal(setplay_position.x),
                                                                   new Decimal(setplay_position.y));
                    ActDribble( setplay_position_simple ).getExecuted( *M_agent );
                    willGoToPosition=false;
                }
                else
                { // Do nothing: not in play_on
                    // LogDrawInvTextN(DSETPLAY,23,WS->PO_MyPos.x-5, WS->PO_MyPos.y-5,"NADA!","Red");
                    willGoToPosition=false;
                }
            }
            else
            { // Not the lead player: will move to positioning, further down...
              willGoToPosition=true; // Do nothing here
            }
          }
          else if(!chosen_action->done )
          {
              //DEBUG
              std::stringstream ss;
              // chosen_action->print(ss);
              dlog.addText( Logger::PLAN, "Going to execute the action %s", ss.str().c_str());

              // chosenAction(chosen_transition_id, agent->fcp_world_model());

              // Order the execution of the action => Shouldn't this be hidden in a framework method
              bool action_executed = chosen_action->getExecuted( *M_agent );
              dlog.addText( Logger::PLAN, __FILE__ " Action executed %d", action_executed);

              willGoToPosition = false;
           }
          else
          {
              // Action is done: back to positioning! Should check for more actions (TO BE HANDLED BY FRAMEWORK)
              willGoToPosition=true; // Do nothing here
           }
        }

        //All other cases: move to positioning in setplay
        if( willGoToPosition && setplay_position != Vector2D::INVALIDATED &&
            M_agent->world().self().pos().dist(setplay_position) > 0.2)
        {
          // If there is no relevant player, turn to lead player
          if(playerToTurnTo == 0)
          {
              playerToTurnTo = M_current_step->leader()->number();
          }

          // Truncate move position to within field boundaries?
          setplay_position = M_agent->fcp_world_model().truncateToField(setplay_position);

          dlog.addText( Logger::PLAN, __FILE__ " Moving to setplay positon (%.2f,%.2f) from (%.2f,%.2f)",
                       M_agent->world().self().pos().x, M_agent->world().self().pos().y,
                       setplay_position.x, setplay_position.y);

          ActPos move_to_setplay_pos(new Point2D(new Decimal(setplay_position.x),
                                                 new Decimal(setplay_position.y));
          move_to_setplay_pos.update( M_context_provider );
          move_to_setplay_pos.getExecuted( *M_agent );
          return;
       }

        double player_dist_to_setplay_position = M_agent->world().self().pos().dist(setplay_position);

        if ( playerToTurnTo != 0 && (int) playerToTurnTo != M_agent->world().self().unum() &&
           ( !willGoToPosition || setplay_position==Vector2D::INVALIDATED ||  player_dist_to_setplay_position < 0.5) &&
           !am_current_leader())
        {
            //RL if stopped at position can turn body with no conflict
            //DEBUG
          dlog.addLine( Logger::PLAN, M_agent->world().self().pos(),
                        M_agent->world().ourPlayer(playerToTurnTo)->pos(), "red");

          // Turn neck only
          if (chosen_action == NULL || !chosen_action->active )
          {
              dlog.addText( Logger::PLAN, __FILE__" 1) No chosen action => turning body and neck to ball");
              Bhv_BodyNeckToBall().execute( M_agent );
          }
        }
        else if(playerToTurnTo!=0 && (int)playerToTurnTo!=M_agent->world().self().unum())
        {
          // if on way to position just turns neck if possible
          //DEBUG
          // LogDrawInvTextN(DSETPLAY,23, WS->PO_TeammatePos[(int)playerToTurnTo].x,
          //                 WS->PO_TeammatePos[(int)playerToTurnTo].y-5, "Look at %u! (JN)","Red",playerToTurnTo);
          // Turn neck only
          // if (chosen_action == NULL || !chosen_action->active)
          // {
          //     dlog.addText( Logger::PLAN,
            //                __FILE__" 2) No chosen action => turning body and neck to player %d",
          //                  playerToTurnTo);
            //   Bhv_BodyNeckToPoint(M_agent->world().ourPlayer(playerToTurnTo)->pos()).execute( M_agent );
         //  }
          // ActAttentionToReg(new RegPoint(new PointPlayer(new PlayerID( TA_OUR ,
          //                                                              playerToTurnTo)))).getExecuted( *agent );
        // }
       //  else
       //  { // default neck behaviour
          // TODO Decide where to look! For now look to ball, but could look to leader or relevant player for my next action??
        //   if (chosen_action == NULL || !chosen_action->active)
         //  {
        //       dlog.addText( Logger::PLAN, __FILE__" 3) No chosen action => turning body and neck to ball or scan");
          //     Bhv_BodyNeckToBall().execute( M_agent );
         //  }
        // }

        if (am_current_leader())
        {
            if ( (chosen_action == NULL || (chosen_action != NULL && chosen_action->done)) &&
                 M_next_step != NULL)
            {
                const IPlayerID& next_leader = *M_next_step->leader();
                // TODO What if there is no next step leader
                if ( next_leader.number() != M_context_provider->self().number() )
                {
                    dlog.addText( Logger::PLAN,
                                 __FILE__" I'm the leader and have no more actions => "
                                 "Paying attention to next step leader %d",
                                 next_leader.number() );
                    M_agent->doAttentionto(M_agent->world().ourSide(), next_leader.number());
                }
                else
                {
                    dlog.addText( Logger::PLAN,
                                 __FILE__" I'm the next leader =>"
                                 "Turning attentionTo off to anyone specific (general hearing)");
                    M_agent->doAttentiontoOff();
                }
            }
            else
            {
                dlog.addText( Logger::PLAN,
                             __FILE__" I'm the leader => "
                             "turning attentionTo off to anyone specific (general hearing)");
                M_agent->doAttentiontoOff();
            }
        }
        else if ( M_current_step != NULL )
        {

            const IPlayerID& curr_leader = *M_current_step->leader();
            // Start paying attention to the current setplay leader
            dlog.addText( Logger::PLAN,
                         __FILE__" Not the leader so paying attention to leader %d",
                         curr_leader.number());
            M_agent->doAttentionto( M_agent->world().ourSide(), curr_leader.number() );
        }
    }*/


    //Players communicate: decision is made by Setplay framework
    // was: Start of Plan, Change of Step, Eco of Change, and Plan End!


    //**************************************************************************************
    //***************************** UTILITY FUNCTIONS **************************************
    //**************************************************************************************

    // Desactiva o DPRE durante a execucao de um plano e uns momentos apos (porque os jogadores
    // nao saiem do plano todos ao mesmo tempo e podem reentrar)
    // TODO deixar fazer DPRE para jogadores sobrepostos
    //bool Portugal1Info::PlanDisableDPRE(void) { return false;}


    /*bool Planner::playerParticipates(unsigned int player_unum) const
    {
      return SetplayExecutor::playerParticipates(setplay::PlayerID(TA_OUR,player_unum));
    }*/

}
