#include "strategy_2d.h"

#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>
#include <rcsc/player/abstract_player_object.h>
#include <rcsc/player/intercept_table.h>
#include <rcsc/param/param_map.h>
#include <rcsc/player/player_agent.h>

#include <fcportugal_player.h>
#include <fcportugal/dpre.h>
#include <fcportugal/planning.h>

#include <strategy/strategy_config.h>
#include <strategy/formation_config.h>
#include <strategy/league_config.h>
#include <strategy/tactic/tactic.h>


#include "../fcportugal_player.h"
// Flux Manager

#include "flux_manager.h"
#include <fcportugal/dpre.h>

using namespace soccer;
using namespace rcsc;
using namespace setplay;

using namespace soccer::strategy;
using namespace soccer::tactic;
using namespace soccer::flux;

namespace fcportugal
{
    namespace strategy
    {

        Strategy2D::Strategy2D()
        {
            init();
        }

        Strategy2D& Strategy2D::instance()
        {
            static Strategy2D strategy;
            return strategy;
        }

        void Strategy2D::setAgent( const FCPortugalPlayer *agent)
        {
            M_agent = agent;
        }

        void Strategy2D::setWorldModel( const soccer::IWorldModel* world_model)
        {
            M_world_model = world_model;
        }

        void Strategy2D::update( )
        {
            dlog.addText( Logger::POSITIONING,
                         __FILE__ " Tactic: %d, Formation: %d, FluxType: %d, BPos: (%.2f,%.2f)",
                         M_current_tactic != NULL ? M_current_tactic->id() : -1,
                         M_current_formation != NULL ? M_current_formation->id() : -1,
                         M_current_flux != NULL ? M_current_flux->type() : -1,
                         M_world_model->ballPosition().x, M_world_model->ballPosition().y );

            Strategy::update( M_world_model );

            /*FCPortugalPlayer* sagent = static_cast<FCPortugalPlayer*>( M_agent );
            // HACK TO KEEP PREV FORMATION FOR A WHILE
            if ( M_last_formation != M_current_formation &&
                 M_last_formation.get() != NULL &&
                 M_agent->world().gameMode().type() == rcsc::GameMode::PlayOn &&
                 M_agent->world().time().cycle() - M_last_formation_change.cycle() < FormationConfig::i().smoothTransitionsDelay() )
            {
                dlog.addText( Logger::POSITIONING,
                              __FILE__ " OldForm: %d, NewForm: %d,Keeping last formation %d positioning instead for %d cycles",
                              M_last_formation->id() ? M_last_formation->id() : -1,
                              M_current_formation ? M_current_formation->id() : -1,
                              M_last_formation->id(),
                              FormationConfig::i().smoothTransitionsDelay() - (M_agent->world().time().cycle() - M_last_formation_change.cycle()) );
                // Keep the positions of the previous formation during a given period
                stringstream ss;
                ss << "HoldOnFormations " << endl;
                for ( unsigned int unum = 1; unum <= (unsigned int) ServerParam::i().maxPlayer(); unum++ )
                {
                    ss << " #" << unum
                       << " OldPos" << M_last_formation->position( unum, sagent->fcp_world_model() )
                       << " NewPos(" <<  M_current_formation->position( unum, sagent->fcp_world_model())
                       << endl;
                    Vector2D player_position = M_last_formation->position(unum, sagent->fcp_world_model());
                    boost::shared_ptr<PlayerType> player_type = playerType(unum);
                    player_position = positionLeagueSpecific(player_position);
                    ss << " PLeagueSpecific" << player_position;

                    // Modify player position after formation
                    if ( player_type )
                    {
                        ss << " PlayerType "  << player_type->role();
                        player_position = positionMaxMin(*player_type, player_position);
                        ss << " PMaxMin" << player_position;
                    }
                    M_positions[unum] = player_position;
                }
                dlog.addText( Logger::POSITIONING,
                              __FILE__ "%s", ss.str().c_str());
            } */

            if ( StrategyConfig::i().useDPRE() )
            {
                dlog.addText( Logger::POSITIONING, " Calculating DPRE...");
                if ( M_agent->world().gameMode().type() == rcsc::GameMode::AfterGoal_ ||
                     M_agent->world().gameMode().type() == rcsc::GameMode::BeforeKickOff)
                {
                    fcportugal::DPRE::instance().reset( );
                }

                if ( M_agent->world().gameMode().type() == rcsc::GameMode::PlayOn ||
                     M_agent->world().gameMode().isTeamsSetPlay(rcsc::LEFT) ||
                     M_agent->world().gameMode().isTeamsSetPlay(rcsc::RIGHT))
                {
                    fcportugal::DPRE::instance().update( );
                }
            }
            // The use of fluxes needs to be executed only when the player has the ball and for the player who has it
            // Other players will adjust their position based on the strategic position or should it also be
            // updated according to flux

            // DEBUG Positions
            dlog.addText( Logger::POSITIONING, __FILE__, " Debugging formation positionings");
            for (  UnumPositions::const_iterator it_our_pos = M_positions.begin();
                   it_our_pos != M_positions.end();
                   it_our_pos++)
            {
                dlog.addCircle( Logger::POSITIONING, rcsc::Circle2D( (*it_our_pos).second, 0.25), "#515383");

                /*Vector2D pl_pos = world_model->ourPlayerPosition(pid);
                debug_player_pos << "Cur" << pid;
                if ( pl_pos == Vector2D::INVALIDATED )
                {
                    debug_player_pos << " (inf)";
                }
                else
                {
                    debug_player_pos << " " << pl_pos;
                }

                debug_player_pos << " " << position(pid);
                debug_player_pos << ")";
                debug_player_pos << endl;*/
            }
        }

        bool Strategy2D::init( )
        {
            // Initialize strategic positions
            for ( unsigned int i = 1; i <= (unsigned int) ServerParam::i().maxPlayer() ; i++ )
            {
                M_positions[i] = Vector2D(0.0,0.0);
            }
            return true;
        }

        bool Strategy2D::init( CmdLineParser & cmd_parser )
        {
            bool help = false;
            std::string strategy_conf;

            ParamMap param_map( M_agent->config().teamName() + " strategy options" );
            param_map.add()
                ( "help" , "", rcsc::BoolSwitch( &help ), "print help message.")
                ( "strategy-config", "", &strategy_conf, " league config file." );

            if ( cmd_parser.count( "help" ) > 0 )
            {
                param_map.printHelp( std::cout );
                return false;
            }

            cmd_parser.parse(param_map);

            // Analyze strategy-specific configuration file
            if ( strategy_conf.empty() )
            {
                std::cerr << __FILE__ << "No strategy configuration file specified" << std::endl;
                return false;
            }

            if (! Strategy::init(strategy_conf) )
            {
                std::cerr << __FILE__ << "Could not parse the strategy configuration file" << std::endl;
                return false;
            }

            return true;
        }

        const rcsc::Vector2D& Strategy2D::position(unsigned int player_id, bool use_dpre) const
        {
            if ( StrategyConfig::instance().useDPRE() && use_dpre)
            {
                return M_positions.at( fcportugal::DPRE::instance().positioning(player_id) );
            }
            return M_positions.at( player_id );
        }

        bool Strategy2D::decidePlayerPositionAccordingToPlan(unsigned int player_number)
        {
            return false;
            /* FCPortugalPlayer* agent = static_cast<FCPortugalPlayer*>(M_agent);
            if ( agent->planner()->active_setplay() == NULL)
            {
                dlog.addText( Logger::PLAN, __FILE__ " No current step was chosen.");
                return false;
            } */

            // HACK Must define the player position here!!!
            /*if ( !agent->planner()->playerParticipates( player_number ))
            {
                return false;
            } */

            /* const ::setplay::SetplayExecutor& setplay_executor = *agent->planner();

            if ( setplay_executor.waitTimeHasElapsed( ) )
            {

                rcsc::Vector2D player_position = world_model->ourPlayerPosition(player_number);
                // The player is following actions, so his current position is the strategic position also
                world_model->setStrategicPosition(player_number, player_position);
                dlog.addText( Logger::POSITIONING,
                             __FILE__ " # %d sPos = curPos(%.2f.,%.2f) as he is executing actions in plan",
                             player_number,
                             player_position.x,
                             player_position.y);
                return true;
            }*/

            // Player strategic position will the one he occupies in the setplay
            // HACK This function returns a Point* when it should return a PointSimple
            /*Vector2D plan_position = setplay_executor.currentPositioning(PlayerID("our", player_number));
            if ( plan_position != Vector2D::INVALIDATED )
            {
                // Waiting for the wait time to elapse and start the setplay
                // This always returns the pos at the start of the step and disconsiders the actions already executed
                world_model->setStrategicPosition(player_number, plan_position);
                dlog.addText( Logger::POSITIONING,
                             __FILE__ " # %d sPos = stepInitPos(%.2f.,%.2f) as he is executing actions in plan",
                             player_number,
                             plan_position.x,
                             plan_position.y);
                return true;
            }*/

            // Player has no valid position in the setplay => Already in the plan position
            rcsc::Vector2D curr_player_pos = M_agent->world().self().pos();
            M_positions[player_number] = curr_player_pos;
            dlog.addText( Logger::POSITIONING,
                         __FILE__ " # %d sPos = currPos(%.2f,%.2f)",
                         player_number, curr_player_pos.x, curr_player_pos.y);
            return true;
        }

        rcsc::Vector2D Strategy2D::positionLeagueSpecific( rcsc::Vector2D player_pos,
                                                           const soccer::IWorldModel *world_model )
        {
            return player_pos;
            double buffer_ball = 0.5;  // FM_OffsideBallBuffer;
            double buffer_line = 0.5; // FM_OffsideBuffer;
            double offside_line = world_model->offsideLine( soccer::TA_OUR);
            const rcsc::Vector2D& ball_pos = world_model->ballPosition();
            const soccer::GameMode game_mode = world_model->currentGameMode();

            // Adjust player position to offside line
            player_pos.x = ball_pos.x > offside_line ?
                           std::min( player_pos.x, ball_pos.x - buffer_ball ) :
                           std::min( player_pos.x, offside_line - buffer_line );

            // Adjust player position relative to the ball in freekicks
            if ( (game_mode.gameSituation().type() == soccer::GameSituation::FreeKick && game_mode.gameSituation().team_affiliation() == soccer::TA_OPP) &&
                 ball_pos.x > 0.0 &&
                 player_pos.x > ball_pos.x - buffer_ball)
            {
                // TODO Check if this works for all situations => Guess I should check the side
                player_pos.x = ball_pos.x - buffer_ball;
            }

            // Adjust player x-position if it's their goal-kick
            if ( game_mode.gameSituation().type() == soccer::GameSituation::GoalKick && game_mode.gameSituation().team_affiliation() == soccer::TA_OPP && player_pos.x > 35.0)
            {
                player_pos.x = 35.0;
            }

            // Opponent KickOff
            if ( game_mode.gameSituation().type() == soccer::GameSituation::KickOff && game_mode.gameSituation().team_affiliation() == soccer::TA_OUR )
            {   // ????? The previous condition was in the while loop why?
                while ( player_pos.dist(Vector2D(0.0,0.0))<9.1 )
                {
                    player_pos.x -= 0.5;
                }
            }

            // Make another adjustment to player x-position if the game is on kick-off mode
            if ( game_mode.gameSituation().type() == soccer::GameSituation::KickOff && player_pos.x > -0.7 )
            {
                player_pos.x = -0.7;  // KickOff
            }

            return player_pos;
        }

        void Strategy2D::tacticChanged( )
        {
            stringstream ss;
            M_world_model->gameSituation().print( ss );
            dlog.addText( Logger::POSITIONING,
                         __FILE__ " TacticChanged: %d",
                         currentTactic() != NULL ? currentTactic()->id() : -1,
                         ss.str().c_str());

            /* if ( new_tactic == NULL )
            {
                return;
            } */
            // fcportugal::FluxManager::instance().setFlux( M_flux_matrices.at( new_tactic->flux() ) );
        }


        void Strategy2D::formationChanged( )
        {
            stringstream ss;
            M_world_model->gameSituation().print( ss );
            dlog.addText( Logger::POSITIONING,
                         __FILE__ " FormationChanged: %d %s",
                         M_current_formation != NULL ? M_current_formation->id() : -1,
                          ss.str().c_str() );

        }

    }
}
