#ifndef FCPORTUGAL_STRATEGY_2D_H
#define FCPORTUGAL_STRATEGY_2D_H

#include <rcsc/game_time.h>
#include <rcsc/geom/vector_2d.h>
#include <rcsc/param/cmd_line_parser.h>

#include <strategy/strategy.h>
#include <strategy/singleton.h>


class FCPortugalPlayer;

namespace soccer {
    class IWorldModel;
}

namespace fcportugal
{
    namespace strategy
    {

        class Strategy2D
                : public soccer::strategy::Strategy
        {
        private:
            Strategy2D();

        public:
            static Strategy2D& instance();
            void update( );
            bool init( );
            bool init( rcsc::CmdLineParser & cmd_parser );

            const rcsc::Vector2D& position(unsigned int player_id, bool use_dpre = true) const;

            void setAgent( const FCPortugalPlayer *agent);
            void setWorldModel( const soccer::IWorldModel* world_model);

        protected:
            // Override methods
            bool planInExecution() const;
            bool decidePlayerPositionAccordingToPlan(unsigned int player_number);

            void tacticChanged( );
            void formationChanged( );

            // Override league specific position
            rcsc::Vector2D positionLeagueSpecific(rcsc::Vector2D player_pos,
                                                  const soccer::IWorldModel* world_model);


        private:
            // Utility method => should be rcsc::geom scope
            rcsc::Vector2D closestPoint(const rcsc::Ray2D& ray,
                                        const rcsc::Vector2D& point);

            // Owner of this strategy module
            const FCPortugalPlayer* M_agent;
            const soccer::IWorldModel* M_world_model;

            rcsc::GameTime M_last_formation_change;
            soccer::formation::Formation::Ptr M_last_formation;
        };

    }
}
#endif // FCPORTUGAL_STRATEGY_2D_H
