#include "fcp_audio_codec.h"

#include <string.h>
#include <rcsc/common/audio_codec.h>
#include <cassert>

using namespace rcsc;

#define X_RESOLUTION 1100
#define Y_RESOLUTION 700
#define CONFIDENCE_RESOLUTION 8

FCPAudioCodec::FCPAudioCodec(unsigned int num_chars)
{
    for ( unsigned int i=0 ; i < num_chars; i++ )
    {
        M_decode_char.push_back( (unsigned int) AudioCodec::i().CHAR_SIZE );
    }

    for( unsigned int i=0; i < (unsigned int) AudioCodec::i().CHAR_SIZE; i++ )
    {
        // Can I use charToIntMap() instead
        M_decode_char[(int) AudioCodec::i().intToCharMap().at(i) ] = i;
    }
}

FCPAudioCodec& FCPAudioCodec::instance()
{
    static FCPAudioCodec instance( 256 );

    return instance;
}

// void FCPAudioCodec::long2alphanum( char *msg )
std::string FCPAudioCodec::long2alphanum( )
{
    std::string msg;

    unsigned int char_value, length;
    char msg_aux[20];
    char *pMsg = msg_aux;

    while ( M_info > 0 )
    {
        char_value = M_info % AudioCodec::i().CHAR_SIZE;
         M_info /= AudioCodec::i().CHAR_SIZE;

        *pMsg= AudioCodec::i().charToIntMap().at(char_value);
        pMsg++;
    }
    *pMsg = 0;
    length=strlen(msg_aux);
    for ( unsigned int i=0 ; i<length ; i++ )
    {
        msg += msg_aux[ length - i - 1 ];
    }

    return msg;
}

void FCPAudioCodec::alphanum2long( char *msg )
{
   unsigned short chVal;
   char ch, *pMsg = msg;

   M_info = 0;
   while( *pMsg!=0 )
   {
        ch = *pMsg;
        pMsg++;

        chVal = M_decode_char[(int)ch];
        assert( chVal!= AudioCodec::i().CHAR_SIZE );

        M_info *= AudioCodec::i().CHAR_SIZE;
        M_info += chVal;
   }
}


void FCPAudioCodec::alphanum2longlong(char *msg )
{
   unsigned short chVal;
   char ch, *pMsg = msg;

   M_info = 0;
   while(*pMsg!=0)
   {
      ch = *pMsg;
      pMsg++;

      chVal=M_decode_char[(int)ch];

      assert(chVal!=AudioCodec::i().CHAR_SIZE);

      M_info *= AudioCodec::i().CHAR_SIZE;
      M_info += chVal;
   }
}


void FCPAudioCodec::encodeConfidence(float value)
{
    // Encode confidence using 3bits
    int conf = (int) ( value * 100.0 - 85 ) / 2;
    if (conf < 0) conf = 0;

    addInfo( 8, conf);
}


float FCPAudioCodec::decodeConfidence()
{
    int conf = getInfo( 8 );
    // Decode confidence value from 3 bits
    float value = 0.0;
    value = (float) conf*2.0 + 85.0;

    return value / 100.0;
}

void FCPAudioCodec::encodePlayerUnum(const unsigned int& unum)
{
    addInfo( lengthPlayerUnum(), unum-1 );
}

unsigned int FCPAudioCodec::decodePlayerUnum( )
{
    unsigned int unum = getInfo(11) + 1;
    return unum;
}

void FCPAudioCodec::encodePosition(const rcsc::Vector2D& point)
{
    addInfo( X_RESOLUTION, 550 + (int) (point.x * 10.0 + 0.5 ) );
    addInfo( Y_RESOLUTION, 350 + (int) (point.y * 10.0 + 0.5 ) );
}

rcsc::Vector2D FCPAudioCodec::decodePosition( )
{
    double y = (double) ( getInfo(Y_RESOLUTION) - 350) / 10.0;
    double x = (double) ( getInfo(X_RESOLUTION) - 550) / 10.0;

    return rcsc::Vector2D(x,y);
}


unsigned int FCPAudioCodec::lengthPosition()
{
    return bitsNeeded(X_RESOLUTION + Y_RESOLUTION);
}

unsigned int FCPAudioCodec::lengthPlayerUnum()
{
    return bitsNeeded(11);
}

unsigned int FCPAudioCodec::lengthConfidence()
{
    return bitsNeeded(8);
}

unsigned int FCPAudioCodec::bitsNeeded(unsigned int num_values)
{
    return ceil( log( num_values ) );
}

void FCPAudioCodec::addInfo( unsigned long num_values,
                              long value)
{
    M_info *= num_values;

    if ( value < 0) value = 0;
    if ( value > (long)num_values-1 )
        value=(long)num_values-1;

    M_info += value;

    //debug
    long dval= M_info % num_values;
    assert(dval == value);

    M_size += bitsNeeded(num_values);
}

long FCPAudioCodec::getInfo(unsigned long num_values)
{
    long value = M_info % num_values;
    M_info /= num_values;

    return value;
}

unsigned int FCPAudioCodec::sizeInBits()
{
    return M_size;
}

unsigned int FCPAudioCodec::sizeInBytes()
{
    return ceil(sizeInBits() / 8);
}

std::string FCPAudioCodec::getMessage()
{
    return long2alphanum();
}

void FCPAudioCodec::reset()
{
    M_info = 0;
}



