#include "logger_setplay_execution_observer.h"

#include <setplay/runtime/setplay_interpreter.h>
#include <setplay/generators/sexpr/setplay_generator_sexpr.h>
#include <sstream>
#include <rcsc/common/logger.h>

using namespace setplay;
using namespace rcsc;
using namespace std;

void LoggerSetplayExecutionObserver::stateChanged( const SetplayInterpreter& interpreter )
{
    stringstream ss;
    SetplayGeneratorSexpr gen(ss);
    ss << " SetplayExecutionObserver: "
        << SetplayInterpreter::executionStateStr( interpreter.executionState() )
        << " ";

    switch ( interpreter.executionState() )
    {
        case SetplayInterpreter::Stopped:

        break;
        case SetplayInterpreter::SetplayStarted: // StepWait condition was not yet verified
        {
            ss << " Id=" << interpreter.currentSetplay()->id()
               << " Inverted=" << (interpreter.inverted() ? "true" : "false");

            if ( interpreter.currentSetplay()->hasParameters() )
            {
                ss << " Parameters ";
                for ( Parameters::const_iterator itp = interpreter.currentSetplay()->parameters()->begin() ;
                      itp != interpreter.currentSetplay()->parameters()->end() ; itp++ )
                {
                    (*itp).second->visit( gen );
                }
            }
        }
        break;

        /// Step events
        case SetplayInterpreter::StepChosen:
            ss << "Step=" << interpreter.currentStep()->id();
        break;
        case SetplayInterpreter::StepStarted: // StepStart condition was not yet verified
            ss << "Step=" << interpreter.currentStep()->id();
        break;
        case SetplayInterpreter::StepEnded: // StepStart condition was not yet verified
            ss << "Step=" << interpreter.currentStep()->id();
        break;
        case SetplayInterpreter::StepAborted:
            ss << "Step=" << interpreter.currentStep()->id() << " ";
            interpreter.currentStep()->abortCondition()->visit( gen );
        break;
        /// Transition events
        case SetplayInterpreter::TransitionChosen:
        {
            ss << "Transition=" << interpreter.currentTransition()->transitionType() << " ";
            if ( interpreter.hasNextStep() )
            {
                ss << "NextStep " << interpreter.nextStep()->id();
            }
        }
        break;
        case SetplayInterpreter::TransitionStarted:
        {
            ss << "Transition=" << interpreter.currentTransition()->transitionType() << " ";
            if ( interpreter.hasNextStep() )
            {
                ss << "NextStep " << interpreter.nextStep()->id();
            }
        }
        break;
        case SetplayInterpreter::TransitionEnded:
        {
            ss << "Transition=" << interpreter.currentTransition()->transitionType() << " ";
            if ( interpreter.hasNextStep() )
            {
                ss << "NextStep " << interpreter.nextStep()->id();
            }
        }
        break;
        case SetplayInterpreter::TransitionAborted:
        {
            ss << "Transition=" << interpreter.currentTransition()->transitionType() << " ";
            if ( interpreter.hasNextStep() )
            {
                ss << "NextStep " << interpreter.nextStep()->id() ;
            }
        }
        break;
        case SetplayInterpreter::SetplayAborted:
        {
            ss << "Setplay=" << interpreter.currentSetplay()->id();
            if ( interpreter.currentSetplay()->hasAbortCondition() )
            {
                ss << " AbortCond=";
                interpreter.currentSetplay()->abortCondition()->visit( gen );
            }
        }
        break;
        case SetplayInterpreter::SetplayFinished:
            ss << "Setplay=" << interpreter.currentSetplay()->id();
        break;
    }
    ss << std::endl;

    dlog.addText( Logger::PLAN,
                  __FILE__ " LoggerSetplayExecutionObserver: %s", ss.str().c_str() );
}

