#ifndef SETPLAY_ACTION_EXECUTOR_H
#define SETPLAY_ACTION_EXECUTOR_H

#include <setplay/runtime/iaction_executor.h>
#include <setplay/runtime/icondition_evaluator.h>

class FCPortugalPlayer;

class SetplayActionExecutor
        : public setplay::IActionExecutor
{
public:
    SetplayActionExecutor(FCPortugalPlayer* agent,
                          setplay::IConditionEvaluator* condition_evaluator);

    /// True if the action was executed
    bool noAction();

    bool moveTo( const rcsc::Vector2D& position );
    bool moveTo( const rcsc::Region2D& region );

    bool kickTo( const rcsc::Vector2D& position );

    bool passToPlayer( const setplay::PlayerAffiliation& player );
    bool passToPoint( rcsc::Vector2D point );
    bool passToRegion( rcsc::Region2D& region );
    bool forwardToPlayer( const setplay::PlayerAffiliation& player );

    bool shoot( );

    bool dribbleTo( const rcsc::Vector2D& position );
    bool dribbleTo( const rcsc::Region2D& region );
    bool holdBall();

    bool waitAtPosition( const rcsc::Vector2D&  );
    bool waitAtRegion( const rcsc::Vector2D& );

    bool openPassLineForPlayer( const setplay::PlayerAffiliation &player );

    // Interception
    bool interceptPlayer( const setplay::PlayerAffiliation &player);
    bool interceptBall( );
    bool interceptBallAt( rcsc::Vector2D point );

    bool followPlayer( const setplay::PlayerAffiliation &player,
                       const rcsc::Vector2D &offset );
    bool followBall( rcsc::Vector2D offset = rcsc::Vector2D(0,0) );

    bool markPlayer( const setplay::PlayerAffiliation& player );

    /// Look at commands
    bool lookAt( const rcsc::Vector2D& position );
    bool lookAt( const rcsc::Region2D& region );
    bool lookAt( const setplay::PlayerAffiliation& player );
    bool lookAt( const std::vector<setplay::PlayerAffiliation>& players );

private:
    FCPortugalPlayer* M_agent;
    setplay::IConditionEvaluator* M_condition_evaluator;
};

#endif // SETPLAY_ACTION_EVALUATOR_H
