#include "setplay_condition_evaluator.h"

#include <setplay/model/conditions.h>
#include <rcsc/common/server_param.h>
#include <rcsc/player/intercept_table.h>

#include <rcsc/action/body_pass.h>

#include "setplay_world_model.h"

#include <rcsc/common/logger.h>

using namespace setplay;
using namespace rcsc;

// Static variables
const double SetplayConditionEvaluator::S_position_reached_distance_threshold = 0.5;

SetplayConditionEvaluator::SetplayConditionEvaluator(const fcportugal::SetplayWorldModel* world)
    : M_setplay_world( world )
{

}

/// Implementation of IConditionEvaluator interface


/*bool SetplayConditionEvaluator::evaluate( const CondClearShotAtGoal& condition ) const
{
    stringstream ss;
    condition.print(ss);
    dlog.addText( Logger::PLAN, __FILE__" CondClearShotAtGoal: Starting evaluation of %s", ss.str().c_str());
    return true; // HACK

    // TODO HACK Due to bug in set-play framework concept definitions
    // vector<IParticipant*>* players = const_cast<vector<IParticipant* >* >(condition.getFromPlayerSet());
    vector<IParticipant*>* players = new vector<IParticipant*>();
    for (unsigned int i = 0; i < condition.getFromPlayerSet()->size (); i++) {
        players->push_back(condition.getFromPlayerSet()->at(i));
    }

    // Trivial condition verification => Player has the ball and is in their penalty box
    return evaluate(CondBallOwner(players));
    // HACK
       // && evaluate(CondPlayerPos(players,
       //                       M_name_substitutions.getSubstitutionRegion("their_penalty_box")->deepCopy(NULL,NULL),
       //                         new Integer(1),
       //                         new Integer(players->size())));
}

bool SetplayConditionEvaluator::evaluate( const CondClearPassToPlayer& condition ) const{
    return true; // HACK to always enable passing

    stringstream ss;
    condition.print(ss);
    cerr << ss.str() << endl;
    // dlog.addText( Logger::PLAN, "CondClearPass: Evaluating condition %s", ss.str().c_str());

  vector<IParticipant*>::const_iterator player_to_begin =  condition.getToPlayerSet()->begin();
  vector<IParticipant*>::const_iterator player_to_end =  condition.getToPlayerSet()->end();
  vector<IParticipant*>::const_iterator player_from_begin =  condition.getFromPlayerSet()->begin();
  vector<IParticipant*>::const_iterator player_from_end =  condition.getFromPlayerSet()->end();

  vector<IParticipant*>::const_iterator it_player_from = player_from_begin;
  while (it_player_from != player_from_end) {
    for( vector<IParticipant*>::const_iterator it_player_to = player_to_begin; it_player_from != player_to_end; it_player_to++) {
      const setplay::PlayerID* player_id_from = (*it_player_from)->value(); // Ball owner player is core dumping here, but text of cond is correct and so should the world model be
      const setplay::PlayerID* player_id_to = (*it_player_to)->value();

      if (player_id_from == NULL || player_id_to == NULL) {
          stringstream ss1;
          cerr << "One of the player ids is NULL " << endl;
          dlog.addText(Logger::PLAN,
                       "isNull(PlayerIdFrom) = %d isNull(playerIdTo)= %d ",
                       player_id_from != NULL ? player_id_from->value()->number : -1,
                       player_id_to != NULL ? player_id_to->value()->number : -1);
          // assert (player_from != NULL && player_to != NULL);
          continue;
      }

      const AbstractPlayerObject* player_from = getPlayer(*player_id_from);
      const AbstractPlayerObject* player_to = getPlayer(*player_id_to);

      if (player_from == NULL || player_to == NULL) {
          cerr << "One of the concrete players is NULL " << endl;
          dlog.addText(Logger::PLAN,
                       "isNull(PlayerFrom) = %d isNull(playerTo)= %d",
                       player_from != NULL ? player_from->unum() : -1,
                       player_to != NULL ? player_to->unum() : -1);
          // assert (player_from != NULL && player_to != NULL);
          continue;
      }

      double distance = player_from->pos().dist(player_to->pos());

      // TODO Check if it is possible to generate a pass between the 2 players
      if (distance <= PASS_MAX_DIST ) {
          // The pass_angle should be used in a pass evaluator
          //double pass_angle = AngleDeg::atan2_deg(player_from->pos().y - player_to->pos().y,
          //                                       player_from->pos().x - player_to->pos().x);

        Triangle2D triangle(Vector2D(player_from->pos().x, player_from->pos().y),
                                  Vector2D(player_to->pos().x - distance * .2, player_to->pos().y),
                                  Vector2D(player_to->pos().x + distance * .2, player_to->pos().y));
        // TODO Check the correctness of this
        // Pass only if there are no more than 2 players within the triangle (even if they are from the same team??)
        // Should use a pass evaluator based on the world model
        // canPass(wm, pl1, pl2)
        // None of the players should be outside the field (or if it is it must not be too far from it)
        const unsigned int MAX_PLAYERS_IN_PASS_TRIANGLE = 2;
        unsigned int players_in_triangle = 0;
        for (AbstractPlayerCont::const_iterator it_player = world_model->allPlayers().begin();
             it_player != world_model->allPlayers().end() && players_in_triangle < MAX_PLAYERS_IN_PASS_TRIANGLE;
             it_player++) {
            const AbstractPlayerObject* player = *it_player;
            if (triangle.contains(player->pos())) {
                players_in_triangle++;
            }
        }

        if (players_in_triangle == MAX_PLAYERS_IN_PASS_TRIANGLE) {
            return true;
        }
      }

    }
  }
  return false;
}

bool SetplayConditionEvaluator::evaluate( const CondClearPassToRegion& condition ) const {
    assert ( condition.getToRegion() != NULL &&
            condition.getFromPlayerSet() != NULL &&
            !condition.getFromPlayerSet()->empty());

    stringstream ss_debug;
    condition.print(ss_debug);
    dlog.addText( Logger::PLAN, __FILE__ " CondClearPassToReg: %s", ss_debug.str().c_str());

    Vector2D target_point = condition.getToRegion()->center();

    vector<IParticipant*>::const_iterator it_player_from = condition.getFromPlayerSet()->begin();
    while (it_player_from != condition.getFromPlayerSet()->end()) {
        const setplay::PlayerID* player_id_from = (*it_player_from)->value();
        if ( player_id_from == NULL ) {
            dlog.addText( Logger::PLAN, __FILE__" CondClearPassToReg: From PlayerId is NULL");
            continue; // assert (player_id_from != NULL);
        }

        const AbstractPlayerObject* player_from = getPlayer(*player_id_from);
        if ( player_from == NULL) {
            dlog.addText( Logger::PLAN, __FILE__" CondClearPassToReg: Concrete player not found");
            continue; // assert ( player_from != NULL );
        }

        double distance = player_from->pos().dist(target_point);

        dlog.addText( Logger::PLAN, __FILE__ " CondClearPassToReg: Check canPass from %d to (%.2f,%.2f) distance=%.2f<%.2f",
                     player_from->unum(), target_point.x, target_point.y, distance, PASS_MAX_DIST);
        // TODO Check if it is possible to generate a pass between the 2 players
        if (distance <= PASS_MAX_DIST ) {
            // The pass_angle should be used in a pass evaluator
            //double pass_angle = AngleDeg::atan2_deg(player_from->pos().y - player_to->pos().y,
              //                                      player_from->pos().x - player_to->pos().x);

          Triangle2D pass_triangle(Vector2D(player_from->pos().x, player_from->pos().y),
                                         Vector2D(target_point.x - distance * .2, target_point.y),
                                         Vector2D(target_point.x + distance * .2, target_point.y));
          // TODO Check the correctness of this
          // Pass only if there are no more than 2 players within the triangle (even if they are from the same team??)
          // Should use a pass evaluator based on the world model // canPass(wm, pl1, pl2)
          // None of the players should be outside the field (or if it is it must not be too far from it)
          const unsigned int MAX_PLAYERS_IN_PASS_TRIANGLE = 2;
          unsigned int players_in_triangle = 0;
          for (AbstractPlayerCont::const_iterator it_player = world_model->allPlayers().begin();
               it_player != world_model->allPlayers().end() && players_in_triangle <= MAX_PLAYERS_IN_PASS_TRIANGLE;
               it_player++) {
              const AbstractPlayerObject* player = *it_player;
              if (player != NULL && pass_triangle.contains(player->pos())) {
                  dlog.addText( Logger::PLAN,
                               __FILE__" CondClearPassToReg: %s#%d contained in triangle (%.2f,%.2f, %.2f)",
                               player->side() == world_model->ourSide() ? "our" : "opp",
                               player->unum(),
                               pass_triangle.a(), pass_triangle.b(), pass_triangle.c());
                  players_in_triangle++;
              }
          }

          dlog.addText( Logger::PLAN,
                       __FILE__ " CondClearPassToReg: Can Pass because players in triangle %d <= %d == %s",
                       players_in_triangle, MAX_PLAYERS_IN_PASS_TRIANGLE,
                       players_in_triangle <= MAX_PLAYERS_IN_PASS_TRIANGLE ? "true" : "false");
          if (players_in_triangle <= MAX_PLAYERS_IN_PASS_TRIANGLE) {
              return true;
          }
        }
        it_player_from++;
    }
    dlog.addText( Logger::PLAN, __FILE__ " CondClearPassToReg: Cannot pass to region");
    return false;
}

// Are any of the players near their opponent offside line
bool
SetplayConditionEvaluator::evaluate(const CondNearOffsideLine& condition) const{
    //DEBUG
    //cout<<"SP: teste NearOffsideLine"<<endl;
    // Setup
    vector<IParticipant*>::const_iterator it_player = condition.getPlayerSet()->begin();

    while (it_player != condition.getPlayerSet()->begin()) {
        const setplay::PlayerID* player_id = (*it_player)->value();

        assert(player_id != NULL);
        // if (player_id == NULL) { it_player++; continue; }

        // Get a reference to the abstract player from the worldModel
        // NOTE: Danger returns [0] if number is invalid => Should we check this for NULL values
        // Figure out what is in this position
        const AbstractPlayerObject* player = getPlayer(*player_id);

        assert(player != NULL);

        double defense_line_x = player_id.team == "our" ?
                              world_model->theirDefenseLineX() :
                              world_model->ourDefenseLineX();
        //DEBUG
        //Mem->LogDrawCircle(DSETPLAY,23,position.x,position.y,
        //	       BALL_OWNER_DISTANCE_THRESHOLD,"White");

        // Check if player is up to 1.5 m before the offside line
        double distance_to_defense_line_x = player->side() == LEFT ?
                    player->pos().x - defense_line_x :
                    defense_line_x - player->pos().x;

        // TODO Make sure this logic is correct!!!
        // Check if distance from player to defense line is lower or equal than a threshold
        if( distance_to_defense_line_x <= 0
         && distance_to_defense_line_x >= -1.5 )
          return true;

        it_player++;
    }
    return false;
} */

bool SetplayConditionEvaluator::playerAtRegion( const PlayerAffiliation &player,
                                                const Region2D &region )
{
    const AbstractPlayerObject* aplayer = M_setplay_world->getAbstractPlayerObject( player );

    if ( aplayer == NULL )
    {
        return false;
    }

    bool res = region.contains( aplayer->pos() );

    dlog.addText( Logger::PLAN,
                  __FILE__, " SetplayConditionEvaluator: playerAtRegion=%s",
                  res ? "TRUE" : "FALSE");

    return res;
}

bool SetplayConditionEvaluator::playerAtPosition( const setplay::PlayerAffiliation& player,
                                                  const rcsc::Vector2D& position )
{
    const AbstractPlayerObject* aplayer =  M_setplay_world->getAbstractPlayerObject( player );
    if ( aplayer == NULL )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayConditionEvaluator: playerAtPosition(%s,%d,(%.2f,%.2f)) = FALSE (Could not find player reference)",
                      player.affiliation() == setplay::TA_OUR ? "our" : "opp",
                      player.number(),
                      position.x, position.y);
        return false;
    }

    // Calculate my dist to position to infer level of uncertainty
    // double dist_to_player = world->self().pos().dist( position );

    rcsc::Circle2D region(position, ServerParam::i().defaultKickableArea() + 1.0); // Using player types is not working !!!!! Code: player->playerTypePtr()->playerSize()
    bool res = region.contains( aplayer->pos() );  // Code for will be there: || region.contains( player->inertiaPoint(1) );

    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayConditionEvaluator: playerAtPosition(%s,%d,(%.2f,%.2f)) playerPos=(%.2f,%.2f) if within circle((%.2f, %.2f), %.2f) = %s VisibleDistance=%.2f",
                  player.affiliation() == setplay::TA_OUR ? "our" : "opp",
                  player.number(),
                  position.x, position.y,
                  aplayer->pos().x, aplayer->pos().y,
                  region.center().x, region.center().y, region.radius(),
                  res ? "TRUE" : "FALSE",
                  ServerParam::i().visibleDistance());
    return res;
}

bool SetplayConditionEvaluator::ballAtRegion( const rcsc::Region2D& region )
{
    const rcsc::WorldModel* world = M_setplay_world->world();
    bool res = region.contains( world->ball().pos() );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayConditionEvaluator: ballAtRegion => BPos(%.2f,%.2f) => %s",
                  world->ball().pos().x, world->ball().pos().y, res ? "TRUE" : "FALSE");
    return res;
}

bool SetplayConditionEvaluator::ballAtPosition( const rcsc::Vector2D& position )
{
    const rcsc::WorldModel* world = M_setplay_world->world();
    rcsc::Circle2D region(position, 1.5 );
    bool res = region.contains( world->ball().pos() );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayConditionEvaluator: ballAtPosition(%.2f,%.2f) => BPos(%.2f,%.2f) radius=%.2f => %s",
                  position.x, position.y,
                  world->ball().pos().x, world->ball().pos().y,
                  ServerParam::i().ballSize() * 2, res ? "TRUE" : "FALSE");
    return res;
}

bool SetplayConditionEvaluator::canPassToPlayer( const PlayerAffiliation &player1,
                                                 const PlayerAffiliation &player2)
{
    const rcsc::WorldModel* world = M_setplay_world->world();
    const AbstractPlayerObject* aplayer1 = NULL, *aplayer2 = NULL;

    aplayer1 = player1.affiliation() == setplay::TA_OUR
                ? world->ourPlayer( player1.number() )
                : world->theirPlayer( player1.number() );

    aplayer2 = player2.affiliation() == setplay::TA_OUR
                ? world->ourPlayer( player2.number() )
                : world->theirPlayer( player2.number() );

    // Simplification: Assume that player can pass if he can kick the ball
    return aplayer1 != NULL && aplayer1->isKickable() &&
           aplayer2 != NULL;
}

bool SetplayConditionEvaluator::canPassToRegion( const PlayerAffiliation &player,
                                                 const Region2D &region )
{
    const AbstractPlayerObject* aplayer = M_setplay_world->getAbstractPlayerObject( player );


    // Simplification
    return aplayer != NULL && aplayer->isKickable() && M_setplay_world->world()->countOpponentsIn( region, 3, true ) == 0;
}

bool SetplayConditionEvaluator::canPassToPosition( const setplay::PlayerAffiliation& player,
                                                   const rcsc::Vector2D& position )
{
    return canPassToRegion( player, rcsc::Circle2D(position, ServerParam::i().defaultPlayerSize() * 2));
}

bool SetplayConditionEvaluator::isOffside( const setplay::PlayerAffiliation& player )
{
    const rcsc::WorldModel* world = M_setplay_world->world();
    if ( player.affiliation() == TA_OUR )
    {
        const AbstractPlayerObject* aplayer = world->ourPlayer( player.number() );
        if ( aplayer == NULL )
        {
            return true;
        }
        return aplayer->pos().x > world->offsideLineX()  + 1;
    }
    const AbstractPlayerObject* aplayer = world->ourPlayer( player.number() );
    if ( aplayer == NULL )
    {
        return true;
    }
    return aplayer->pos().x < world->theirDefenseLineX() - 1;
}

bool SetplayConditionEvaluator::canInterceptBall( const setplay::PlayerAffiliation& player )
{
    const rcsc::WorldModel* world = M_setplay_world->world();
    // Check for play-on and non-play on modes
    if ( world->gameMode().type() != rcsc::GameMode::PlayOn )
    {
        // HACK Should also consider wait times of play-modes
        return true;
    }

    bool can_intercept = false;
    if ( player.affiliation() == TA_OUR )
    {
        if ( (int) player.number() == world->self().unum() )
        {
            can_intercept = world->interceptTable()->selfReachCycle() < world->interceptTable()->opponentReachCycle() &&
                            ServerParam::i().pitchRect().contains( world->ball().inertiaPoint( world->interceptTable()->selfReachCycle() ));
        }
        else
        {
            can_intercept = world->interceptTable()->teammateReachCycle() < world->interceptTable()->opponentReachCycle() &&
                            world->interceptTable()->fastestTeammate() != NULL &&
                            world->interceptTable()->fastestTeammate()->unum() == (int) player.number() &&
                            ServerParam::i().pitchRect().contains( world->ball().inertiaPoint( world->interceptTable()->teammateReachCycle() ));
        }
    }
    else
    {
        can_intercept = world->interceptTable()->opponentReachCycle() < world->interceptTable()->teammateReachCycle() &&
                        world->interceptTable()->fastestOpponent() != NULL &&
                        world->interceptTable()->fastestOpponent()->unum() == (int) player.number() &&
                        ServerParam::i().pitchRect().contains( world->ball().inertiaPoint( world->interceptTable()->opponentReachCycle() ));
    }

    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayConditionEvaluator: CanIntercept(%s,%d)=%s",
                  player.affiliation() == TA_OUR ? "our" : "opp",
                  player.number(),
                  can_intercept ? "true" : "false");

    return can_intercept;
}

bool SetplayConditionEvaluator::isBallKickable( const PlayerAffiliation &player)
{
    const rcsc::WorldModel* world = M_setplay_world->world();
    const AbstractPlayerObject* aplayer = player.affiliation() == TA_OUR
                                            ? world->ourPlayer( player.number() )
                                            : world->theirPlayer( player.number() );

    bool is_kickable = aplayer != NULL && aplayer->isKickable();

    if ( aplayer == NULL )
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayConditionEvaluator: IsBallKickable(%s,%d)=UNKNOWN BPos(%.2f,%.2f) PPos=Unknown playerRef=NULL",
                      player.affiliation() == TA_OUR ? "our" : "opp",
                      player.number(),
                      world->ball().pos().x, world->ball().pos().y);
    }
    else
    {
        dlog.addText( Logger::PLAN,
                      __FILE__ " SetplayConditionEvaluator: IsBallKickable(%s,%d)=%s BPos(%.2f,%.2f) PPos(%.2f,%.2f)",
                      player.affiliation() == TA_OUR ? "our" : "opp",
                      player.number(),
                      is_kickable ? "true" : "false",
                      world->ball().pos().x, world->ball().pos().y,
                      aplayer->pos().x, aplayer->pos().y);
    }

    return is_kickable;
}

bool SetplayConditionEvaluator::ownsBall( const setplay::PlayerAffiliation& player)
{
    bool owns_ball = isBallKickable( player ) ||
                     canInterceptBall( player );
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayConditionEvaluator: OwnsBall(%s,%d)=%s",
                  player.affiliation() == TA_OUR ? "our" : "opp",
                  player.number(),
                  owns_ball ? "true" : "false");
    return owns_ball;
}

bool SetplayConditionEvaluator::isTeamInPossession ( const setplay::TeamAffiliation& affiliation)
{
    bool res = affiliation == M_setplay_world->teamInPossession();
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayConditionEvaluator: TeamInPossession %s == CurrentTeamInPossession %s => %s",
                  affiliation == setplay::TA_OUR ? "our" : "opp",
                  M_setplay_world->teamInPossession() == setplay::TA_OUR ? "our" : "opp",
                  res ? "true" : "false");
    return res;
}

bool SetplayConditionEvaluator::isGameSituation( const setplay::GameSituation& game_situation)
{
    bool res = game_situation == M_setplay_world->gameSituation();
    dlog.addText( Logger::PLAN,
                  __FILE__ " SetplayConditionEvaluator: GameSituation %s == CurrentGameSituation %s => %s",
                  setplay::GameSituation::gameSituation( game_situation ).c_str(),
                  setplay::GameSituation::gameSituation( M_setplay_world->gameSituation() ).c_str(),
                  res ? "true" : "false");
    return res;
}
