#include "step_transition_message.h"


namespace setplay {

    StepTransitionMessage::StepTransitionMessage(const PlayerAffiliation &sender,
                                                 unsigned int time,
                                                 unsigned int setplay_id,
                                                 unsigned int current_step_id, unsigned int next_step_id,
                                                 bool inverted)
        : IMessage(sender, time, setplay_id, inverted)
        , M_current_step_id(current_step_id)
        , M_next_step_id(next_step_id)
    {

    }

    StepTransitionMessage::StepTransitionMessage(const StepTransitionMessage& other)
        : IMessage(other)
        , M_current_step_id(other.M_current_step_id)
        , M_next_step_id(other.M_next_step_id)
    {

    }

    StepTransitionMessage::~StepTransitionMessage()
    {

    }

    MessageType StepTransitionMessage::type() const
    {
        return MSG_STEP_TRANSITION;
    }

    unsigned int StepTransitionMessage::currentStepId() const
    {
        return M_current_step_id;
    }

    unsigned int StepTransitionMessage::nextStepId() const
    {
        return M_next_step_id;
    }

    void StepTransitionMessage::setCurrentStepId( unsigned int current_step_id )
    {
        M_current_step_id = current_step_id;
    }

    void StepTransitionMessage::setNextStepId( unsigned int next_step_id )
    {
        M_next_step_id = next_step_id;
    }


    void StepTransitionMessage::getProcessed(IMessageProcessor &processor) const
    {
        processor.process(*this);
    }


    StepTransitionMessage* StepTransitionMessage::clone() const
    {
        return new StepTransitionMessage( *this );
    }

    void StepTransitionMessage::print(std::ostream& os) const
    {
        os << "StepTransitionMessage" << std::endl;
        IMessage::print(os);
        os << std::endl;
        os << "FromStep: " << M_current_step_id << std::endl;
        os << "ToStep: " << M_next_step_id << std::endl;
    }
}
