#include "setplay_generator_sexpr.h"

#include <iomanip>

#include <setplay/model/setplay.h>
#include <setplay/model/actions.h>
#include <setplay/model/conditions.h>
#include <setplay/model/transitions.h>
#include <setplay/model/directives.h>
#include <setplay/model/numbers.h>
#include <setplay/model/geom.h>
#include <setplay/model/assign.h>
#include <setplay/model/types.h>

namespace setplay {

    // Initialize map of playmode to string
    /*static const std::map<PlayMode, std::string> SetplayWriterSExpr::playmode_to_str = {
        std::pair<PlayMode, std::string>(BeforeKickOff, "bko"),
        std::pair<PlayMode, std::string>(TimeOver, "time_over"),
        std::pair<PlayMode, std::string>(PlayOn, "play_on"),
        std::pair<PlayMode, std::string>(KickOff_Our, "ko_our"),
        std::pair<PlayMode, std::string>(KickOff_Opp, "ko_opp"),
        std::pair<PlayMode, std::string>(FreeKick_Our, "fk_our"),
        std::pair<PlayMode, std::string>(FreeKick_Opp, "fk_opp"),
        std::pair<PlayMode, std::string>(Ind_FreeKick_Our, "ifk_our"),
        std::pair<PlayMode, std::string>(Ind_FreeKick_Opp, "ifk_opp"),
        std::pair<PlayMode, std::string>(CornerKick_Our, "ck_our"),
        std::pair<PlayMode, std::string>(CornerKick_Opp, "ck_opp"),
        std::pair<PlayMode, std::string>(GoalKick_Our, "gk_our"),
        std::pair<PlayMode, std::string>(GoalKick_Opp, "gk_opp"),
        std::pair<PlayMode, std::string>(GoalieCatch_Our, "gc_our" ),
        std::pair<PlayMode, std::string>(GoalieCatch_Opp, "gc_opp"),
        std::pair<PlayMode, std::string>(AfterGoal_Our, "ag_our" ),
        std::pair<PlayMode, std::string>(AfterGoal_Opp, "ag_opp" )
    };*/


    SetplayGeneratorSexpr::SetplayGeneratorSexpr(std::ostream& os)
        : M_os(os)
    {

    }

    void SetplayGeneratorSexpr::visit(Setplay& setplay)
    {
        // Write the setplay
        M_os << "(setplay"
             << " :name " << setplay.name()
             << " :id "<< setplay.id();

        M_os << " :invertible ";
        visitBool( setplay.invertible() );

        // Write parameters
        if ( setplay.hasParameters() )
        {
            M_os << " :parameters ";
            if ( setplay.parameters()->size() == 1 )
            {

            }
            else
            {
                M_os << " (list";
                for( Parameters::const_iterator itp = setplay.parameters()->begin();
                     itp != setplay.parameters()->end(); itp++)
                {
                    M_os << " ";
                    Variable* variable = (*itp).second;
                    variable->visit( *this );
                }
                // End the list of parameters
                M_os << ")";
            }
        }

        // Write players
        M_os << " ";
        visitPlayers( "players", setplay.players() );

        // Write abortCond
        if ( setplay.hasAbortCondition() )
        {
            M_os << " ";
            visitCondition( "abortCond" , setplay.abortCondition() );
        }

        // Write steps
        if ( setplay.hasSteps() )
        {
            M_os << " :steps ";
            if ( setplay.steps()->size() == 1)
            {
                setplay.steps()->at(0)->visit(*this);
            }
            else
            {
                M_os << "(seq";
                for( Steps::const_iterator its = setplay.steps()->begin();
                     its != setplay.steps()->end(); its++)
                {
                    M_os << " ";
                    (*its).second->visit( *this );
                }
                // End of list of steps
                M_os << ")";
            }
        }

        // End of set-play
        M_os << ")" ;
    }

    void SetplayGeneratorSexpr::visit(Step& step)
    {
        M_os << "(step :id " << step.id();

        if ( step.hasStartCondition() )
        {
            M_os << " ";
            visitCondition( "condition", step.startCondition() );
        }

        if ( step.hasAbortCondition() )
        {
            M_os << " ";
            visitCondition( "abortCond", step.abortCondition() );
        }

        if ( step.hasAbortCondition() )
        {
            M_os << " ";
            visitCondition( "waitCond", step.waitCondition() );
        }

        if ( step.numParticipations() > 0)
        {
            M_os << " :participants ";
            if ( step.numParticipations() == 1 )
            {
                step.participations()->at(0)->visit( *this );
            }
            else
            {
                M_os << "(list";
                for ( Participations::const_iterator itp = step.participations()->begin();
                      itp != step.participations()->end(); itp++)
                {
                    M_os << " ";
                    (*itp)->visit( *this );
                }
                // End of participations list
                M_os <<")";
            }
        }

        // LeadPlayer
        if( step.hasLeader() )
        {
            M_os <<" :leadPlayer ";
            step.leader()->visit( *this );
        }

        // Transitions
        if ( step.numTransitions() > 0 )
        {
            M_os <<" :transitions ";
            if (step.transitions()->size() == 1)
            {
                step.transitions()->at(0)->visit( *this );
            }
            else
            {
                M_os << "(list";
                for( Transitions::const_iterator itt = step.transitions()->begin();
                     itt != step.transitions()->end(); itt++)
                {
                    M_os << " ";
                    (*itt)->visit( *this );
                }
                M_os << ")";
            }
        }

        M_os << ")"; // END OF STEP
    }

    void SetplayGeneratorSexpr::visit( IParticipation& participation)
    {
        if ( participation.hasLocation() )
        {
            M_os << "(at ";
            if ( participation.player()->type() != TT_VAR_PLAYER )
            {
                M_os << ":player ";
            }
            participation.player()->visit( *this );
            M_os << " :location ";
            participation.location()->visit( *this );
            M_os << ")";
        }
        else
        {
            if ( participation.player()->type() != TT_VAR_PLAYER )
            {
                M_os << ":player ";
            }
            participation.player()->visit( *this );
        }
    }

    /// Transitions
    void SetplayGeneratorSexpr::visit(NextStep& transition)
    {
        M_os << "(nextStep :id " << transition.nextStepId();

        if ( transition.hasStartCondition() )
        {
            M_os << " ";
            visitCondition( "condition", transition.startCondition());
        }

        if ( transition.hasAbortCondition() )
        {
            M_os << " ";
            visitCondition( "abortCond", transition.abortCondition());
        }

        if ( transition.hasDirectives() )
        {
            M_os << " :directives ";
            visitDirectives( transition.directives() );
        }

        // End of transition
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(Finish& transition)
    {
        M_os << "(finish";

        if ( transition.hasStartCondition() )
        {
            M_os << " ";
            visitCondition( "condition", transition.startCondition());
        }

        if ( transition.hasDirectives() )
        {
            M_os << " :directives ";
            visitDirectives( transition.directives() );
        }

        // End of transition
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(Abort& transition)
    {
        M_os << "(abort";

        if ( transition.hasStartCondition() )
        {
            visitCondition( "condition", transition.startCondition());
        }

        if ( transition.hasDirectives() )
        {
            M_os << " :directives ";
            visitDirectives( transition.directives() );
        }

        // End of transition
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(Do& directive)
    {
        visitDirective( "do", directive);
    }

    void SetplayGeneratorSexpr::visit( DoWhile& directive)
    {
        M_os << "(doWhile";
        visitCondition(":condition", directive.condition() );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(Dont& directive)
    {
        visitDirective( "dont", directive );
    }


    void SetplayGeneratorSexpr::visit(ActDribble& action)
    {
        M_os << "(dribble";

        if ( action.location() != NULL)
        {
            M_os << " :location ";
            action.location()->visit( *this );
        }
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(ActFollow& action)
    {
        M_os << "(follow";

        /// Target
        if ( action.target() != NULL )
        {
            M_os << " :target ";
            action.target()->visit( *this );
        }

        /// Displacement
        if (action.displacement() != NULL)
        {
            M_os << " :displacement ";
            action.displacement()->visit( *this );
        }

        M_os << ")";
    }


    void SetplayGeneratorSexpr::visit(ActHold&)
    {
        M_os << "(hold)";
    }

    void SetplayGeneratorSexpr::visit(ActIntercept& action)
    {
        M_os << "(intercept";
        if ( action.target() != NULL )
        {
            M_os << " :target ";
            action.target()->visit( *this );
        }
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(ActPass& action)
    {
        M_os << "(pass";

        if ( action.location() != NULL )
        {
            M_os << " :to ";
            action.location()->visit( *this );
        }
        // TODO Ball move type

        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(ActPos& action)
    {
        M_os << "(pos";

        if ( action.location() != NULL )
        {
            M_os << " :location ";
            action.location()->visit( *this );
        }

        M_os << ")";
    }


    void SetplayGeneratorSexpr::visit(ActShoot& )
    {
        M_os << "(shoot)";
    }

    void SetplayGeneratorSexpr::visit( ActLookAt& action)
    {
        M_os << "(lookAt ";
        visitPlayers("players", &action.players() );
        M_os << ")";
    }


    /// Conditions


    /// CondAnd
    void SetplayGeneratorSexpr::visit(CondAnd& condition)
    {
        M_os << "(and ";
        visitConditions( condition.conditions() );
        M_os << ")";
    }

    /// CondOr
    void SetplayGeneratorSexpr::visit(CondOr& condition)
    {
        M_os << "(or ";
        visitConditions( condition.conditions() );
        M_os << ")";
    }


    /// CondNot
    void SetplayGeneratorSexpr::visit(CondNot& condition)
    {
        M_os << "(not ";
        condition.condition()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(CondBallOwner& condition)
    {
        M_os << "(bowner :player ";
        condition.player()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(CondTeamInPossession& condition)
    {
        M_os << "(teamInPossession :team ";
        visitTeamAffiliation( condition.teamInPossession() );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(CondBool& condition)
    {
        M_os << "("; // Parenthesis are inconsistent and should not be used
        visitBool( condition.state() );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(CondGameSituation& condition)
    {
        M_os << "(gameSituation ";
        visitGameSituation( condition.gameSituation() );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(CondAtLocation& condition)
    {
        M_os << "(objectAtLocation";
        M_os << " :object ";
        condition.object()->visit( *this );
        M_os << " :location ";
        condition.location()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(CondTeamFormation& condition)
    {
        M_os << "(teamFormation ";
        M_os << " :formation ";
        M_os << condition.formation(); // TODO Write formation string
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(CondOffside& condition)
    {
        M_os << "(offside ";
        M_os << " :player ";
        condition.player()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(CondCompDecimal& condition)
    {
        M_os << "(compDecimal ";
        M_os << condition.operationType(); // TODO Write operation type
        M_os << " ";
        condition.operandLeft()->visit( *this );
        M_os << " ";
        condition.operandRight()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(CondKickable& condition)
    {
        M_os << "(kickable ";
        M_os << " :player ";
        condition.player()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( CondCanIntercept& condition)
    {
        M_os << "(canIntercept";
        M_os << " :player ";
        condition.player()->visit( *this );
        M_os << " :object ";
        condition.mobileObject()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( CondCanShoot& condition)
    {
        M_os << "(canShoot";
        M_os << " :player ";
        condition.player()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( CondCanPass& condition)
    {
        M_os << "(canPass";
        M_os << " :from ";
        condition.player()->visit( *this );
        M_os << " :to ";
        condition.location()->visit( *this );
        M_os << ")";
    }

    /// Locations
    void SetplayGeneratorSexpr::visit(Rect2D& location)
    {
        M_os << "(rect";
        M_os << " :topLeft ";
        location.topLeft()->visit( *this );
        M_os << " :bottomRight ";
        location.bottomRight()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(Triangle2D& location)
    {
        M_os << "(triangle";
        M_os << " :a ";
        location.a()->visit( *this );
        M_os << " :b ";
        location.b()->visit( *this );
        M_os << " :c ";
        location.c()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(Sector2D& location)
    {
        M_os << "(sector";
        M_os << " :center ";
        location.center()->visit( *this );
        M_os << " :minRadius ";
        location.minRadius()->visit( *this );
        M_os << " :maxRadius ";
        location.maxRadius()->visit( *this );
        M_os << " :startAngle ";
        location.startAngle()->visit( *this );
        M_os << " :endAngle ";
        location.endAngle()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(Circle2D& location)
    {
        M_os << "(circle";
        M_os << " :center ";
        location.center()->visit( *this );
        M_os << " :radius ";
        location.radius()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(Ray2D& location)
    {
        M_os << "(ray";
        M_os << " :origin ";
        location.origin()->visit( *this );
        M_os << " :direction ";
        location.direction()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(Line2D& location)
    {
        M_os << "(line";
        M_os << " :a ";
        location.a()->visit( *this );
        M_os << " :b ";
        location.b()->visit( *this );
        M_os << " :c ";
        location.c()->visit( *this );
        M_os << ")";
    }

    /// Participant
    void SetplayGeneratorSexpr::visit(PlayerAffiliation& participant)
    {
        M_os << "(player";
        M_os << " :team ";
        visitTeamAffiliation( participant.affiliation());
        M_os << " :number ";
        M_os << participant.number();
        M_os << ")";
    }

    /// Player functors
    void SetplayGeneratorSexpr::visit( PlayerSelf& player_functor)
    {
        M_os << "(self)";
    }

    void SetplayGeneratorSexpr::visit( NearestPlayer& player_functor)
    {
        M_os << "(nearestPlayer";
        M_os << " :location ";
        player_functor.location()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( NearestTeammate& player_functor)
    {
        M_os << "(nearestTeammate";
        M_os << " :location ";
        player_functor.location()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( NearestOpponent& player_functor)
    {
        M_os << "(nearestOpponent";
        M_os << " :location ";
        player_functor.location()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( Goalie& player_functor)
    {
        M_os << "(goalie";
        M_os << " :team ";
        visitTeamAffiliation( player_functor.affiliation() );
        M_os << ")";
    }

    /// Points

    void SetplayGeneratorSexpr::visit(Point2D& point)
    {
        M_os << "(pt";
        M_os << " :x ";
        point.x()->visit( *this );
        M_os << " :y ";
        point.y()->visit( *this );
        M_os << ")";
    }

    /// Variables

    void SetplayGeneratorSexpr::visit(Variable& variable)
    {
        visitVariable(variable);
    }

    void SetplayGeneratorSexpr::visit(VarLocation& variable)
    {
        visitVariable(variable);
    }

    void SetplayGeneratorSexpr::visit( VarRegion& variable)
    {
        visitVariable(variable);
    }

    void SetplayGeneratorSexpr::visit( VarPoint& variable)
    {
        visitVariable(variable);
    }

    void SetplayGeneratorSexpr::visit(VarCondition& variable)
    {
        visitVariable(variable);
    }

    void SetplayGeneratorSexpr::visit(VarDecimal& variable)
    {
        visitVariable(variable);
    }

    void SetplayGeneratorSexpr::visit(VarInteger& variable)
    {
        visitVariable(variable);
    }

    void SetplayGeneratorSexpr::visit(VarPlayer& variable)
    {
        visitVariable(variable);
    }

    /// Point Functors
    void SetplayGeneratorSexpr::visit( MobileObjectLocation& point_functor)
    {
        M_os << "(objectLocation";
        M_os << " :object ";
        point_functor.mobileObject()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(PlayerPosition& point_functor)
    {
        M_os << "(playerPosition";
        M_os << " :player ";
        point_functor.player()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( BallPosition& )
    {
        M_os << "(ballPosition)";
    }

    // Vector unary operations
    void SetplayGeneratorSexpr::visit( VectorNormalize& point_functor )
    {
        visit("normVec", point_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorInvertX& point_functor )
    {
        visit("invVecX", point_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorInvertY& point_functor )
    {
        visit("invVecY", point_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorInvertXY& point_functor )
    {
        visit("invVecXY", point_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorSubtraction& point_functor )
    {
        visit("subVec", point_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorAddition& point_functor )
    {
        visit("addVec", point_functor);
    }

    // Vector scalar operations
    void SetplayGeneratorSexpr::visit( VectorScale& point_functor )
    {
        visit("scaleVec", point_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorRotation& point_functor )
    {
        visit("rotateVec", point_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorDivision& point_functor )
    {
        visit("divVec", point_functor);
    }

    void SetplayGeneratorSexpr::visit( NearestIntersectionPoint& point_functor)
    {
        visit("nearestIntersectionPt", point_functor);
    }

    void SetplayGeneratorSexpr::visit( RegionCentroid& point_functor)
    {
        visit("centroid", point_functor);
    }

    /// Basic Data Types
    void SetplayGeneratorSexpr::visit(Decimal& value)
    {
        M_os << setprecision(3) << value.value();
    }

    void SetplayGeneratorSexpr::visit(Integer& value)
    {
        M_os << value.value();
    }
 
    void SetplayGeneratorSexpr::visit(Bool& value)
    {
        M_os << value.value() ? "true" : "false";
    }

    /// Decimal Functors

    // Number operations
    void SetplayGeneratorSexpr::visit( NumberAbs& decimal_functor )
    {
        visit("absNum", decimal_functor);
    }

    void SetplayGeneratorSexpr::visit( NumberSignChange& decimal_functor )
    {
        visit("changeSignNum", decimal_functor);
    }

    void SetplayGeneratorSexpr::visit( NumberMultiplication& decimal_functor )
    {
        visit("mulNum", decimal_functor);
    }

    void SetplayGeneratorSexpr::visit( NumberSubtraction& decimal_functor )
    {
        visit("subNum", decimal_functor);
    }

    void SetplayGeneratorSexpr::visit( NumberDivision& decimal_functor )
    {
        visit("divNum", decimal_functor);
    }

    void SetplayGeneratorSexpr::visit( NumberAddition& decimal_functor )
    {
        visit("addNum", decimal_functor);
    }

    // Vector to scalar operations
    void SetplayGeneratorSexpr::visit( VectorDirection& decimal_functor )
    {
        visit("vecDir", decimal_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorLength& decimal_functor )
    {
        visit("vecLen", decimal_functor);
    }

    // Two vector to scalar operations
    void SetplayGeneratorSexpr::visit( VectorDistance& decimal_functor )
    {
        visit("vecDist", decimal_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorAngularDiff& decimal_functor )
    {
        visit("vecAngDiff", decimal_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorInnerProduct& decimal_functor )
    {
        visit("vecInProd", decimal_functor);
    }

    void SetplayGeneratorSexpr::visit( VectorOuterProduct& decimal_functor )
    {
        visit("vecOutProd", decimal_functor);
    }

    // In game queries

    void SetplayGeneratorSexpr::visit(DefenseLine& decimal_functor)
    {
        M_os << "(defenseLine";
        M_os << " :team ";
        visitTeamAffiliation( decimal_functor.teamAffiliation() );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(OffenseLine& decimal_functor)
    {
        M_os << "(offenseLine";
        M_os << " :team ";
        visitTeamAffiliation( decimal_functor.teamAffiliation() );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(OffsideLine& decimal_functor)
    {
        M_os << "(offsideLine";
        M_os << " :team ";
        visitTeamAffiliation( decimal_functor.teamAffiliation() );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit(MobileObjectSpeed& decimal_functor)
    {
        M_os << "(speed";
        M_os << " :object ";
        decimal_functor.mobileObject()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( PointX& decimal_functor )
    {
        M_os << "(ptX";
        M_os << " :point ";
        decimal_functor.point()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( PointY& decimal_functor )
    {
        M_os << "(ptY";
        M_os << " :point ";
        decimal_functor.point()->visit( *this );
        M_os << ")";
    }


    /// Integer Functors
    void SetplayGeneratorSexpr::visit(CurrentTime& )
    {
        M_os << "(time)";
    }

    void SetplayGeneratorSexpr::visit(RemainingTime& )
    {
        M_os << "(remaningTime)";
    }

    void SetplayGeneratorSexpr::visit( GoalsScored& integer_functor)
    {
        M_os << "(goalsScored";
        M_os << " :team ";
        visitTeamAffiliation( integer_functor.teamAffiliation() );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( GoalDiff& integer_functor)
    {
        M_os << "(goalDiff)";
    }

    void SetplayGeneratorSexpr::visit( InterceptionTime& integer_functor )
    {
        M_os << "(interceptionTime";
        M_os << " :player ";
        integer_functor.player()->visit( *this );
        M_os << " :object ";
        integer_functor.mobileObject()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( Assignment& assign )
    {
        M_os << "(assign ";
        assign.variable()->visit( *this );
        M_os << " ";
        assign.expression()->visit( *this );
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visit( Ball& )
    {
        M_os << "(ball)";
    }

    /// Helper methods implementation

    void SetplayGeneratorSexpr::visitPlayers( const std::string& label,  const vector<IPlayer* >* participants)
    {
        M_os << ":" << label << " ";
        if ( participants->size() ==  1 )
        {
            participants->at(0)->visit( *this );
        }
        else
        {
            M_os << "(list";
            for( Participants::const_iterator itp = participants->begin();
                 itp != participants->end(); itp++ )
            {
                M_os << " ";
                ( *itp )->visit( *this );
            }
            // End of list of players
            M_os << ")";
        }
    }

    void SetplayGeneratorSexpr::visitCondition( std::string label, ICondition* condition )
    {
        M_os << ":" << label << " ";
        condition->visit( *this );
    }

    void SetplayGeneratorSexpr::visitDirectives( Directives* directives )
    {
        if ( directives->size() == 1 )
        {
            directives->at(0)->visit( *this );
        }
        else
        {
            M_os << "(list";
            for ( Directives::const_iterator itd = directives->begin();
                  itd != directives->end(); itd++ )
            {
                M_os << " ";
                (*itd)->visit( *this );
            }
            M_os << ")";
        }
    }

    void SetplayGeneratorSexpr::visitDirective( const std::string& label, IDirective &directive)
    {
        // Sanity check
        assert ( directive.players() != NULL && !directive.players()->empty() &&
                 directive.actions() != NULL && !directive.actions()->empty());

        M_os << "(" << label;

        // Write players
        M_os << " ";
        visitPlayers( "players", directive.players() );

        // Actions to execute
        M_os << " :actions ";
        visitActions( directive.actions() );

        // End of directive
        M_os << ")";
    }

    void SetplayGeneratorSexpr::visitActions( Actions* actions )
    {
        if ( actions->size() == 1 )
        {
            actions->at(0)->visit( *this );
        }
        else
        {
            M_os << "(list";
            for ( Actions::const_iterator ita = actions->begin();
                  ita != actions->end(); ita++)
            {
                M_os << " ";
                (*ita)->visit( *this );
            }
            M_os << ")";
        }
    }

    void SetplayGeneratorSexpr::visitConditions( Conditions* conditions )
    {
        if ( conditions->size() == 1 )
        {
            conditions->at(0)->visit( *this );
        }
        else
        {
            M_os << "(list";
            for ( Conditions::const_iterator itc = conditions->begin();
                  itc != conditions->end(); itc++)
            {
                M_os << " ";
                (*itc)->visit( *this );
            }
            M_os << ")";
        }
    }

    void SetplayGeneratorSexpr::visitTeamAffiliation( TeamAffiliation team_affiliation)
    {
        M_os << ((team_affiliation == TA_OUR) ? "our" : "opp");
    }

    void SetplayGeneratorSexpr::visitBool(bool value)
    {
        M_os << ((value == true) ? "true" : "false");
    }

    void SetplayGeneratorSexpr::visitGameSituation( GameSituation game_situation)
    {
        std::string game_situation_str = GameSituation::gameSituation( game_situation );
        M_os << game_situation_str;
    }

    void SetplayGeneratorSexpr::visitVariable(const Variable& variable)
    {
        M_os << variable.name();
    }


}

