#include "act_follow.h"

#include <setplay/runtime/iaction_executor.h>
#include <setplay/runtime/iaction_evaluator.h>
#include <setplay/model/ivisitor.h>

namespace setplay {

    /// ActFollow

    ActFollow::ActFollow(IMobileObject* target,
                         IPoint2D* displacement)
        : ActIntercept(target)
        , M_displacement(displacement)
    {

    }

    ActFollow::ActFollow(const ActFollow& action)
        : ActIntercept(action)
    {
        M_displacement = action.M_displacement != NULL ? action.M_displacement->clone() : NULL;
    }

    ActFollow::~ActFollow()
    {
        if ( M_displacement != NULL) { delete M_displacement; }
    }

    TokenType ActFollow::type() const
    {
        return TT_ACTION_FOLLOW;
    }

    IAction::ActionType ActFollow::actionType() const
    {
        return AT_FOLLOW;
    }

    // Accessors
    IPoint2D* ActFollow::displacement() const
    {
        return M_displacement;
    }

    ActFollow& ActFollow::operator=(const ActFollow& other)
    {
        if (this != &other)
        {
            setTarget( other.M_target != NULL ? other.M_target->clone() : NULL);
            setDisplacement( other.M_displacement != NULL ? other.M_displacement->clone() : NULL);
        }
        return *this;
    }

    ActFollow& ActFollow::setDisplacement(IPoint2D* point)
    {
        if ( M_displacement != NULL ) { delete M_displacement; }
        M_displacement = point;
        return *this;
    }

    ActFollow& ActFollow::operator*()
    {
        return *this;
    }

    ActFollow* ActFollow::clone() const
    {
       return new ActFollow(*this);
    }

    void ActFollow::visit(IVisitor& visitor)
    {
        visitor.visit( *this );
    }

}
