#include "cond_can_intercept.h"

namespace setplay
{

CondCanIntercept::CondCanIntercept( IPlayer* interceptor,
                                    IMobileObject* interceptee)
    : M_player(interceptor)
    , M_mobile_object(interceptee)
{

}

CondCanIntercept::CondCanIntercept(const CondCanIntercept& other)
{
    M_player = other.M_player != NULL ? other.M_player->clone() : NULL;
    M_mobile_object = other.M_mobile_object != NULL ? other.M_mobile_object->clone() : NULL;
}

CondCanIntercept::~CondCanIntercept()
{
    if ( M_player != NULL ) { delete M_player; }
    if ( M_mobile_object != NULL ) { delete M_mobile_object; }
}

TokenType CondCanIntercept::type() const
{
    return TT_CONDITION_CAN_INTERCEPT;
}

IPlayer* CondCanIntercept::player() const
{
    return M_player;
}

IMobileObject* CondCanIntercept::mobileObject() const
{
    return M_mobile_object;
}

CondCanIntercept& CondCanIntercept::setMobileObject(IMobileObject* mobile_object)
{
    if ( M_mobile_object != NULL ) { delete M_mobile_object; }

    M_mobile_object = mobile_object;
    return *this;
}

CondCanIntercept& CondCanIntercept::setPlayer(IPlayer* player)
{
    if ( M_player != NULL ) { delete M_player; }

    M_player = player;
    return *this;
}

CondCanIntercept* CondCanIntercept::clone() const
{
    return new CondCanIntercept( *this );
}

CondCanIntercept& CondCanIntercept::operator*()
{
    return *this;
}

void CondCanIntercept::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

}
