#include "idirective.h"

#include <cstddef>

namespace setplay {

    IDirective::IDirective(Players* players,
                           Actions* actions)
        : M_players(players)
        , M_actions(actions)
    {
        if ( M_players == NULL) { M_players = new Players(); }
        if ( M_actions == NULL) { M_actions = new Actions(); }
    }

    IDirective::IDirective(const IDirective& other)
    {
        M_players = new Players();
        if ( other.M_players != NULL && !other.M_players->empty())
        {
            for ( Players::iterator itp = other.M_players->begin();
                  itp != other.M_players->end();
                  itp++)
            {
                M_players->push_back( (*itp)->clone());
            }
        }

        M_actions = new vector<IAction*>();
        if ( other.M_actions != NULL && !other.M_actions->empty() )
        {
            for ( vector<IAction*>::iterator ita = other.M_actions->begin();
                  ita != other.M_actions->end();
                  ita++)
            {
                M_actions->push_back( (*ita)->clone());
            }
        }
    }

    IDirective::~IDirective()
    {
        if ( M_players != NULL )
        {
            for ( Players::iterator itp = M_players->begin();
                  itp != M_players->end();
                  itp++)
            {
                delete *itp;
            }
        }

        if ( M_actions != NULL )
        {
            for ( Actions::iterator ita = M_actions->begin();
                  ita != M_actions->end();
                  ita++)
            {
                delete *ita;
            }
        }
    }

    Players* IDirective::players() const
    {
      return M_players;
    }

    Actions* IDirective::actions() const
    {
      return M_actions;
    }

    bool IDirective::hasActions() const
    {
        return M_actions != NULL && !M_actions->empty();
    }

    bool IDirective::hasPlayers() const
    {
        return M_players != NULL && !M_players->empty();
    }

    IDirective& IDirective::addAction( IAction* action)
    {
        M_actions->push_back(action);
        return *this;
    }

    IDirective& IDirective::addPlayer( IPlayer* player)
    {
        if ( M_players == NULL ) { M_players = new Players(); }

        M_players->push_back( player );
        return *this;
    }

    IDirective& IDirective::setPlayers( Players* players )
    {
        if ( M_players != NULL )
        {
            for ( Players::iterator it = M_players->begin(); it != M_players->end(); it++ )
            {
                delete *it;
            }
            delete M_players;
        }
        M_players = players;
        return *this;
    }

    IDirective& IDirective::setActions( Actions* actions )
    {
        if ( M_actions != NULL )
        {
            for ( Actions::iterator it = M_actions->begin(); it != M_actions->end(); it++ )
            {
                delete *it;
            }
            delete M_actions;
        }
        M_actions = actions;
        return *this;
    }
}
