#ifndef SP_EXECUTION_PATH_H
#define SP_EXECUTION_PATH_H

#include <vector>
#include <iostream>

#include <setplay/model/step.h>

using namespace std;

namespace setplay {

class ExecutionPath
{
    public:
        ExecutionPath() { }
        ExecutionPath(const ExecutionPath& sep)
        {
            // Copy existing steps to the new instance
            M_steps.assign( sep.M_steps.begin(), sep.M_steps.end() );
        }

        virtual ~ExecutionPath() { }

        void addStep(Step* step)
        {
            M_steps.push_back(step);
        }

        unsigned int length()
        {
            return M_steps.size();
        }

        bool complete()
        {
            if (M_steps.empty())
            {
                return false;
            }

            bool starts_with_first_step = M_steps.at(0)->is_first();
            Step& last_step = *M_steps.at(M_steps.size()-1);
            std::vector<ITransition*>::iterator it_transition = last_step.transitions()->begin();
            while (it_transition != last_step.transitions()->end() &&
                   (*it_transition)->type() == TT_TRANSITION_NEXT_STEP)
            {
                it_transition++;
            }

            return starts_with_first_step && it_transition != last_step.transitions()->end();
        }

        const std::vector<Step*>& steps() const
        {
            return M_steps;
        }

    private:
        std::vector<Step*> M_steps;
};

inline std::ostream& operator<<( std::ostream& os, ExecutionPath& sep )
{
    typename std::vector<Step*>::const_iterator it = sep.steps().begin();
    while (it != sep.steps().end())
    {
        os << (*it)->id() << ",";
        it++;
    }
    return os;
}

}

#endif // SP_EXECUTION_PATH_H
