#include "game_situation.h"

#include <sstream>
#include <cassert>

using namespace std;

namespace setplay {


const GameSituation& GameSituation::gameSituation(std::string game_situation)
{
    static std::map<std::string, GameSituation> string_to_game_situation = createStringToGameSituationMap();

    map<string, GameSituation>::const_iterator itf = string_to_game_situation.find( game_situation );
    assert (  itf != string_to_game_situation.end());
    return itf->second;
}

const std::string& GameSituation::gameSituation(const GameSituation& game_situation)
{
    static std::map<GameSituation, std::string> game_situation_to_string = createGameSituationToStringMap();

    map<GameSituation, string>::const_iterator itf = game_situation_to_string.find( game_situation );
    assert (  itf != game_situation_to_string.end() );
    return itf->second;
}


std::map<string, GameSituation> GameSituation::createStringToGameSituationMap()
{
    map<string, GameSituation> string_to_game_situation;

    string_to_game_situation.insert(pair<string, GameSituation>("undefined", GameSituation(GameSituation::Undefined)));
    string_to_game_situation.insert(pair<string, GameSituation>("before_kick_off", GameSituation(GameSituation::BeforeKickOff)));
    string_to_game_situation.insert(pair<string, GameSituation>("time_over", GameSituation(GameSituation::TimeOver)));
    string_to_game_situation.insert(pair<string, GameSituation>("first_half_over", GameSituation(GameSituation::FirstHalfOver)));
    string_to_game_situation.insert(pair<string, GameSituation>("over_time", GameSituation(GameSituation::OverTime)));

    string_to_game_situation.insert(pair<string, GameSituation>("play_on_our", GameSituation(GameSituation::PlayOn,
                                                                                             TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("play_on_opp", GameSituation(GameSituation::PlayOn,
                                                                                             TA_OPP)));

    string_to_game_situation.insert(pair<string, GameSituation>("play_on_our_possession", GameSituation(GameSituation::PlayOnInPossession,
                                                                                             TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("play_on_opp_possession", GameSituation(GameSituation::PlayOnInPossession,
                                                                                             TA_OPP)));

    string_to_game_situation.insert(pair<string, GameSituation>("play_on_our_losing_possession", GameSituation(GameSituation::PlayOnLosingPossession,
                                                                                             TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("play_on_opp_losing_possession", GameSituation(GameSituation::PlayOnLosingPossession,
                                                                                             TA_OPP)));

    string_to_game_situation.insert(pair<string, GameSituation>("play_on_our_regaining_possession", GameSituation(GameSituation::PlayOnRegainingPossession,
                                                                                             TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("play_on_opp_regaining_possession", GameSituation(GameSituation::PlayOnRegainingPossession,
                                                                                             TA_OPP)));

    string_to_game_situation.insert(pair<string, GameSituation>("kick_off_our", GameSituation(GameSituation::KickOff,
                                                                                              TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("kick_off_opp", GameSituation(GameSituation::KickOff,
                                                                                              TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("throw_in_our", GameSituation(GameSituation::ThrowIn,
                                                                                              TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("throw_in_opp", GameSituation(GameSituation::ThrowIn,
                                                                                              TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("free_kick_our", GameSituation(GameSituation::FreeKick,
                                                                                               TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("free_kick_opp", GameSituation(GameSituation::FreeKick,
                                                                                               TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("indirect_free_kick_our", GameSituation(GameSituation::IndirectFreeKick,
                                                                                                        TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("indirect_free_kick_opp", GameSituation(GameSituation::IndirectFreeKick,
                                                                                                        TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("corner_kick_our", GameSituation(GameSituation::CornerKick,
                                                                                                 TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("corner_kick_opp", GameSituation(GameSituation::CornerKick,
                                                                                                 TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("goal_kick_our", GameSituation(GameSituation::GoalKick,
                                                                                               TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("goal_kick_opp", GameSituation(GameSituation::GoalKick,
                                                                                               TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("goalie_catch_our", GameSituation(GameSituation::GoalieCatch,
                                                                                                  TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("goalie_catch_opp", GameSituation(GameSituation::GoalieCatch,
                                                                                                  TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("goal_our", GameSituation(GameSituation::Goal,
                                                                                          TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("goal_opp", GameSituation(GameSituation::Goal,
                                                                                          TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("offside_our", GameSituation(GameSituation::OffSide,
                                                                                             TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("offside_opp", GameSituation(GameSituation::OffSide,
                                                                                             TA_OPP)));

    string_to_game_situation.insert(pair<string, GameSituation>("penalty_kick_our", GameSituation(GameSituation::PenaltyKick,
                                                                                                  TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("penalty_kick_opp", GameSituation(GameSituation::PenaltyKick,
                                                                                                  TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("penalty_setup_our", GameSituation(GameSituation::PenaltySetup,
                                                                                                   TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("penalty_setup_opp", GameSituation(GameSituation::PenaltySetup,
                                                                                                   TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("penalty_ready_our", GameSituation(GameSituation::PenaltyReady,
                                                                                                   TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("penalty_ready_opp", GameSituation(GameSituation::PenaltyReady,
                                                                                                   TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("penalty_taken_our", GameSituation(GameSituation::PenaltyTaken,
                                                                                                   TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("penalty_taken_opp", GameSituation(GameSituation::PenaltyTaken,
                                                                                                   TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("penalty_score_our", GameSituation(GameSituation::PenaltyScore,
                                                                                                   TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("penalty_score_opp", GameSituation(GameSituation::PenaltyScore,
                                                                                                   TA_OPP)));

    string_to_game_situation.insert(pair<string, GameSituation>("catch_fault_our", GameSituation(GameSituation::CatchFault
                                                                                                 , TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("catch_fault_opp", GameSituation(GameSituation::CatchFault,
                                                                                                 TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("foul_charge_our", GameSituation(GameSituation::FoulCharge,
                                                                                                 TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("foul_charge_opp", GameSituation(GameSituation::FoulCharge,
                                                                                                 TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("foul_push_our", GameSituation(GameSituation::FoulPush,
                                                                                               TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("foul_push_opp", GameSituation(GameSituation::FoulPush,
                                                                                               TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("foul_multiple_attacker_our", GameSituation(GameSituation::FoulMultipleAttacker,
                                                                                                            TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("foul_multiple_attacker_opp", GameSituation(GameSituation::FoulMultipleAttacker,
                                                                                                            TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("foul_ball_out_our", GameSituation(GameSituation::FoulBallOut,
                                                                                                   TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("foul_ball_out_opp", GameSituation(GameSituation::FoulBallOut,
                                                                                                   TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("back_pass_our", GameSituation(GameSituation::BackPass,
                                                                                               TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("back_pass_opp", GameSituation(GameSituation::BackPass,
                                                                                               TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("free_kick_fault_our", GameSituation(GameSituation::FreeKickFault,
                                                                                                     TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("free_kick_fault_opp", GameSituation(GameSituation::FreeKickFault,
                                                                                                     TA_OPP)));

    /// Legacy PlayModes Robocup - Define aliases?
    string_to_game_situation.insert(pair<string, GameSituation>("bko", GameSituation(GameSituation::BeforeKickOff)));
    string_to_game_situation.insert(pair<string, GameSituation>("time_over", GameSituation(GameSituation::TimeOver)));
    string_to_game_situation.insert(pair<string, GameSituation>("play_on", GameSituation(GameSituation::PlayOn, TA_UNKNOWN )));
    string_to_game_situation.insert(pair<string, GameSituation>("ko_our", GameSituation(GameSituation::KickOff,
                                                                                        TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("ko_our", GameSituation(GameSituation::KickOff,
                                                                                        TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("ki_our", GameSituation(GameSituation::ThrowIn,
                                                                                        TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("ki_opp", GameSituation(GameSituation::ThrowIn,
                                                                                        TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("fk_our", GameSituation(GameSituation::FreeKick,
                                                                                        TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("fk_opp", GameSituation(GameSituation::FreeKick,
                                                                                        TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("ck_our", GameSituation(GameSituation::CornerKick,
                                                                                        TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("ck_opp", GameSituation(GameSituation::CornerKick,
                                                                                        TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("gk_our", GameSituation(GameSituation::GoalKick,
                                                                                        TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("gk_opp", GameSituation(GameSituation::GoalKick,
                                                                                        TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("gc_our", GameSituation(GameSituation::GoalieCatch,
                                                                                        TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("gc_opp", GameSituation(GameSituation::GoalieCatch,
                                                                                        TA_OPP)));
    string_to_game_situation.insert(pair<string, GameSituation>("ag_our", GameSituation(GameSituation::Goal,
                                                                                        TA_OUR)));
    string_to_game_situation.insert(pair<string, GameSituation>("ag_opp", GameSituation(GameSituation::Goal,
                                                                                        TA_OPP)));

    return string_to_game_situation;
}

std::map<GameSituation, string> GameSituation::createGameSituationToStringMap()
{
    map<GameSituation, string> game_situation_to_string;

    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::Undefined), "undefined"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::BeforeKickOff), "before_kick_off"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::TimeOver), "time_over"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FirstHalfOver), "first_half_over"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::OverTime), "over_time"));

    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PlayOn, TA_OUR), "play_on_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PlayOn, TA_OPP), "play_on_opp"));

    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PlayOnInPossession, TA_OUR),
                                                                "play_on_our_possession"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PlayOnInPossession, TA_OPP),
                                                                "play_on_opp_possession"));

    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PlayOnLosingPossession, TA_OUR),
                                                                "play_on_our_losing_possession"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PlayOnLosingPossession, TA_OPP),
                                                                "play_on_opp_losing_possession"));

    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PlayOnRegainingPossession, TA_OUR),
                                                                "play_on_our_regaining_possession"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PlayOnRegainingPossession, TA_OPP),
                                                                "play_on_opp_regaining_possession"));

    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::KickOff, TA_OUR), "kick_off_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::KickOff, TA_OPP), "kick_off_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::ThrowIn, TA_OUR), "throw_in_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::ThrowIn, TA_OPP), "throw_in_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FreeKick, TA_OUR), "free_kick_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FreeKick, TA_OPP), "free_kick_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::IndirectFreeKick, TA_OUR), "indirect_free_kick_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::IndirectFreeKick, TA_OPP), "indirect_free_kick_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::CornerKick, TA_OUR), "corner_kick_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::CornerKick, TA_OPP), "corner_kick_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::GoalKick, TA_OUR), "goal_kick_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::GoalKick, TA_OPP), "goal_kick_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::GoalieCatch, TA_OUR), "goalie_catch_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::GoalieCatch, TA_OPP), "goalie_catch_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::Goal, TA_OUR), "goal_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::Goal, TA_OPP), "goal_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::OffSide, TA_OUR), "offside_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::OffSide, TA_OPP), "offside_opp"));

    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PenaltyKick, TA_OUR), "penalty_kick_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PenaltyKick, TA_OPP), "penalty_kick_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PenaltySetup, TA_OUR), "penalty_setup_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PenaltySetup, TA_OPP), "penalty_setup_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PenaltyReady, TA_OUR), "penalty_ready_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PenaltyReady, TA_OPP), "penalty_ready_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PenaltyTaken, TA_OUR), "penalty_taken_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PenaltyTaken, TA_OPP), "penalty_taken_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PenaltyScore, TA_OUR), "penalty_score_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::PenaltyScore, TA_OPP), "penalty_score_opp"));

    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::CatchFault, TA_OUR), "catch_fault_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::CatchFault, TA_OPP), "catch_fault_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FoulCharge, TA_OUR), "foul_charge_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FoulCharge, TA_OPP), "foul_charge_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FoulPush, TA_OUR), "foul_push_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FoulPush, TA_OPP), "foul_push_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FoulMultipleAttacker, TA_OUR), "foul_multiple_attacker_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FoulMultipleAttacker, TA_OPP), "foul_multiple_attacker_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FoulBallOut, TA_OUR), "foul_ball_out_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FoulBallOut, TA_OPP), "foul_ball_out_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::BackPass, TA_OUR), "back_pass_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::BackPass, TA_OPP), "back_pass_opp"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FreeKickFault, TA_OUR), "free_kick_fault_our"));
    game_situation_to_string.insert(pair<GameSituation, string>(GameSituation(GameSituation::FreeKickFault, TA_OPP), "free_kick_fault_opp"));

    return game_situation_to_string;
}

bool GameSituation::operator==(const GameSituation& other) const
{
    return this->M_type == other.M_type &&
           this->M_team_affiliation == other.M_team_affiliation;
}

bool GameSituation::operator!=(const GameSituation& other) const
{
    return ! ( *this == other );
}

bool GameSituation::operator<(const GameSituation& rhs) const
{
    return M_type < rhs.M_type;
}


GameSituation& GameSituation::operator=(const GameSituation& rhs)
{
    if ( this == &rhs)
        return *this;

    M_type = rhs.M_type;
    M_team_affiliation = rhs.M_team_affiliation;

    return *this;
}

bool GameSituation::isPlayOn() const
{
    return M_type == PlayOnInPossession ||
           M_type == PlayOnLosingPossession ||
           M_type == PlayOnRegainingPossession;
}


bool GameSituation::isPenaltyShootout() const
{
    return M_type == PenaltyKick  ||
           M_type == PenaltySetup ||
           M_type == PenaltyReady ||
           M_type == PenaltyTaken ||
           M_type == PenaltyMiss  ||
           M_type == PenaltyScore;
}

bool GameSituation::isGameStopped() const
{
    return ! isPlayOn() && !isPenaltyShootout();
}

void GameSituation::print(std::ostream& os) const
{
    os << "GameSituation: "
       << gameSituation(M_type)
       << ( M_team_affiliation == TA_OUR ? "our" : "opp" );
}


GameSituation::GameSituation( GameSituation::GameSituationType type, TeamAffiliation team_affiliation)
    : M_type( type )
    , M_team_affiliation( team_affiliation )
{

}

GameSituation::GameSituation( const GameSituation& other)
    : M_type( other.M_type )
    , M_team_affiliation( other.M_team_affiliation)
{

}

GameSituation::GameSituationType GameSituation::type() const
{
    return M_type;
}

TeamAffiliation GameSituation::team_affiliation() const
{
    return M_team_affiliation;
}


}

std::ostream& operator<<( std::ostream& os,
                          const setplay::GameSituation& game_situation)
{
    game_situation.print(os);
    return os;
}
