#include "line_2d.h"

#include <setplay/model/numbers/number_var.h>
#include <setplay/model/ivisitor.h>

namespace setplay {

    Line2D::Line2D(IDecimal* a, IDecimal* b, IDecimal* c)
        : M_a(a)
        , M_b(b)
        , M_c(c)
    {

    }

    Line2D::~Line2D()
    {
        if ( M_a != NULL ) { delete M_a; }
        if ( M_b != NULL ) { delete M_b; }
        if ( M_c != NULL ) { delete M_c; }
    }

    Line2D::Line2D(const Line2D& other)
    {
        M_a = other.M_a != NULL ? M_a->clone() : NULL;
        M_b = other.M_b != NULL ? M_b->clone() : NULL;
        M_c = other.M_c != NULL ? M_c->clone() : NULL;
    }

    TokenType Line2D::type() const
    {
        return TT_LINE;
    }

    ILocation2D::LocationType Line2D::locationType() const
    {
        return LT_LINE;
    }

    Line2D& Line2D::operator*()
    {
        return *this;
    }

    Line2D* Line2D::clone() const
    {
        return new Line2D(*this);
    }

    IDecimal* Line2D::a() const
    {
        return M_a;
    }

    IDecimal* Line2D::b() const
    {
        return M_b;
    }

    IDecimal* Line2D::c() const
    {
        return M_c;
    }

    Line2D& Line2D::setA(IDecimal* value)
    {
        if ( M_a != NULL ) { delete M_a; }
        M_a = value;
        return *this;
    }

    Line2D& Line2D::setB(IDecimal* value)
    {
        if ( M_b != NULL ) { delete M_b; }
        M_b = value;
        return *this;
    }

    Line2D& Line2D::setC(IDecimal* value)
    {
        if ( M_c != NULL ) { delete M_c; }
        M_c = value;
        return *this;
    }

    Line2D& Line2D::operator=(const Line2D& other)
    {
        if ( this != &other )
        {
            setA( other.M_a != NULL ? M_a->clone() : NULL );
            setB( other.M_b != NULL ? M_b->clone() : NULL );
            setC( other.M_c != NULL ? M_c->clone() : NULL );
        }
        return *this;
    }


    void Line2D::visit(IVisitor& visitor)
    {
        visitor.visit( *this );
    }
}
