#include "point_2d.h"

namespace setplay
{

Point2D::Point2D(IDecimal* x, IDecimal* y)
    : M_x(x)
    , M_y(y)
{

}

// Define a copy constructor
Point2D::Point2D(const Point2D& other)
{
    M_x = other.M_x != NULL ? other.M_x->clone() : NULL;
    M_y = other.M_y != NULL ? other.M_y->clone() : NULL;
}

Point2D::~Point2D()
{
    if ( M_x != NULL ) { delete M_x; }
    if ( M_y != NULL ) { delete M_y; }
}

TokenType Point2D::type() const
{
    return TT_POINT;
}

IDecimal* Point2D::x() const
{
    return M_x;
}

IDecimal* Point2D::y() const
{
    return M_y;
}

Point2D& Point2D::setX(IDecimal* value)
{
    if ( M_x != NULL ) { delete M_x; }
    M_x = value;
    return *this;
}

Point2D& Point2D::setY(IDecimal* value)
{
    if ( M_y != NULL ) { delete M_y; }
    M_y = value;
    return *this;
}

Point2D& Point2D::operator=(const Point2D& other)
{
    if ( this != &other )
    {
        setX( other.M_x != NULL ? other.M_x->clone() : NULL );
        setY( other.M_y != NULL ? other.M_y->clone() : NULL );
    }
    return *this;
}

Point2D& Point2D::operator*()
{
    return *this;
}

Point2D* Point2D::clone() const
{
    return new Point2D( *this );
}

void Point2D::visit(IVisitor& visitor)
{
    visitor.visit( *this );
}

}
