#include "functor_player.h"

namespace setplay
{

/// NearestPlayer Implementation
PlayerSelf::PlayerSelf()
{

}

PlayerSelf::~PlayerSelf()
{

}

TokenType PlayerSelf::type() const
{
    return TT_PLAYER_SELF;
}

PlayerSelf* PlayerSelf::clone() const
{
    return new PlayerSelf();
}

IPlayer::PlayerType PlayerSelf::playerType() const
{
    return PT_PlayerSelf;
}

void PlayerSelf::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

/// NearestPlayer Implementation

NearestPlayer::NearestPlayer(ILocation2D* location)
    : M_location(location)
{

}

NearestPlayer::NearestPlayer(const NearestPlayer& other)
{
    M_location = other.M_location != NULL
                    ? other.M_location->clone()
                    : NULL;
}

NearestPlayer::~NearestPlayer()
{
    if ( M_location != NULL )
    {
        delete M_location;
    }
}

ILocation2D* NearestPlayer::location() const
{
    return M_location;
}

TokenType NearestPlayer::type() const
{
    return TT_NEAREST_PLAYER;
}

IPlayer::PlayerType NearestPlayer::playerType() const
{
    return PT_NearestPlayer;
}

void NearestPlayer::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

NearestPlayer* NearestPlayer::clone() const
{
    return new NearestPlayer( *this );
}

/// NearestTeammate Implementation

NearestTeammate::NearestTeammate(ILocation2D* location)
    : NearestPlayer(location)
{

}

NearestTeammate::NearestTeammate(const NearestTeammate& other)
    : NearestPlayer(other)
{

}

NearestTeammate::~NearestTeammate()
{

}

TokenType NearestTeammate::type() const
{
    return TT_NEAREST_TEAMMATE;
}

IPlayer::PlayerType NearestTeammate::playerType() const
{
    return PT_NearestTeammate;
}

void NearestTeammate::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

NearestTeammate* NearestTeammate::clone() const
{
    return new NearestTeammate( *this );
}

/// NearestTeammate Implementation

NearestOpponent::NearestOpponent(ILocation2D *location)
    : NearestPlayer(location)
{

}

NearestOpponent::NearestOpponent(const NearestOpponent& other)
    : NearestPlayer(other)
{

}

NearestOpponent::~NearestOpponent()
{

}

TokenType NearestOpponent::type() const
{
    return TT_NEAREST_OPPONENT;
}

IPlayer::PlayerType NearestOpponent::playerType() const
{
    return PT_NearestOpponent;
}

void NearestOpponent::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

NearestOpponent* NearestOpponent::clone() const
{
    return new NearestOpponent( *this );
}

/// Goalie Implementation

Goalie::Goalie(TeamAffiliation team_affiliation)
    : M_affiliation(team_affiliation)
{

}

Goalie::Goalie(const Goalie& other)
    : M_affiliation(other.M_affiliation)
{

}

Goalie::~Goalie()
{

}

const TeamAffiliation& Goalie::affiliation() const
{
    return M_affiliation;
}

TokenType Goalie::type() const
{
    return TT_GOALIE;
}

IPlayer::PlayerType Goalie::playerType() const
{
    return PT_Goalie;
}

void Goalie::visit(IVisitor &visitor)
{
    visitor.visit( *this );
}

Goalie* Goalie::clone() const
{
    return new Goalie( *this );
}

}
