#include <setplay/model/participants/player_affiliation.h>

#include <cassert>
#include <sstream>

#include <setplay/model/ivisitor.h>

namespace setplay {

    struct PlayerReferenceComparator {
      bool operator()(const IPlayer* p1, const IPlayer* p2) const
      {
          // p1 and p2 cannot be NULL
          assert (p1 != NULL && p2 != NULL);

          if (!p2->type() == TT_PLAYER_AFFILIATION) {  return true; }
          if (!p1->type () == TT_PLAYER_AFFILIATION) { return false; } // p2 is not role
          // p1 and p2 are playerIDs
          // TA_OUR team comes before TA_OPP
          const PlayerAffiliation* pid1 = static_cast<const PlayerAffiliation*>( p1 );
          const PlayerAffiliation* pid2 = static_cast<const PlayerAffiliation*>( p2 );
          if ( pid1->affiliation() == pid2->affiliation() )
          {
              return pid1->number() <= pid2->number();
          }

          // Teams are different
          if ( pid1->affiliation() == TA_OUR)
          {
              return true;
          }

          return false;
      }
    };

    PlayerAffiliation::PlayerAffiliation(unsigned int number, TeamAffiliation affiliation)
        : M_affiliation(affiliation)
        , M_number( number )
    {

    }

    PlayerAffiliation::PlayerAffiliation(const PlayerAffiliation& other)
        : M_affiliation( other.M_affiliation )
        , M_number( other.M_number )
    {

    }

    PlayerAffiliation::~PlayerAffiliation()
    {

    }

    TeamAffiliation PlayerAffiliation::affiliation() const
    {
        return M_affiliation;
    }

    unsigned int PlayerAffiliation::number() const
    {
        return M_number;
    }

    TokenType PlayerAffiliation::type() const
    {
        return TT_PLAYER_AFFILIATION;
    }

    IPlayer::PlayerType PlayerAffiliation::playerType() const
    {
        return IPlayer::PT_PlayerAffiliation;
    }

    bool PlayerAffiliation::operator==(const PlayerAffiliation& other) const
    {
        return M_affiliation == other.M_affiliation &&
               M_number == other.M_number;
    }

    bool PlayerAffiliation::operator!=(const PlayerAffiliation& other) const
    {
        return ! ( *this == other );
    }


    PlayerAffiliation* PlayerAffiliation::clone() const
    {
        return new PlayerAffiliation( *this );
    }

    void PlayerAffiliation::visit(IVisitor& visitor)
    {
        visitor.visit( *this );
    }

}
