#include "next_step.h"

#include <setplay/model/ivisitor.h>

namespace setplay {

        NextStep::NextStep(unsigned int next_step_id,
                           ICondition* abort_condition,
                           ICondition* start_condition,
                           Directives* directives)
            : ITransition(start_condition, directives)
            , M_next_step_id(next_step_id)
            , M_abort_condition(abort_condition)
        {

        }

        NextStep::NextStep(const NextStep&other)
            : ITransition(other)
            , M_next_step_id(other.M_next_step_id)
        {
            M_abort_condition = other.M_abort_condition != NULL
                                    ? other.M_abort_condition->clone()
                                    : NULL;
        }

        NextStep::~NextStep()
        {
            if ( M_abort_condition != NULL )
            {
                delete M_abort_condition;
            }
        }

        TokenType NextStep::type() const
        {
            return TT_TRANSITION_NEXT_STEP;
        }

        ITransition::TransitionType NextStep::transitionType() const
        {
            return ITransition::TT_NEXT_STEP;
        }

        unsigned int NextStep::nextStepId() const
        {
            return M_next_step_id;
        }


        ICondition* NextStep::abortCondition() const
        {
            return M_abort_condition;
        }

        bool NextStep::hasAbortCondition() const
        {
            return M_abort_condition != NULL;
        }

        NextStep& NextStep::setNextStepId(unsigned int next_step_id)
        {
            M_next_step_id = next_step_id;
            return *this;
        }

        NextStep& NextStep::setAbortCondition(ICondition* condition)
        {
            if ( M_abort_condition != NULL ) { delete M_abort_condition; }
            M_abort_condition = condition;
            return *this;
        }

        NextStep& NextStep::operator=(const NextStep& other)
        {
            if ( this != &other )
            {
                M_next_step_id = other.M_next_step_id;
                setStartCondition( other.M_start_condition != NULL ? other.M_start_condition->clone() : NULL );
                setAbortCondition( other.M_abort_condition != NULL ? other.M_abort_condition->clone() : NULL );

                resetDirectives();

                if ( other.M_directives != NULL && !other.M_directives->empty())
                {
                    // TODO Clear existing directives
                    for ( Directives::iterator itd = other.M_directives->begin();
                          itd != other.M_directives->end();
                          itd++)
                    {
                        addDirective( (*itd)->clone() );
                    }
                }
            }
            return *this;
        }

        void NextStep::visit(IVisitor& visitor)
        {
            visitor.visit( *this );
        }

        NextStep* NextStep::clone() const
        {
            return new NextStep( *this );
        }

}
