#ifndef SETPLAYPARSER_H
#define SETPLAYPARSER_H

#include <map>

#include <setplay/readers/isetplay_reader.h>
#include <setplay/model/setplay.h>

namespace setplay {

    class SetplayParserSExpr
            : public ISetplayParser
    {
    public:
        SetplayParserSExpr();

        virtual vector<Setplay>* parse(const string filePath);
        virtual vector<Setplay>* parse(const char* in);

    protected:
        static const string& setplay_element;
        static const string& step_element;
        static const string& next_step_element;
        static const string& abort_transition_element;
        static const string& finish_transition_element;
        static const string& abort_condition_attribute;
        static const string& start_condition_attribute;
        static const string& wait_condition_attribute;
        static const string& wait_time_attribute;
        static const string& abort_time_attribute;
        static const string& condition_attribute;
        static const string& directives_attribute;
        static const string& transitions_attribute;
        static const string& steps_attribute;
        static const string& name_attribute;
        static const string& comment_attribute;
        static const string& id_attribute;
        static const string& invertable_attribute;
        static const string& participations_attribute;
        static const string& participants_attribute;
        static const string& leader_attribute;
        static const string& next_step_id_attribute;
        static const string& next_step_attribute;

        // Participations
        static const string& player_element;
        static const string& player_var;
        static const string& player_number_attribute;
        static const string& player_team_attribute;
        static const string& participation_element;

        // Action elements
        static const string& act_pos_element;
        static const string& act_pass_element;
        static const string& act_dribble_element;
        static const string& act_intercept_element;
        static const string& act_hold_element;
        static const string& act_shoot_element;

        // Directives
        static const string& directive_do;
        // static const string& directive_do = "dont"; // NOT USED EVER => DISREGARD
        static const string& player_attribute;
        static const string& actions_attribute;

        // Regions
        static const string& region_variable_element;
        static const string& region_arc_element; // NOT USED EVER
        static const string& region_triangle_element; // NOT USED EVER
        static const string& region_rect_element;

        // Points
        static const string& point_var_element;
        static const string& point_element;
        static const string& point_ball_element;
        static const string& point_player_element;
        static const string& point_rel_element;
        static const string& point_x_attribute;
        static const string& point_y_attribute;
        // static const string& point_arith_element = ""; // NOT USED EVER

        // Conditions
        static const string& cond_player_pos_element;
        static const string& cond_ball_pos_element;
        static const string& cond_ball_owner_element;
        static const string& cond_playmode_element;
        static const string& cond_and_element;
        static const string& cond_or_element;
        static const string& cond_not_element;

        static const std::map<std::string, PlayMode> str_to_playmode;


        // Helper methods
        int parseStartElementSymbol(const char* in, const string& element_name);
        int parseAttribute(const string& attribute_name, const char* in);
        int parseEndElementSymbol(const char* in);
        int parseInt(const char* in, int& value);
        int parseUint(const char* in, unsigned int& value);
        int parseDouble(const char* in, double& value);
        int parseString(const char* in, string& value);
        int parseBool(const char* in, bool& value);
        int parsePlayMode(const char* in, PlayMode& value);

        virtual int parseSetplay(const char* in, Setplay** out);
        virtual int parseStep(const char* in, Step** out);
        virtual int parseParticipation(const char* in, Participation** out);
        virtual int parseTransition(const char* in, Transition**);
        virtual int parseNextStep(const char* in, NextStep** out);
        virtual int parseAbort(const char* in, Abort** out);
        virtual int parseFinish(const char* in, Finish** out);
        virtual int parseDirective(const char* in, Directive** out);
        virtual int parsePlayer(const char* in, IPlayer** out);
        virtual int parseAction(const char* in, IActionSoccer** out);
        virtual int parseRegion(const char* in, IRegion2D** out);
        virtual int parsePoint(const char* in, IPoint2D** out);
        // IAction* parsePosition(const char* in);
        virtual int parseCondition(const char* in, ICondition* out);
        virtual int parseVariable(const char* in, SetplayVar* out);
    };

}

#endif // SETPLAYPARSER_H
