#ifndef SETPLAY_PARSER_DRIVER_H
#define SETPLAY_PARSER_DRIVER_H

#include <vector>
#include <string>
#include <setplay/parsers/sexpr/setplay_parser_sexpr.hpp>
#include <setplay/model/setplay.h>

namespace setplay {
namespace parsers {

// Forward declare the lexical scanner
class SetplayScannerSexpr;

class SetplayParserSexprDriver
{
public:
    enum State {
        OTHER,
        COMMENT
    };

    SetplayParserSexprDriver(bool trace_scanning = false,
                             bool trace_parsing = false);
    ~SetplayParserSexprDriver();


   SetplayScannerSexpr* lexer() const;

   // Run the parser.  Return 0 on success.
   bool parseStream(std::istream& in);
   bool parseFile(const std::string &filename);
   bool parseString(const std::string& f);

   std::string& file();

   void setState(State state);
   State state() const;


   void add(Setplay* setplay);
   vector<Setplay *>& setplays();

   // Error handling.
   void error(const location& l, const std::string& m);
   void error(const std::string& m);
   void log(const std::string& m);
private:

   // Forward declaration of a scanner
   class SetplayScannerSexpr* M_lexer;
   std::string M_file;
   bool M_trace_scanning;
   bool M_trace_parsing;
   State M_state;
   vector<Setplay*> M_setplays;
};

}
}
#endif // SETPLAY_PARSER_DRIVER_H
