#include "memory_space.h"

#include <cassert>
#include <setplay/generators/sexpr/setplay_generator_sexpr.h>

using namespace std;

namespace setplay
{


MemorySpace::MemorySpace(const std::string& name)
    : M_name(name)
{

}

MemorySpace::~MemorySpace()
{
    reset();
}

const std::string& MemorySpace::name() const
{
    return M_name;
}

const Object* MemorySpace::get(const std::string& name) const
{
    map<string, Object*>::const_iterator it = M_objects.find(name);

    if ( it == M_objects.end() )
    {
        return NULL;
    }
    return it->second;
}

bool MemorySpace::put(const std::string& name, Object* value)
{
    const Object* existing_value = get(name);

    if ( existing_value != NULL )
    {
        return false;
    }

    pair< map<string, Object*>::iterator, bool> res = M_objects.insert(pair<string, Object*>(name, value));

    return res.second;
}


bool MemorySpace::free(const std::string& name)
{
   map<string, Object*>::iterator it = M_objects.find(name);

   if ( it == M_objects.end() )
   {
       return false;
   }
   
   delete it->second;
   M_objects.erase( it );
   return true;
}

void MemorySpace::print(std::ostream& os) const
{
    os << "[MemorySpace=" << M_name << "]" << std::endl;

    SetplayGeneratorSexpr gen(os);
    for ( map<string, Object*>::const_iterator it = M_objects.begin();
          it != M_objects.end(); it++ )
    {
        os << "Name=" << it->first << " ";
        it->second->visit( gen );
        os << std::endl;
    }
}

void MemorySpace::reset()
{
    // Clear the space for all the existing objects
    for ( map<string, Object*>::iterator it = M_objects.begin();
          it != M_objects.end();
          it++ )
    {
        delete it->second;
    }
    M_objects.clear();
}

}

std::ostream& operator<<( std::ostream& os,
                          const setplay::MemorySpace& memory_space)
{
    memory_space.print(os);
    return os;
}
