#ifndef PLAYER_SELECTOR_H
#define PLAYER_SELECTOR_H

#include <map>
#include <string>

#include <setplay/model/setplay.h>
#include <setplay/runtime/setplay_interpreter.h>
#include <setplay/runtime/iworld_model.h>

namespace setplay {

class IPlayerSelector
{
public:
    virtual ~IPlayerSelector() { }

    virtual bool selectPlayers( SetplayInterpreter& interpreter ) = 0;
};

/*!
  \brief Select the players closest to self when no location is specified for them or the ones closest to that location
         regardless of whether players are tires
*/
class PlayersMinGlobalDistanceSelector
        : public IPlayerSelector
{
public:
    PlayersMinGlobalDistanceSelector( IWorldModel* world );
    ~PlayersMinGlobalDistanceSelector();

    bool selectPlayers( SetplayInterpreter& interpreter );

private:
    IWorldModel* M_world;
};

/*!
  \brief Calculate the minimum weight bipartite matching between a set of player variables and concrete players given their positions.
  \abstract The formula to calculate weights between the links is equal to "1 / sqr(dist)"
  \param player_vars A mapping from player variables to their respective positions
  \param concrete_players A mapping from concrete players to their respective positions
  \return Mapping of a player variable to a concrete player
  */
bool minWeightBipartiteMatching( SetplayInterpreter& interpreter,
                                 const std::map<std::string, rcsc::Vector2D>& player_vars,
                                 const std::map<unsigned int, rcsc::Vector2D>& concrete_players);


/*
 * PLAYER SELECTION
 */
// For allocation of players to setplay...
// Numbers in output will start at 1, like normal football...
// Inactive... Not implemented
/*map<std::string, PlayerAffiliation*>* participatingPlayersDistanceFromSelf();
map<std::string, PlayerAffiliation*>*
participatingPlayersDistanceFromPositions();
map<std::string, PlayerAffiliation*>*
participatingPlayersDistanceFromPosAsArg(const vector<const IPoint2D* >* positions );
// LMOTA: TODO
// create this alternative method:
//vector<unsigned int>*
//participatingPlayersGlobalDistanceFromPositions(const IWorldModel& observer) const;
map<std::string, PlayerAffiliation*>*
participatingPlayersGlobalDistanceFromPosAsArg(const vector<const IPoint2D* >* positions );*/

}

#endif // PLAYER_SELECTOR_H
