#ifndef SETPLAY_AUGMENTER_H
#define SETPLAY_AUGMENTER_H

#include <setplay/model/setplay.h>

namespace setplay {

class ISetplayAugmenter
{
public:
    virtual ~ISetplayAugmenter() { }

    /*!
      \brief Augment a set-play definition with additional features based on its interpretation
      \param setplay The setplay definition to augment
      \return True if the set-play was augmented in any way or false otherwise
      */
    virtual bool augment( Setplay* setplay ) = 0;
};



class StepWaitConditionAugmenter
        : public ISetplayAugmenter
{
public:
    StepWaitConditionAugmenter( bool ignore_steps_with_wait_conditions = true );

    bool augment( Setplay* setplay );

    static ICondition* generateStepWaitConditionFromParticipations( const Step* step );
private:
    bool M_ignore_steps_with_wait_conditions;
};

class VirtualTransitionsAugmenter
{
public:
    VirtualTransitionsAugmenter( bool check_bowner_action = true);

    bool augment( Setplay* setplay );

private:
    bool M_check_bowner_action;


    /*!
      \brief Try to create a virtual transition between two steps
      \param from The step from which the transition originates
      \param to The step to which the transition leads
      \returns True if the virtual transition was created or false otherwise
      */
    bool createTransition( const Setplay* setplay,
                                 Step* from,
                           const Step* to );

    /*!
      \brief Check if a player participates in a step
      \param player The player to check for participation
      \param step The step to be checked for participation
      \returns True if the player participates in the step
      */
    const IParticipation* playerStepParticipation( const IPlayer* player, const Step* step );

    /*!
      \brief Check if two players are the same
      \param p1 The first player to compare
      \param p2 The second player to compare
      \returns True if the players are the same or false otherwise
      */
    bool samePlayer( const IPlayer* p1, const IPlayer* p2);

    /*!
      \brief Determine the ball owner in a step
      \param step The step to be checked for participation
      \returns Const reference to the ball owner player in the step or NULL otherwise
      */
    const IParticipation* stepLeaderParticipation( const Step* step );
};


class OffensiveSetplayAbortConditionAugmenter
{
public:
    OffensiveSetplayAbortConditionAugmenter();

    bool augment( Setplay* setplay );
};

}
#endif // SETPLAY_AUGMENTER_H
