#include "setplay_manager_cbr.h"

#include "participants/IPlayerID.h"

#include "communication/init_message.h"
#include "setplay.h"

namespace setplay {


    SetplayManagerCBR::SetplayManagerCBR(const IPlayer& self,
                                                 const string cbrFile,
                                                 bool writeCBRToFile,
                                                 bool writeCBRToFileAfterExecution,
                                                 double weightThisGame,
                                                 double weightThisTeam,
                                                 double weightOtherTeams,
                                                 bool useComm = true,
                                                 bool limitedComm = true,
                                                 unsigned int messageRepeatTime = -1)
     :IExecutor(self, useComm,limitedComm,messageRepeatTime),
      M_case_base(new CaseBase("FCPortugal",cbrFile,writeCBRToFile,
                            writeCBRToFileAfterExecution,
                            weightThisGame,weightThisTeam,
                            weightOtherTeams)),
      M_playmode(Invalid) {
      //DEBUG
      //cerr<<"SP: CBR initial case base:"<<endl;
      //caseBase->write(cerr);
    }

    SetplayManagerCBR::~SetplayManagerCBR(){
      if(M_case_base!=NULL) M_case_base->writeToFile();
    }

    unsigned int
    SetplayManagerCBR::selectSetplayByCBR(vector<CaseBase::SetplayID>& possibleSetplays,
                                              const IContextProvider& observer) const{
      // CBR select
      int chosenSPId=M_case_base->selectSetplay(possibleSetplays,observer);

      //DEBUG
      cerr<<"SP: chosen setplay is nr. "<<chosenSPId<<" in "
          <<possibleSetplays.size()<<endl;

      return chosenSPId;
    }

    void SetplayManagerCBR::autoActivateSetplay(const IContextProvider& observer, const IConditionEvaluator& evaluator){
      // get all valid setplays
      vector<const Setplay*>* feasibleSPs = feasible_setplays(evaluator);

      // Create vector with possible setplay names
      vector<CaseBase::SetplayID>feasibleSPIds;
      for(unsigned int i=0;feasibleSPs!=NULL && i != feasibleSPs->size();i++) // Removed !=0 cond
      for(unsigned int i=0;feasibleSPs!=NULL && i!=feasibleSPs->size();i++)
        feasibleSPIds.push_back(CaseBase::SetplayID(feasibleSPs->at(i)->name(),
                                                    feasibleSPs->at(i)->id()));

      // CBR select
      int chosenSPId = selectSetplayByCBR(feasibleSPIds,observer);

      // Check if no Setplay could be chosen
      if(chosenSPId==10000)
        return;

      const Setplay* chosen_setplay = getSetplay(chosenSPId);

      // Choose participating players
      vector<unsigned int>* participating_player_numbers = participatingPlayersDistanceFromPositions(observer);

      // Activate the setplay with chosen players
      // No other arguments for the time being

      // Get current time
      SetplayStartMessage init_message(observer.time(), chosen_setplay->id(), *participating_player_numbers);
      activate_setplay(init_message, observer);

      //DEBUG
      // cerr<<"SP: Init:"<<messageToSend(observer)->toString() <<endl;

      // for CBR, register start data
      M_ball_initial_position = observer.ball_position();
      M_playmode = observer.play_mode();
    }

    void SetplayManagerCBR::activate_setplay(const SetplayStartMessage& init,
                                                const IContextProvider& observer){
      IExecutor::activate_setplay(init,observer);

      // for CBR, register start data
      M_ball_initial_position = observer.ball_position();
      M_playmode = observer.play_mode();

    }

    void SetplayManagerCBR::handle_setplay_end(const IContextProvider& observer){
      // register CBR
      if(M_case_base!=NULL){
        M_case_base->registerCase(M_active_setplay->name(),
                               M_ball_initial_position,
                               M_playmode,
                               false, // HACK TODO M_active_setplay->wasSuccessful(),
                               observer);
        //DEBUG
        cout<<"SP: "<<M_self.number() <<" setplay with CBR ended. Success: "<<
          false // HACK TODO M_active_setplay->wasSuccessful()
            << " at "<<observer.time()<<endl;
      }

      M_last_setplay_end_time=observer.time();
      M_last_setplay_successful= false; // HACK TODO M_active_setplay->wasSuccessful();
      if(true) { // HACK TODO WAS M_active_setplay->amLeadPlayer()
          M_was_last_step_leader=true;
      }
      delete  M_active_setplay;
      M_active_setplay=NULL;
    }

}
