#ifndef IEVENT_H
#define IEVENT_H

#include <map>
#include <string>

namespace soccer 
{
	namespace event 
	{
		class IEventVisitor;

		class IEvent {
		public:
            enum EventType {
                Null,
                ThrowIn,
                ThrowInTaken,
                KickOff,
                KickOffTaken,
                CornerKick,
                CornerKickTaken,
                FreeKick,
                FreeKickTaken,
                IndirectFreeKick,
                IndirectFreeKickTaken,
                GoalKick,
                GoalKickTaken,
                Goal,
                GoalOpportunity,
                Offside,
                Dribble,
                Pass,
                Shoot,
                BallDispute,
                BallKick,
                BallHold,
                GoalieCatch,
                Interception,
                BallMove,
                PlayerMove,
                PossessionChanged,
                Turnover,
                SoccerMomentChanged,
                GoalPassChain,
                PassChain
            };

			virtual ~IEvent() { }

            unsigned int duration() const;
			virtual unsigned int startTime() const = 0;
			virtual unsigned int endTime() const = 0;

            virtual EventType type() const = 0;

            /// For event comparison
            bool operator<( const IEvent& other )
            {
                if ( startTime() == other.startTime() )
                {
                    return endTime() < other.endTime();
                }

                return startTime() < other.startTime();
            }
			
            virtual void visit(IEventVisitor& visitor) const = 0;
		};


        std::map<IEvent::EventType, std::string> createEventTypeToStringMap();
        std::map<std::string, IEvent::EventType> createStringToEventTypeMap();

        std::string eventType( const IEvent::EventType& event_type );
        IEvent::EventType eventType( const std::string& event_type_str );

	}
}
#endif // IEVENT_H
