#ifndef SOCCER_MOMENT_H
#define SOCCER_MOMENT_H

#include <soccer/events/event.h>
#include <soccer/types.h>

namespace soccer {
namespace event {

class SoccerMomentChanged
        : public Event
{
public:

    class TeamSideMoment
    {
    public:
        enum Moment {
            LOSING_POSSESSION,
            IN_POSSESSION,
            NO_POSSESSION,
            REGAINING_POSSESSION
        };

        static Moment moment( const std::string& moment_str);
        static const std::string& moment( Moment moment );

        TeamSideMoment( TeamSide team_side, Moment moment );
        TeamSideMoment( const TeamSideMoment& other);

        TeamSide teamSide() const;
        Moment moment() const;

        bool operator==( const TeamSideMoment& other);
    private:
        static std::map<Moment, std::string> createMomentToStringMap();
        static std::map<std::string, Moment> createStringToMomentMap();

        TeamSide M_team_side; /// Team side that owns the moment
        Moment M_moment;
    };

    SoccerMomentChanged(unsigned int time, TeamSideMoment team_moment);

    EventType type() const;
    const TeamSideMoment& teamMoment() const;

    void visit(IEventVisitor &visitor) const;
private:
    TeamSideMoment M_team_moment;
};

}
}
#endif // SOCCER_MOMENT_H
