#ifndef IBALL_MOTION_MODEL_H
#define IBALL_MOTION_MODEL_H

#include "ball.h"
#include <rcsc/geom.h>

namespace soccer {

    class IBallMotionModel
    {
    public:
        virtual ~IBallMotionModel() { }

        /*!
          \brief Predict ball physical characteristics after time
          \param time The time the ball is in free motion
          */
        virtual rcsc::Vector2D inertiaTravel(const Ball& ball, unsigned int time) = 0;

        /*!
          \brief Predict ball physical characteristics until it stops
          */
        virtual rcsc::Vector2D inertiaPoint(const Ball& ball, unsigned time) = 0;

        virtual Ball inertia(const Ball& ball, unsigned time) = 0;
        virtual Ball inertia(const Ball& ball) = 0;

        /*!
          \brief Calculate the inertia time for the ball to reach a given point from its current position
          \param ball The reference to the ball
          \param end_position The final position the ball should reach
          \returns The time needed to reach the end position or -1 if it is unreachable
          */
        virtual int inertiaTimeToPoint(const Ball& ball, rcsc::Vector2D end_position) = 0;

        /*!
          \brief Check if the ball in free motion can intercept the line
          \param line The line to be intercepted
          */
        virtual Ball freeMotionIntercept(const Ball& ball, rcsc::Line2D line) = 0;


    };

}

#endif
