/*
 * DMU.cpp
 *
 *  Created on: Mar 26, 2010
 *      Author: LPR, Nima, Nuno 
 */
#include "DMU.h"
#include "Geometry.h"
#include "WorldState.h"
#include "Strategy.h"
#include "Singleton.h"
#include "Logger.h"
#include "Vector3f.h"
#include "Types.h"
#include "Logger.h"
#include "Debug.h"

#include "LogStreams.h"

using namespace std;

DMU::DMU() {
	prevTarget = Vector(2.0,0.0);
	prevEval = -10.0;
	actTarget = prevTarget;
	actEval = prevEval;
	actOri = 0.0; prevOri=0.0;
	// Auto-generated constructor stub
}

DMU::~DMU() {
	// Auto-generated destructor stub
}

// For all angles, for dist=0 to maxdist, 
// calculate final points, calculate weight sum. Select best... (use errors)
Vector DMU::calculateTarget(Vector InitPos, Vector BallPos)
{
	WorldState& world = SWorldState::getInstance();
	Logger& logger = SLogger::getInstance();
	AngleDeg angle;
    float dist;
    float maxOri = 0.0;
    float maxeval = -100.0;
    Vector SelPos = prevTarget;

   logger.LogActionN(30, "Flux: %1.2f Safe %1.2f Easy %1.2f, ", WFlux_Now, WSafe_Now, WEasy_Now);

    for(angle=0.0+world.myGlobalDir; angle<360.0+world.myGlobalDir; ) {
    	Vector orientation = Vector(1.0,0.0).rotate(GetNormalizeAngleDeg(angle));
    	for(dist=2.0; dist<3.01; dist+=0.5) {
        	Vector FinalPos = BallPos + orientation*dist;
			float eval = evaluate(InitPos, BallPos, FinalPos);
			if (eval>maxeval) {maxeval = eval; SelPos = FinalPos; }
			if (eval>1.0)
				logger.LogDrawCircle(30,FinalPos.x+0.1,FinalPos.y-0.1, eval/20.0, "Yellow");
		}

    	if (fabs(GetNormalizeAngleDeg(angle-world.myGlobalDir))< 50.1) angle+=5; else angle+=10;
    }

    float SelPosDir = GetNormalizeAngleDeg((SelPos-InitPos).getDirection());
    float BallDir = GetNormalizeAngleDeg((BallPos-InitPos).getDirection());

 //   if (fabs(GetNormalizeAngleDeg(SelPosDir-BallDir)) < 8.0) {
 //   	actTarget = SelPos;
 //   	actEval = maxeval;
 //   }
    if (maxeval < prevEval+0.5) {
    	maxeval = prevEval;
    	SelPos = prevTarget;
    }

 //   if (BallPos.getDistanceTo(InitPos)>0.5) {  //HACK LPR 2010 HACK not to change target near the ball... change this..
    	actEval = maxeval;
    	float distBall = InitPos.getDistanceTo(BallPos);
    	float factorAct = distBall/50.0;
    	if (factorAct > 0.05) factorAct = 0.05; else if (factorAct <0.01) factorAct=0.01;
    	actTarget = actTarget*(1-factorAct)+SelPos*factorAct; //very low pass filtering LPR 2010 HACK
 //   }


    logger.LogDrawCircle(30,SelPos.x,SelPos.y, 0.25, "Red");
    logger.LogDrawCircle(30,SelPos.x,SelPos.y, 0.3, "Red");
    logger.LogDrawCircle(30,actTarget.x,actTarget.y, 0.35, "Black");

    return actTarget;
}

float DMU::evaluate(Vector InitPos, Vector BallPos, Vector FinalPos)
{
   WorldState& world = SWorldState::getInstance();

   float Value = WFlux_Now*evaluateFlux(InitPos, BallPos, FinalPos)+
                 WSafe_Now*evaluateSafe(InitPos, BallPos, FinalPos)+
                 WEasy_Now*evaluateEasy(InitPos, BallPos, FinalPos);

   if (evaluateSafe(InitPos, BallPos, FinalPos)<1.0) Value = 0.0; //LPR 2010 HACK (not to change dir near ball)

   return Value;
}

Vector fluxToField(Vector FluxPos, Vector FluxSize, Vector FieldSize)
{
  Vector FieldPos;
  FieldPos.x = FieldSize.x*(FluxPos.x/(FluxSize.x-1.0)-0.5);
  FieldPos.y = FieldSize.y*(FluxPos.y/(FluxSize.y-1.0)-0.5);
  return FieldPos;
}

Vector fieldToFlux(Vector FieldPos, Vector FluxSize, Vector FieldSize)
{
  Vector FluxPos;
  FluxPos.x = (FluxSize.x-1.0) * (0.5 + FieldPos.x/FieldSize.x);
  FluxPos.y = (FluxSize.y-1.0) * (0.5 + FieldPos.y/FieldSize.y);
  return FluxPos;
}

float DMU::interpolateFlux(Vector FieldPos, Vector FluxSize, Vector FieldSize)
{
    Logger& logger = SLogger::getInstance();
    WorldState& world = SWorldState::getInstance();
    int Fl = FL_Now;

    if (FieldPos.x > FieldSize.x/2.0-0.5) return -10.0;
    else if (FieldPos.x > FieldSize.x/2.0-1.0) return -5.0;
    if (FieldPos.x < -FieldSize.x/2.0-0.5) return -10.0;
    else if (FieldPos.x < -FieldSize.x/2.0-1.0) return -5.0;
    if (fabs(FieldPos.y) > FieldSize.y/2.0-0.8) return -10.0;
    else if (fabs(FieldPos.y) > FieldSize.y/2.0-1.2) return -5.0;
    else if (fabs(FieldPos.y) > FieldSize.y/2.0-1.5) return -2.0;

    Vector fluxPos = fieldToFlux(FieldPos, FluxSize, FieldSize);
    int Xi = (int) floor(fluxPos.x); int Xf = (int) ceil(fluxPos.x);
    int Yi = (int) floor(fluxPos.y); int Yf = (int) ceil(fluxPos.y);
    float X = fluxPos.x-(float)Xi; 
    float Y = fluxPos.y-(float)Yi;
    Vector fieldM  = fluxToField(fluxPos, FluxSize, FieldSize);
    Vector fieldI  = fluxToField(Vector(Xi,Yi), FluxSize, FieldSize);
    Vector fieldF  = fluxToField(Vector(Xf,Yf), FluxSize, FieldSize);

    //logger.LogDrawRectangle(30, fieldI.x, fieldI.y, fieldF.x, fieldF.y,"Yellow");
    //logger.LogDrawCircle(30, fieldM.x, fieldM.y, 0.1, "Yellow");
    //logger.LogActionN(30,"pos(%1.1f %1.1f) p(%1.1f %1.1f) i(%1.1f %1.1f) f(%1.1f %1.1f)",
	//		fluxPos.x, fluxPos.y, X,Y, Xi,Yi, Xf,Yf);
    //logger.LogActionN(30,"pos(%1.1f %1.1f) p(%1.1f %1.1f) i(%1.1f %1.1f) f(%1.1f %1.1f)",
	//		FieldPos.x, FieldPos.y, fieldM.x,fieldM.y, fieldI.x, fieldI.y,
 	//		fieldF.x, fieldF.y);

    float Value = FL_Flux[Fl][Xi][Yi]*(1-X)*(1-Y) + FL_Flux[Fl][Xi][Yf]*(1-X)*Y + 
    		  FL_Flux[Fl][Xf][Yi]*X*(1-Y) + FL_Flux[Fl][Xf][Yf]*X*Y;   //interpolate
   
    return Value;
}

//Evaluate also to near points with error...
float DMU::evaluateFlux(Vector InitPos, Vector BallPos, Vector FinalPos)
{
    WorldState& world = SWorldState::getInstance();
    Logger& logger = SLogger::getInstance();
    Vector FluxSize = Vector(9.0, 7.0); //to change to any dimension
    Vector FieldSize = Vector(world.conditions->fieldLength, world.conditions->fieldWidth);

    float BallValue = interpolateFlux(BallPos, FluxSize, FieldSize);
    float FinalValue = interpolateFlux(FinalPos, FluxSize, FieldSize);

    Line shootLine = Line(BallPos, FinalPos);

    Line theirGoalLine = Line(world.getFrontLeftGoal().to2d(), world.getFrontRightGoal().to2d());
    Line ourGoalLine = Line(Vector(-world.getFrontLeftGoal().x,world.getFrontLeftGoal().y),
							Vector(-world.getFrontRightGoal().x,world.getFrontRightGoal().y));

    Vector intersectTheir, intersectOur;
    intersectTheir = shootLine.getIntersection(theirGoalLine);
    intersectOur = shootLine.getIntersection(ourGoalLine);

    bool shootTheirGoal = (FinalPos.x > 8.0 && //LPR 2010 NEW
    	 (FinalPos.x > intersectTheir.x && fabs(intersectTheir.y) < 0.4) ||  //really inside goal
        (fabs(BallPos.y)< 2.0 && FinalPos.x > intersectTheir.x && fabs(intersectTheir.y) < 0.7));  //really inside goal

    bool shootOurGoal = FinalPos.x < -7.0 && (FinalPos.x < intersectOur.x) && (fabs(intersectOur.y) < 2.0);    //not even close of our goal line...

    if (shootTheirGoal) FinalValue = 25.0;  //shoot!you will score!
    if (shootOurGoal) FinalValue = -10.0;    //no own goals!

    float FluxValue = 2.0*(FinalValue - BallValue);

  //  if (FluxValue>0.0)
  //  	logger.LogDrawCircle(30,FinalPos.x,FinalPos.y, FluxValue/20.0, "White");
  //  if (FluxValue<0.0)
  //  	logger.LogDrawCircle(30,FinalPos.x,FinalPos.y, FluxValue/20.0, "Gray");

    return FluxValue;
}


float calculateSafety(Vector BallPos, Vector Pos)
{
    WorldState& world = SWorldState::getInstance();
    Logger& logger = SLogger::getInstance();
    float valSafe=0.0, valSafeLine=5.0, valSafePoint=5.0;

    int clOpp = 0, clTeam = 0;
    float clDistOpp = 100.0, clDistTeam = 100.0;
    Ray shootRay= Ray(BallPos, Pos);

 	for(int i=1; i<7; i++)
	{
 		Vector teamPos = world.getFCPortugalPlayer(i)->position.to2d();
		float teamDistRay = teamPos.getDistanceTo(shootRay.GetClosestPoint(teamPos));
		if (teamPos.getDistanceTo(BallPos) > Pos.getDistanceTo(BallPos)) {
			teamDistRay = teamDistRay + teamPos.getDistanceTo(Pos);
		}

		if (i != world.unum) {
			if (teamDistRay < 0.2) valSafeLine -= 2.5;
			else if (teamDistRay < 0.4) valSafeLine -=2.0;
			else if (teamDistRay < 0.6) valSafeLine -=1.0;
		}

 		teamPos = teamPos + ((world.theirGoalPos - Pos).normalize())*0.5; //LPR 2010 HACK
		if (teamPos.getDistanceTo(Pos) < clDistTeam) {
				clDistTeam = teamPos.getDistanceTo(Pos);
				clTeam = i;
		}
		Vector oppPos = world.getOpponentPlayer(i)->position.to2d();
		float oppDistRay = oppPos.getDistanceTo(shootRay.GetClosestPoint(oppPos));
		if (oppPos.getDistanceTo(BallPos) > Pos.getDistanceTo(BallPos)) {
			oppDistRay = oppDistRay + oppPos.getDistanceTo(Pos);
		}
		if (oppDistRay < 0.2) valSafeLine -= 5.0;
		else if (oppDistRay < 0.4) valSafeLine -=3.0;
		else if (oppDistRay < 0.6) valSafeLine -=2.0;
		else if (oppDistRay < 0.8) valSafeLine -=1.0;

		oppPos = oppPos + ((world.myGoalPos - Pos).normalize())*0.5; //LPR 2010 HACK
		if (oppPos.getDistanceTo(Pos) < clDistOpp) {
				clDistOpp = oppPos.getDistanceTo(Pos);
				clOpp = i;
		}
	}

 	if (clDistTeam < clDistOpp-4.0) valSafePoint = 5.0;  //LPR 2010 HACK
 	else if (clDistTeam < clDistOpp-3.0) valSafePoint = 4.5;  //LPR 2010 HACK
	else if (clDistTeam < clDistOpp-2.0&& clDistOpp > 3.0) valSafePoint = 3.5;
	else if (clDistTeam < clDistOpp-2.0) valSafePoint = 3.0;
	else if (clDistTeam < clDistOpp-1.0 && clDistOpp > 2.0) valSafePoint = 2.0;
	else if (clDistTeam < clDistOpp-1.0) valSafePoint = 1.5;
	else if (clDistTeam < clDistOpp) valSafePoint = 1.0;
	else if (clDistTeam < clDistOpp+1.0) valSafePoint = 1.0;
	else valSafePoint = 0.0;

 	valSafe = (valSafeLine+valSafePoint)/2.0;

 	return valSafe;
}

//Safety of FinalPos compared with BallPos
float DMU::evaluateSafe(Vector InitPos, Vector BallPos, Vector FinalPos)
{
    WorldState& world = SWorldState::getInstance();
    Logger& logger = SLogger::getInstance();

    float SafetyValueBall=0.0;
    //float SafetyValueBall = calculateSafety(BallPos);
    float SafetyValueFinal = calculateSafety(BallPos, FinalPos);
    float SafetyValue = SafetyValueFinal - SafetyValueBall;

    //if (SafetyValue>0.0)
    //	logger.LogDrawCircle(30,FinalPos.x-0.2,FinalPos.y-0.2, SafetyValue/20.0, "Blue");

    return SafetyValue;
}

//Easy to get ball from Ballpos to Finalpos starting at InitPos
float DMU::evaluateEasy(Vector InitPos, Vector BallPos, Vector FinalPos)
{
    WorldState& world = SWorldState::getInstance();
    Logger& logger = SLogger::getInstance();
    float valEasy=0.0;

    Vector blTarget	= (FinalPos-BallPos).normalize();
	Vector plBall  =(BallPos-InitPos).normalize();
	AngleDeg blTargetDir = GetNormalizeAngleDeg(blTarget.getDirection());
	AngleDeg plBallDir = GetNormalizeAngleDeg(plBall.getDirection());

	AngleDeg angDiff = fabs(GetNormalizeAngleDeg(blTargetDir - plBallDir));
	float dist = BallPos.getDistanceTo(InitPos);

	if(angDiff < 5.1 && dist<0.7) valEasy=5.0;
	else if(angDiff < 10.1 && dist<0.7) valEasy=3.0;
	else if(angDiff < 15.1 && dist<0.7) valEasy=2.0;
	else if(dist<0.7) valEasy=0.0;

	else if(angDiff < 5.1 && dist<1.2) valEasy=5.0;
	else if(angDiff < 10.1 && dist<1.2) valEasy=4.0;
	else if(angDiff < 15.1 && dist<1.2) valEasy=3.0;
	else if(angDiff < 20.1 && dist<1.2) valEasy=2.0;
	else if(dist<1.2) valEasy=0.0;

	else if(angDiff < 10.1 && dist<2.0) valEasy=5.0;
	else if(angDiff < 20.1 && dist<2.0) valEasy=4.0;
	else if(angDiff < 30.1 && dist<2.0) valEasy=3.0;
	else if(angDiff < 50.1 && dist<2.0) valEasy=2.0;
	else if(angDiff < 70.1 && dist<2.0) valEasy=1.0;
	else if(dist<2.0) valEasy=0.0;

	else if(angDiff < 30.1 && dist<3.0) valEasy=5.0;
	else if(angDiff < 50.1 && dist<3.0) valEasy=4.0;
	else if(angDiff < 70.1 && dist<3.0) valEasy=3.0;
	else if(angDiff < 90.1 && dist<3.0) valEasy=2.0;
	else if(angDiff < 120.1 && dist<3.0) valEasy=1.0;
	else if(dist<3.0) valEasy=0.0;

	else if(angDiff < 30.1 && dist<3.0) valEasy=5.0;
	else if(angDiff < 50.1 && dist<3.0) valEasy=4.0;
	else if(angDiff < 70.1 && dist<3.0) valEasy=3.0;
	else if(angDiff < 90.1 && dist<3.0) valEasy=2.0;
	else if(angDiff < 120.1 && dist<3.0) valEasy=1.0;
	else if(dist<3.0) valEasy=0.0;

	else if(angDiff < 50.1) valEasy=5.0;
	else if(angDiff < 70.1) valEasy=4.0;
	else if(angDiff < 90.1) valEasy=3.0;
	else if(angDiff < 120.1) valEasy=2.0;
	else if(angDiff < 150.1) valEasy=1.0;
	else valEasy=0.0;

	if (BallPos.getDistanceTo(FinalPos)<0.6) valEasy--;  //difficult short and long kicks
	if (BallPos.getDistanceTo(FinalPos)>3.1) valEasy--;

  //  if (valEasy>0.0)
  //  	logger.LogDrawCircle(30,FinalPos.x+0.2,FinalPos.y+0.2, valEasy/20.0, "Orange");

    return valEasy; //HACK
}

