#ifndef _STRATEGY_H_
#define _STRATEGY_H_

#include "WorldState.h"
#include "Geometry.h"
#include "form/delaunay_triangulation.h"

#define NUM_MAXTACTICS 20       // Tactics
#define NUM_MAXFORM 30          // Formations (like 433Open, 422, etc)
#define NUM_MAXPLTYPES 30       // Player Types
#define NUM_MAXPLAYERS 12       // Players - 12 Players
#define NUM_MAXFLUX 30          // FluxMatrixs
#define NUM_MAXFORM_PTS 200     // Formation or Flux Points
#define NUM_MAXTIMETACTICS 10   // Time Tactics
#define NUM_MAXOPP 20		// Opponents
#define NUM_MAXSETS 10		// Max Sets of Setplays
#define NUM_MAXSETPLAYS 30	// Max Setplays in Set

#define ST_LEAGUE 2 // This is a 3D File

#define ST_2D 1   // 2D
#define ST_3D 2   // 3D
#define ST_MR 3   // MR

#define ST_VG 5   // Very Good
#define ST_GO 4   // Good
#define ST_ME 3   // Medium
#define ST_BA 2   // Bad
#define ST_VB 1   // Very Bad

#define SI_ATT 1  // Attack
#define SI_DEF 2  // Defense
#define SI_OKO 3  // Our Kick Off
#define SI_TKO 4  // Their Kick Off
#define SI_OKI 5  // Our Kick In
#define SI_TKI 6  // Their Kick In
#define SI_OFK 7  // Our Free Kick
#define SI_TFK 8  // Their Free Kick
#define SI_OGF 9  // Our Goalie Free Kick
#define SI_TGF 10 // Their Goalie Free Kick
#define SI_OPE 11 // Our Penalty
#define SI_TPE 12 // Their Penalty

// Defines team general Strategy for 2D, 3D and MR Leagues
// Note: All Arrays start at 1 (not 0)
class Strategy
{
	friend class Singleton<Strategy>;
    public:
        
	Vector WS_TeamPos[12], WS_OppPos[12], WS_BallPos; 	//Fill in these arrays before calling strategic reasoning
	int WS_Situation, WS_BallOwner;

	int ST_StrategyType;     	//Strategy Type 1-2D, 2-3D, 3-MR

	// Game Domain Parameters - used to get the environment main paramters inside strategic module
	int ST_GameTime, ST_ExtraTime;
	Vector ST_FieldSize, ST_PenaltyAreaSize;
	float ST_GoalSize, ST_AgentMaxKickDist, ST_AgentMaxSpeed;
	
        int ST_NUM_TACTICS, ST_NUM_PLAYERS, ST_NUM_FORM, ST_NUM_PLTYPES, ST_NUM_FLUX, ST_NUM_SETPLANS; 
		//Number of tactics, Players, Formations, PlTypes and Flux Matrixes and Sets of Setplays
        int ST_Opponent, ST_NUM_TIME_TACTICS, ST_NUM_OPP_TACTICS; //Opponent

	int ST_TimeInit[NUM_MAXTIMETACTICS+1];
	int ST_TimeEnd[NUM_MAXTIMETACTICS+1];
	int ST_TimeTactic[NUM_MAXTIMETACTICS+1][NUM_MAXOPP+1][6];
	
        float TA_Formation[NUM_MAXTACTICS+1];    // Formation used, by default in each tactic
        float TA_Flux[NUM_MAXTACTICS+1];	 // Flux used in each tactic
        float TA_Plans[NUM_MAXTACTICS+1];        // Set of setplays used in each tactic
        float TA_WFlux[NUM_MAXTACTICS+1];	 // WFlux used in each tactic
        float TA_WSafe[NUM_MAXTACTICS+1];        // WSafe used in each tactic
        float TA_WEasy[NUM_MAXTACTICS+1];        // WEase used in each tactic
        float TA_SitForm[NUM_MAXTACTICS+1][14];  // Formation used in each tactic, in each situation (at the moment 12 Situations)
	
	int FM_Type[NUM_MAXFORM+1];     //Formation Type 1-SBSP, 2-Delaunay
        int FM_PlayerType[NUM_MAXFORM+1][12];    
	//Formations SBSP
        Vector FM_Position[NUM_MAXFORM+1][12];
	// Formations Delaunay
	int FM_NPointsDel[NUM_MAXFORM+1];        //2D Formation Points - Number of points in formation
        Vector FM_BallPositionDel[NUM_MAXFORM+1][NUM_MAXFORM_PTS+1];  //2D FormationBall - Ball Position at point
        Vector FM_PositionDel[NUM_MAXFORM+1][NUM_MAXFORM_PTS+1][12];  //2D FormationPos - players position at point - 11 players
	
	//int FM_ChangedPos[NUM_MAXFORM+1][MAX_CHANGES+1]  //TODO ...
	DelaunayTriangulation * FM_Triangulations[NUM_MAXFORM+1];
	const DelaunayTriangulation::Triangle *FM_Triangle; //triangle that contains point RL

	// Flux Normal
	int FL_Type[NUM_MAXFLUX+1];     //Flux Type 1-old, 2-Delaunay
	int FL_Flux[NUM_MAXFLUX+1][10][12]; //7x9 for now...
	// Flux Delaunay
	int FL_NPointsDel[NUM_MAXFLUX+1];        //Number of flux points in formation
        Vector FL_BallPositionDel[NUM_MAXFLUX+1][NUM_MAXFORM_PTS+1];  //Ball Position at point
        int FL_FluxValueDel[NUM_MAXFLUX+1][NUM_MAXFORM_PTS+1];  //Flux Value at point
	DelaunayTriangulation * FL_Triangulations[NUM_MAXFORM+1];
	const DelaunayTriangulation::Triangle *FL_Triangle; //triangle that contains point RL

	// Player Types Characteristics
        Vector PT_Attraction[NUM_MAXPLTYPES+1];
        int PT_BehindBall[NUM_MAXPLTYPES+1];
        float PT_MinX[NUM_MAXPLTYPES+1];
        float PT_MaxX[NUM_MAXPLTYPES+1];
        float PT_MinY[NUM_MAXPLTYPES+1];
        float PT_MaxY[NUM_MAXPLTYPES+1];
        int PT_DecisionAlgorithm[NUM_MAXPLTYPES+1];

	//Sets of Plans (setplays) definition
	int SP_SetPlays[NUM_MAXSETS+1][NUM_MAXSETPLAYS+1];   //boolean array of setplays used in each set
        
	// The following two arrays are very Important and essencial in the team decision
	// FM_Positioning and FM_StrategicPos are used in the team decision
	int FM_Positioning[12];			 // Positionings in the formation
	Vector FM_StrategicPos[12];		 // Strategic Positions at a given time

        Strategy();
        void ReadStrategy();			// Reads the Strategy file for the appropriate league
        void DecideTactic(int time, int gm, int gs);  //Decides the tactic based on time, result an opponent
        void ReadDelaunayFormation( int n_formation, char *file );
	void ReadDelaunayFlux( int n_flux, char *file );
	Vector interpolate_2009(Vector BPos, Vector b1,Vector b2,Vector b3, Vector p1,Vector p2, Vector p3);

        // Situation Based Strategic Positioning - returns the strategic position on the field of
        // to each agent given the position of the ball and the current formation. 
        Vector SBSPPosition(int form, int t, Vector BPos, int tBall);
        // Strategic Positioning using Delaunay Triangulation
	Vector SBSPDelaunayPosition(int form, int t, Vector BPos, int tBall);  // Strategic Pos in situation Sit, for player t
								   // when ball at BPos, controlled by tBall player
	//float FluxValue(Vector pos);   		// Flux Value at pos - old Method (matrix 7x9)
	float FluxDelaunayValue(Vector pos);    // Flux value at point pos - Delaunay Triangulation
        
        // Dynamic Positioning and Role Exchange - Players may exchange their positionings and player
        // types in the current formation if the utility of that exchange is positive for the team
        void DPRExchange();        	  // Updates DPRE  (FM_Positioning[12] array)
	void SBSPCalculate();  		  // Updates SBSP (FM_StrategicPos[12] array)
	void ResetPositioning();
	int GetPositioning(int t) { return FM_Positioning[t]; }
	Vector GetStrategicPos(int t) { return FM_StrategicPos[t]; }

	Vector FormationGetBasePos(int t) { return FM_Position[FM_Now][GetPositioning(t)]; }
        int FormationGetPlayerType(int t) { return FM_PlayerType[FM_Now][GetPositioning(t)]; }
	Vector FormationGetBasePos(int form, int t) { return FM_Position[form][GetPositioning(t)]; }
        int FormationGetPlayerType(int form, int t) { return FM_PlayerType[form][GetPositioning(t)]; }

	// Returns Player Types Characteristics: Attraction, BehindBall, Decision Algorithm
	// MinX, MaxX, MinY or MaxY of player t
	Vector PTGetAttraction(int t) { return PT_Attraction[FormationGetPlayerType(t)]; }
        int PTGetBehindBall(int t) { return PT_BehindBall[FormationGetPlayerType(t)]; }
        int PTGetDecisionAlgorithm(int t) { return PT_DecisionAlgorithm[FormationGetPlayerType(t)]; }
        float PTGetMinX(int t) { return PT_MinX[FormationGetPlayerType(t)]; }
        float PTGetMaxX(int t) { return PT_MaxX[FormationGetPlayerType(t)]; }
        float PTGetMinY(int t) { return PT_MinY[FormationGetPlayerType(t)]; }
        float PTGetMaxY(int t) { return PT_MaxY[FormationGetPlayerType(t)]; }
	Vector PTGetAttraction(int form, int t) { return PT_Attraction[FormationGetPlayerType(form, t)]; }
        int PTGetBehindBall(int form, int t) { return PT_BehindBall[FormationGetPlayerType(form, t)]; }
        int PTGetDecisionAlgorithm(int form, int t) { return PT_DecisionAlgorithm[FormationGetPlayerType(form, t)]; }
        float PTGetMinX(int form, int t) { return PT_MinX[FormationGetPlayerType(form, t)]; }
        float PTGetMaxX(int form, int t) { return PT_MaxX[FormationGetPlayerType(form, t)]; }
        float PTGetMinY(int form, int t) { return PT_MinY[FormationGetPlayerType(form, t)]; }
        float PTGetMaxY(int form, int t) { return PT_MaxY[FormationGetPlayerType(form, t)]; }

	// Management of Tactics, Formations, Flux and Setplays
        void SetTactic(int tact);   // Sets Tactic in use and corrresponding formation, flux and set of plans
        void SetFormation(int form) { FM_Now = form;}		// Sets Formation in use
        void SetFlux(int flux) { FL_Now = flux;}		// Sets FluxMatrix in use
        void SetPlans(int setpl) { SP_Now = setpl;}		// Sets SetPlan number in use
	void SetWeights(float wf, float ws, float we) { 	// Sets WFlux, WSafe and WEasy in use
		WFlux_Now = wf; WSafe_Now = ws; WEasy_Now = we;	}
	int SetPlayUsed(int nspl) { return SP_SetPlays[SP_Now][nspl];} //Is it Setplay nspl used? 
        int GetTactic(void) { return TA_Now;}			// Returns Tactic in Use	
        int GetFormation(void) { return FM_Now;}		// Returns Formation in Use
        int GetFlux(void) { return FL_Now;}			// Returns Flux in Use
        int GetPlans(void) { return SP_Now;}			// Returns Plans in Use

//    private:
        int TA_Now;   // Present Tactic 
        int FM_Now;   // Present Formation
        int SI_Now;   // Present Situation
        int FL_Now;   // Present Flux Matrix
        int SP_Now;   // Present Set-SetPlays
	float WFlux_Now, WSafe_Now, WEasy_Now; // Present WFlux, WSafe and WEasy
};
typedef Singleton<Strategy> SStrategy;

#endif // STRATEGY_H
