#ifndef FCPORTUGAL_FORMATION_H
#define FCPORTUGAL_FORMATION_H

#include <sstream>
#include <map>
#include <boost/shared_ptr.hpp>

#include <rcsc/geom/vector_2d.h>
#include <soccer/iplayer_id.h>

namespace soccer
{
    class IWorldModel;

    namespace formation
    {

        typedef enum {
            FT_UNKNOWN = 0,
            FT_SBSP = 1,
            FT_Delaunay = 2
        } FormationType;


        class Formation
        {
        public:
            typedef boost::shared_ptr< Formation > Ptr;
            typedef boost::shared_ptr< const Formation > ConstPtr;
            typedef std::map<soccer::Unum, unsigned int> PlayerTypes;

            Formation(unsigned int id);
            virtual ~Formation();

            virtual FormationType type() const = 0;
            virtual bool read( std::istream & is ) = 0;

            virtual rcsc::Vector2D position(const soccer::Unum& number,
                                            const soccer::IWorldModel& wm) const = 0;

            unsigned int playerType(const soccer::Unum& number) const;
            const PlayerTypes& playerTypes() const;


            unsigned int id() const;
            bool readPlayerTypes(std::istream& is);

            virtual std::ostream& print(std::ostream& os) const;
        protected:

            std::ostream& printPlayerTypes(std::ostream& os) const;

            PlayerTypes M_player_types;
            unsigned int M_id;
        };
        typedef std::map<unsigned int, Formation*> Formations;

    }
}


#endif
