// -*-c++-*-

/*!
  \file sample_data_bowner.h
  \brief formation sample data class Header File.
*/

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifndef FCPORTUGAL_FORMATION_SAMPLE_DATA_H
#define FCPORTUGAL_FORMATION_SAMPLE_DATA_H

#include <cstdio>
#include <map>
#include <vector>
#include <rcsc/geom/line_2d.h>
#include <rcsc/formation/sample_data.h>

namespace soccer
{
    namespace formation
    {

        /*!
          \struct SampleData
          \brief training data type.
        */
        struct SampleDataBowner {
            typedef struct {
                unsigned int unum_;
                rcsc::Vector2D position_;
            } PlayerChange;
            typedef std::list<PlayerChange> PlayerChanges;

            typedef struct {
                int index_;
                std::list<PlayerChange> changes_;
            } IndexChange;
            typedef std::list<IndexChange> IndexChanges;

            unsigned int bowner_;
            IndexChanges  index_changes_; //!< map index changed to vector of player positions

            /*!
              \brief default constructor
            */
            SampleDataBowner();

            // Check if player has changed in a given index
            /*!
              \brief get the position of specified player.
              \param unum player number [1..11].
              \return position value.
            */
            // const rcsc::Vector2D* playerChanged(unsigned int index, unsigned int player_number );

            unsigned int numChanges( ) const;


            bool read(std::istream& is);
        };

    }
}

#endif
