#ifndef FCPORTUGAL_STRATEGY_FORMATION_CONFIG_H
#define FCPORTUGAL_STRATEGY_FORMATION_CONFIG_H

#include <strategy/options_config.h>

namespace soccer
{
    namespace strategy
    {
        class FormationConfig
                : public OptionsConfig<FormationConfig>
        {
            friend class Singleton<FormationConfig>;
        public:
            // Formations configurations
            bool useFormations() const;
            const std::string& formationsDir() const;
            const std::vector<std::string>& formationsFiles() const;
            int defaultFormationId() const;
            bool smoothTransitions() const;
            unsigned int smoothTransitionsDelay() const;


            void print(std::ostream& os) const;
        protected:
            FormationConfig();
            // Formations configuration
            static const bool USE_FORMATIONS;
            static const std::string FORMATIONS_DIR;
            static const int DEFAULT_FORMATION_ID;
            static const bool SMOOTH_TRANSITIONS;
            static const unsigned int SMOOTH_TRANSITIONS_DELAY;

            /// Flag stating whether formations should be used
            bool M_use_formations;
            /// Directory in which the formation files are located
            std::string M_formations_dir;
            /// Relative or absolute files paths of formation files to use
            std::vector<std::string> M_formations_files;
            /// Default formation identifier
            int M_default_formation_id;
            /// Smooth the transition between positions in 2 formations
            bool M_smooth_transitions;
            /// Time to delay the transition (update player positions) between 1 formation to another
            unsigned int M_smooth_transitions_delay;
        };

    }

}
#endif // FCPORTUGAL_STRATEGY_FORMATION_CONFIG_H
