#ifndef FCPORTUGAL_STRATEGY_OPTIONS_CONFIG_H
#define FCPORTUGAL_STRATEGY_OPTIONS_CONFIG_H

#include <string>
#include <vector>
#include <iostream>
#include <boost/program_options.hpp>

#include <strategy/singleton.h>
#include <strategy/util.h>

using namespace std;
using namespace boost;
using namespace boost::program_options;

namespace soccer
{
    namespace strategy
    {

        template <typename T>
        class OptionsConfig
                : public Singleton<T>
        {
        public:
            virtual void printHelp(std::ostream& os) const;
            virtual bool init(const std::string& config_file);

            /*!
              \brief Should the values of all options of the configuration
              */
            virtual void print(std::ostream& os) const = 0;
        protected:
            OptionsConfig(const std::string& caption);

            boost::program_options::options_description M_options_descriptions;
            boost::program_options::variables_map M_options_map;
        };

        template <typename T>
        OptionsConfig<T>::OptionsConfig(const std::string& caption)
            :M_options_descriptions(caption)
        {

        }

        template <typename T>
        void OptionsConfig<T>::printHelp(std::ostream &os) const
        {
            M_options_descriptions.print(os);
        }

        template <typename T>
        bool OptionsConfig<T>::init(const std::string& config_file)
        {
            vector<string> args;
            if ( !util::tokenizeConfigFile(config_file, args) )
            {
                return false;
            }

            // Parse the file and store the options
            parsed_options parsed = command_line_parser(args).options(M_options_descriptions)
                                                                  .allow_unregistered().run();
            store(parsed, M_options_map);
            notify(M_options_map);

            // TODO Check if all necessary options were parsed
        #ifdef DEBUG
        // OUTPUT ALL THE PARSED STRATEGY OPTIONS
            cout << __FILE__ "[FormationConfig]" << endl;
            for ( variables_map::iterator it = M_options_map.begin();
                  it != M_options_map.end();
                 it++)
            {
                cout << it->first << endl;
            }

        #endif
            // Output unrecognized options
            vector<string> unrecognized_options = collect_unrecognized(parsed.options, include_positional);
            if ( !unrecognized_options.empty() )
            {
        #ifdef DEBUG
                cout << "The following options are unrecognized by FormationConfig" << endl;
                for (vector<string>::const_iterator it = unrecognized_options.begin();
                     it != unrecognized_options.end();
                     it++)
                {
                    cout << *it << endl;
                }
        #endif
            }
            // TODO Should return if a minimum number of mandatory options were specified!
            // TODO Should also do a sanity check (e.g. files and dirs => check if they exist and are accessible)
            return true;
        }

    }
}

#endif // FCPORTUGAL_STRATEGY_OPTIONS_CONFIG_H
