#ifndef FCPORTUGAL_PLAYER_TYPE_H
#define FCPORTUGAL_PLAYER_TYPE_H

#include <map>
#include <rcsc/geom/vector_2d.h>

namespace soccer {
namespace strategy {

    class PlayerType
    {
    public:
        // Decision Algorithm 1-Goa, 2-Def, 3-Att
        typedef enum {
            DA_Goalie = 1,
            DA_Defense = 2,
            DA_Attack = 3
        } DecisionAlgorithm;

        typedef enum {
            PT_None,
            PT_Goaltender = 1,
            PT_Sweeper = 2,
            PT_Defender = 3,
            PT_Midfielder = 4,
            PT_Forward = 5
        } PlayerRole;

        PlayerType(PlayerRole player_role,
                   double attraction_x, double attraction_y,
                   double area_attr_attack, double area_attr_defense,
                   bool behind_ball,
                   double min_pos_x, double max_pos_x,
                   double min_pos_y, double max_pos_y,
                   DecisionAlgorithm decision_algorithm);

        // Acessors
        PlayerRole role() const;
        double attractionX() const;
        double attractionY() const;
        double areaAttrAttack() const;
        double areaAttrDefense() const;
        bool behindBall() const;
        double minPosX() const;
        double maxPosX() const;
        double minPosY() const;
        double maxPosY() const;
        DecisionAlgorithm decisionAlgorithm() const;

        // Verifiers
        bool validPos(const rcsc::Vector2D& point) const;
        rcsc::Vector2D truncatePos(const rcsc::Vector2D& point ) const;

        std::ostream& print(std::ostream& os) const;
    private:
        PlayerRole M_role;
        double M_attraction_x; //  (SP)
        double M_attraction_y; //  (SP)
        double M_area_attr_attack; //  AreaAttrAttack (last 1/4) not used
        double M_area_attr_defense; //  AreaAttrAttack (last 1/4) not used
        bool M_behind_ball; // BehindBall (SP and DT)
        double M_min_pos_x; // MinX (SP and DT)
        double M_max_pos_x; // MaxX (SP and DT)
        double M_min_pos_y; // MinX (SP and DT)
        double M_max_pos_y; // MaxX (SP and DT)
        DecisionAlgorithm M_decision_algorithm;
    };

    typedef std::map<unsigned int, PlayerType*> PlayerTypes;
}
}

std::ostream& operator<<(std::ostream& os, const soccer::strategy::PlayerType& player_type);

#endif // FCPORTUGAL_PLAYER_TYPE_H
