#include "strategy_config.h"

using namespace boost::program_options;
using namespace std;

namespace soccer
{
    namespace strategy
    {

        const bool StrategyConfig::USE_DPRE = false;
        const bool StrategyConfig::USE_SBSP = false;
        const std::string StrategyConfig::LEAGUE_CONFIG_FILE = "";
        const std::string StrategyConfig::SETPLAYS_CONFIG_FILE = "";
        const std::string StrategyConfig::FORMATIONS_CONFIG_FILE = "";
        const std::string StrategyConfig::TACTICS_CONFIG_FILE = "";
        const std::string StrategyConfig::FLUXES_CONFIG_FILE = "";
        const std::string StrategyConfig::PLAYER_TYPES_CONFIG_FILE = "";

        StrategyConfig::StrategyConfig()
         : OptionsConfig<StrategyConfig>("Strategy options")
        {
            // Initialize options options
            M_options_descriptions.add_options()
                ("help", "produce help message")
                ("use_dpre",
                 value<bool>(&M_use_dpre)->default_value(USE_DPRE),
                 "Enable the use of Dynamic Positioning and Role Exchange")
                ("use_sbsp",
                 value<bool>(&M_use_sbsp)->default_value(USE_SBSP),
                 "Enable the use of Situation Based Strategic Positioning")
                ("league_config_file",
                 value<std::string>(&M_league_config_file)->default_value(LEAGUE_CONFIG_FILE),
                 "League configuration file")
                ("fluxes_config_file",
                 value<std::string>(&M_fluxes_config_file)->default_value(FLUXES_CONFIG_FILE),
                  "Flux global configuration file")
                ("formations_config_file",
                 value<std::string>(&M_formations_config_file)->default_value(FORMATIONS_CONFIG_FILE),
                 "Formations global configuration file")
                ("tactics_config_file",
                 value<std::string>(&M_tactics_config_file)->default_value(TACTICS_CONFIG_FILE),
                 "Tactics global configuration file")
                ("player_types_config_file",
                 value<std::string>(&M_player_types_config_file)->default_value(PLAYER_TYPES_CONFIG_FILE),
                 "Player types global configuration file")
                ("setplays_config_file",
                 value<std::string>(&M_setplays_config_file)->default_value(SETPLAYS_CONFIG_FILE),
                 "Setplays global configuration file")
            ;
        }

        bool StrategyConfig::useDPRE() const
        {
            return M_use_dpre;
        }

        bool StrategyConfig::useSBSP() const
        {
            return M_use_sbsp;
        }

        const std::string& StrategyConfig::leagueConfigFile()
        {
            return M_league_config_file;
        }

        const std::string& StrategyConfig::setplaysConfigFile()
        {
            return M_setplays_config_file;
        }

        const std::string& StrategyConfig::tacticsConfigFile()
        {
            return M_tactics_config_file;
        }

        const std::string& StrategyConfig::formationsConfigFile()
        {
            return M_formations_config_file;
        }

        const std::string& StrategyConfig::fluxesConfigFile()
        {
            return M_fluxes_config_file;
        }

        const std::string& StrategyConfig::playerTypesConfigFile()
        {
            return M_player_types_config_file;
        }

        void StrategyConfig::print(std::ostream& os) const
        {
            os << "[Strategy Config]" << endl;
            os << "use_dpre: " << M_use_dpre << endl;
            os << "use_sbsp: " << M_use_sbsp << endl;
            os << "league_config_file: " << M_league_config_file << endl;
            os << "setplays_config_file: " << M_setplays_config_file << endl;
            os << "tactics_config_file: " << M_tactics_config_file << endl;
            os << "formations_config_file: " << M_formations_config_file << endl;
            os << "fluxes_config_file: " << M_fluxes_config_file << endl;
            os << "player_types_config_file: " << M_player_types_config_file << endl;
            os << endl;
        }
    }
}
