#include "tactic_with_situations.h"

#include <sstream>
#include <strategy/util.h>

using namespace std;
using namespace soccer::strategy;
using namespace soccer;

namespace soccer
{

    namespace tactic
    {
        TacticWithSituations::TacticWithSituations()
            : Tactic()
        {

        }


        bool TacticWithSituations::read(std::istream& is)
        {
            if ( !Tactic::read(is) )
            {
                return false;
            }

            std::istringstream istr(util::getNextLine(is));

            // Read the number of formations
            unsigned int game_mode = 0;
            // istr >> num_game_mode_formations;
            // 3rd line - Formations used in each game situation
            // TODO For each game mode read the formatio
            while ( game_mode <14 )
            {
                // HACK => 14 fixed situations for now...
                unsigned int formation = -1;
                istr >> formation;

                if ( game_mode == 0) { M_default_formation = formation; }
                else if ( game_mode == 1) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::PlayOn ), formation)); }
                else if ( game_mode == 2)
                {
                    M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::KickOff, TA_OUR ), formation ));
                    M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::Goal, TA_OPP), formation ));
                }
                else if ( game_mode == 3)
                {
                    M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::KickOff, TA_OPP ), formation));
                    M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::Goal, TA_OUR ), formation ));
                }
                else if ( game_mode == 4) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::CornerKick, TA_OUR ), formation)); }
                else if ( game_mode == 5) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::CornerKick, TA_OPP ), formation)); }
                else if ( game_mode == 6) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::FreeKick, TA_OUR ), formation)); }
                else if ( game_mode == 7) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::FreeKick, TA_OPP ), formation)); }
                else if ( game_mode == 8) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::GoalieCatch, TA_OUR ), formation)); }
                else if ( game_mode == 9) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::GoalieCatch, TA_OPP ), formation)); }
                else if ( game_mode == 10) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::GoalKick, TA_OUR ), formation)); }
                else if ( game_mode == 11) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::GoalKick, TA_OPP ), formation)); }
                else if ( game_mode == 12) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::IndirectFreeKick, TA_OUR ), formation)); }
                else if ( game_mode == 13) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::IndirectFreeKick, TA_OPP ), formation)); }
                else if ( game_mode == 14) { M_formations.insert( std::pair<GameSituation, unsigned int>( GameSituation( GameSituation::PenaltyKick, TA_OUR ), formation)); }
                else { std::cerr<< "More game situations than expected!" << std::endl; assert( false); }

                // DANGER If int values don't correspond to enum
                game_mode++;
            }
            return true;
        }


        unsigned int TacticWithSituations::formation(GameSituation game_situation) const
        {
            GameSituationFormations::const_iterator itf = M_formations.find(game_situation);
            if ( itf != M_formations.end() )
            {
                return itf->second;
            }
            return M_default_formation;
        }

        unsigned int TacticWithSituations::numFormations() const
        {
            return M_formations.size();
        }

        const TacticWithSituations::GameSituationFormations& TacticWithSituations::gameSituationFormations() const
        {
            return M_formations;
        }

        bool TacticWithSituations::usesGameSituations() const
        {
            return true;
        }

        Tactic::Type TacticWithSituations::type() const
        {
            return Tactic::TacticWithSituations;
        }

    }
}
