#ifndef FCPORTUGAL_TIME_TACTIC_H
#define FCPORTUGAL_TIME_TACTIC_H

#include <vector>
#include <map>
#include <utility>
#include <boost/shared_ptr.hpp>

#include <soccer/types.h>
#include <strategy/tactic/tactic_selector.h>

namespace soccer
{
    class IWorldModel;

    namespace tactic
    {

        class ScoreAssessmentTacticSelector
            : public ITacticSelector
        {
        public:
            ScoreAssessmentTacticSelector( );

            void setDefaultTactic( unsigned int tactic_id);
            void addTactic( ScoreAssessment score, unsigned int tactic_id);
            unsigned int getTactic( const IWorldModel* world_model ) const;
        private:
            typedef std::map<ScoreAssessment, unsigned int> ScoreAssessmentTactics;
            ScoreAssessmentTactics M_tactics;
            unsigned int M_default_tactic;
        };

        class TimeTacticSelector
                : public ITacticSelector
        {
        public:
            TimeTacticSelector( );

            /// ITacticSelector interface implementation
            unsigned int getTactic( const IWorldModel* world_model ) const;
            void addTactic( ScoreAssessment score_assessment,
                            unsigned int start_time,
                            unsigned int end_time,
                            unsigned int tactic_id);
            void addTactic( unsigned int opponent,
                            ScoreAssessment score_assessment,
                            unsigned int start_time,
                            unsigned int end_time,
                            unsigned int tactic_id);

            bool read(std::istream& is,
                      unsigned int num_opponent_tactics );
        private:
            static unsigned int S_any_opponent;

            struct TimeTactic
            {
                unsigned int tactic_id;
                unsigned int start_time;
                unsigned int end_time;
            };

            typedef std::vector<TimeTactic> TimeTactics;
            typedef std::map<ScoreAssessment, TimeTactics> ScoreAssessmentTactics;
            typedef std::map<unsigned int, ScoreAssessmentTactics> OpponentScoreAssessmentTactics;


            /// Regardless of opponent
            ScoreAssessmentTactics M_default_tactics;
            /// Per opponent tactics
            OpponentScoreAssessmentTactics M_opponent_tactics;
        };
    }
}

#endif // FCPORTUGAL_TIME_TACTIC_H
