#include "tactic_config.h"

using namespace boost::program_options;
using namespace std;

namespace soccer
{
    namespace strategy
    {

        const bool TacticConfig::USE_TACTICS = true;
        const std::string TacticConfig::TACTICS_DIR = "tactics";
        const int TacticConfig::DEFAULT_TACTIC_ID = -1;

        TacticConfig::TacticConfig()
            : OptionsConfig<TacticConfig>("Tactics Config")
        {
            M_options_descriptions.add_options()
                ("help", "produce help message")
                ("use_tactics", value<bool>(&M_use_tactics)->default_value(USE_TACTICS),
                 "Enable the use of tactics")
                ("tactics_dir", value<std::string>(&M_tactics_dir)->default_value(TACTICS_DIR),
                 "Directory that contains the definitions of the tactic to use")
                ("tactic_file",
                 value< std::vector<std::string> >(&M_tactics_files)->composing(),
                 "Tactic file (one or more names or paths of tactic files to use")
                ("default_tactic_id",
                 value< int >(&M_default_tactic_id)->default_value(DEFAULT_TACTIC_ID),
                 "Identifier of the default tactic to be used as a last resort")
            ;
        }

        bool TacticConfig::useTactics() const
        {
            return M_use_tactics;
        }

        const std::string& TacticConfig::tacticsDir() const
        {
            return M_tactics_dir;
        }

        const std::vector<std::string>& TacticConfig::tacticsFiles() const
        {
            return M_tactics_files;
        }

        int TacticConfig::defaultTacticId() const
        {
            return M_default_tactic_id;
        }

        void TacticConfig::print(std::ostream &os) const
        {
            os << "[Tactics Config]" << endl;
            os << "use_tactics: " << M_use_tactics << endl;
            os << "tactics_dir: " << M_tactics_dir << endl;
            os << "default_tactic_id: " << M_default_tactic_id << endl;
            os << "tactics_file: ";
            for (vector<string>::const_iterator it = M_tactics_files.begin(); it != M_tactics_files.end(); it++)
            {
                os << *it << " ";
            }
            os << endl << endl;
        }
    }
}
