#include "util.h"

#include <fstream>
#include <sstream>
#include <boost/tokenizer.hpp>
#include <boost/token_functions.hpp>

using namespace boost;
using namespace std;

namespace soccer {
namespace strategy {
namespace util {

    std::string getNextLine(std::istream& is)
    {

        std::string line_buf;
        while (std::getline( is, line_buf )) {
            if ( line_buf.empty()
                         || line_buf[0] == '#'
                         || ! line_buf.compare( 0, 2, "//" ) )
            {
                // TODO Does this ignore empty spaces?
                continue; // Ignore comments and empty lines
            }
            break;
        }
        return line_buf;
    }


    rcsc::Vector2D interpolate(const rcsc::Vector2D& ball_position,
                               const rcsc::Vector2D& vertex0_ball_position,
                               const rcsc::Vector2D& vertex1_ball_position,
                               const rcsc::Vector2D& vertex2_ball_position,
                               const rcsc::Vector2D& vertex0_triangle_position,
                               const rcsc::Vector2D& vertex1_triangle_position,
                               const rcsc::Vector2D& vertex3_triangle_position)
    {
        rcsc::Line2D line1(vertex0_ball_position,ball_position);
        rcsc::Line2D line2(vertex1_ball_position,vertex2_ball_position);
        rcsc::Vector2D intersection_point = line1.intersection(line2);

        double m1 = intersection_point.dist(vertex1_ball_position);
        double n1 = intersection_point.dist(vertex2_ball_position);
        rcsc::Vector2D posI = vertex1_triangle_position +
                              (vertex3_triangle_position-vertex1_triangle_position)*m1/(m1+n1);

        double m2 = ball_position.dist(vertex0_ball_position);
        double n2 = ball_position.dist(intersection_point);
        return vertex0_triangle_position + (posI-vertex0_triangle_position)*m2/(m2+n2);
    }

    bool tokenizeConfigFile(const std::string& config_file,
                            std::vector<string>& args)
    {
        // Load the file and tokenize it
        ifstream ifs(config_file.c_str());
        if (!ifs)
        {
            cerr << "Could not open the configuration file "
                 << config_file << endl;
            return false;
        }
        // Read the whole file into a string
        stringstream ss;
        ss << ifs.rdbuf();
        // Split the file content
        char_separator<char> sep(" \n\r");
        string sstr = ss.str();
        tokenizer<char_separator<char> > tok(sstr, sep);
        copy(tok.begin(), tok.end(), back_inserter(args));

        return true;
    }

}
}
}
